using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;

namespace CarMod
{
    public class CarMod
    {
        public static SpawnableAsset FindTank()
        {
            return Resources.LoadAll<SpawnableAsset>("SpawnableAssets").First((spawnable) => spawnable.Prefab.transform.Find("Head") != null && spawnable.name == "Tank");
        }
        public static void Main()
        {
#region Heavy Vehicles
        #region Chiller Truck
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Chiller Truck",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "A small truck. Carry everything you want in it! Press the red button (F) at the back of the truck to change the texture. You can also click on the cabin and horn signal will play.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/NSR Truck.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/NSR Truck/Wheel.png");
                        AudioClip Horn = ModAPI.LoadSound("Sfx/Truck Horn.mp3");

                        //Car speed
                        float MotorSpeed = -2500f;

                        //Frame
                        Transform Frame = Instance.transform.GetChild(0);
                        Frame.gameObject.name = "Frame";
                        Frame.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Frame.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        PhysicalBehaviour FramePhysical = Frame.GetComponent<PhysicalBehaviour>();
                        FramePhysical.TrueInitialMass = 30f;
                        FramePhysical.InitialMass = 30f;
                        FramePhysical.rigidbody.mass = 30f;
                        foreach (BoxCollider2D FrameBoxCollider in Frame.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(FrameBoxCollider);
                        }
                        PolygonCollider2D FrameCollider = Frame.GetComponent<PolygonCollider2D>();
                        FrameCollider.pathCount = 1;
                        FrameCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(1.94310784f, -0.780868173f),
                            new Vector2(-4.34571075f, -0.787746429f),
                            new Vector2(-4.34259987f, -0.588298619f),
                            new Vector2(-4.4570899f, -0.585916162f),
                            new Vector2(-4.45713997f, -0.928539395f),
                            new Vector2(-4.30201387f, -0.928384066f),
                            new Vector2(-4.24242878f, -1.23972917f),
                            new Vector2(-3.49087548f, -1.38666177f),
                            new Vector2(-2.00902629f, -1.40035677f),
                            new Vector2(-1.43811643f, -1.34920585f),
                            new Vector2(-0.475689411f, -1.35996211f),
                            new Vector2(0.517740309f, -1.40930104f),
                            new Vector2(2.22149467f, -1.3153317f),
                            new Vector2(2.23236895f, -0.851729751f),
                            new Vector2(2.05511069f, -0.849445224f),
                            new Vector2(2.0552516f, 1.93162251f),
                            new Vector2(-4.45638275f, 1.92397475f),
                            new Vector2(-4.45960808f, 1.7525394f),
                            new Vector2(-4.33928204f, 1.75643778f),
                            new Vector2(-4.34375858f, 1.8125f),
                            new Vector2(1.94651484f, 1.81470776f)
                        });
                        BoxCollider2D FrameTrigger = Frame.gameObject.AddComponent<BoxCollider2D>();
                        FrameTrigger.offset = new Vector2(-1.252396f, 0.5173991f);
                        FrameTrigger.size = new Vector2(6.409492f, 2.570898f);
                        FrameTrigger.isTrigger = true;
                        Frame.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/NSR Truck/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/NSR Truck/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/NSR Truck/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/NSR Truck/Cabin.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/NSR Truck/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/NSR Truck/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/NSR Truck/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/NSR Truck/Cabin.png")
                            }
                        };
                        Frame.GetChild(0).localPosition = new Vector3(0f, 0f); //Frame bottom position
                        Frame.GetChild(0).gameObject.AddComponent<Optout>();
                        Frame.gameObject.AddComponent<Optout>();

                        //Wheels
                        WheelJoint2D[] Wheels = Frame.GetComponents<WheelJoint2D>();
                        Wheels[0].anchor = new Vector2(3.0142f, -1.4865f);
                        Wheels[0].connectedBody.gameObject.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.gameObject.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        Wheels[0].connectedBody.gameObject.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[0].connectedBody.gameObject.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        Wheels[1].anchor = new Vector2(-2.8996f, -1.4865f);
                        Wheels[1].connectedBody.gameObject.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.gameObject.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        Wheels[1].connectedBody.gameObject.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[1].connectedBody.gameObject.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        //Cabine
                        Transform Cabine = Instance.transform.GetChild(3);
                        Cabine.localPosition = new Vector3(3.34286f, -0.1421f);
                        Cabine.gameObject.name = "Cabine";
                        Cabine.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Cabine.GetComponent<SpriteRenderer>().sortingOrder = 1;
                        PhysicalBehaviour CabinePhysical = Cabine.GetComponent<PhysicalBehaviour>();
                        CabinePhysical.TrueInitialMass = 10f;
                        CabinePhysical.InitialMass = 10f;
                        CabinePhysical.rigidbody.mass = 10f;
                        FixedJoint2D CabineFixedJoint = Cabine.GetComponent<FixedJoint2D>();
                        CabineFixedJoint.breakForce = 10000f;
                        CabineFixedJoint.breakTorque = 5000f;
                        HingeJoint2D CabineHingeJoint = Cabine.GetComponent<HingeJoint2D>();
                        CabineHingeJoint.anchor = new Vector2(0.684f, -1.007f);
                        CabineHingeJoint.breakForce = 55000f;
                        CabineHingeJoint.breakTorque = 50000f;
                        Cabine.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D()
                        {
                            min = 0,
                            max = 70
                        };
                        AudioSource HornSource = Cabine.gameObject.AddComponent<AudioSource>();
                        HornSource.spatialBlend = 1f;
                        HornSource.minDistance = 100f;
                        Cabine.gameObject.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            HornSource.PlayOneShot(Horn);
                        };
                        PolygonCollider2D CabineCollider = Cabine.GetComponent<PolygonCollider2D>();
                        CabineCollider.pathCount = 3;
                        CabineCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(-1.11428571f, 1.38571429f),
                            new Vector2(-1.11236191f, -1.27085507f),
                            new Vector2(-0.890135646f, -1.26565695f),
                            new Vector2(-0.76744628f, -0.960015655f),
                            new Vector2(-0.508374691f, -0.7888937f),
                            new Vector2(-0.114546299f, -0.793051422f),
                            new Vector2(0.149471045f, -1.01612675f),
                            new Vector2(0.234630063f, -1.27028143f),
                            new Vector2(0.880298376f, -1.27227914f),
                            new Vector2(0.881875277f, -1.38399053f),
                            new Vector2(1.06211543f, -1.38571429f),
                            new Vector2(1.05519366f, -0.949432731f),
                            new Vector2(1.00119042f, -0.955974519f),
                            new Vector2(1.00331998f, -0.474181652f),
                            new Vector2(0.945303679f, 0.3893345f),
                            new Vector2(0.34003675f, 1.387519f)
                        });
                        CabineCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(-0.139613867f, -0.616731524f),
                            new Vector2(0.20703125f, -0.854346931f),
                            new Vector2(0.370859623f, -1.0978142f),
                            new Vector2(0.780607879f, -1.10049677f),
                            new Vector2(0.771600485f, 0.369089842f),
                            new Vector2(0.278885841f, 1.21838367f),
                            new Vector2(-0.941098452f, 1.21078706f),
                            new Vector2(-0.953560233f, -0.615295827f)
                        });
                        CabineCollider.SetPath(2, new Vector2[]
                        {
                            new Vector2(-0.460375547f, 0.787469387f),
                            new Vector2(-0.499092817f, 0.985782266f),
                            new Vector2(-0.656153202f, 0.975601912f),
                            new Vector2(-0.689359426f, 0.536615252f),
                            new Vector2(-0.605263054f, -0.462465316f),
                            new Vector2(-0.492100239f, -0.469435483f),
                            new Vector2(-0.482424498f, -0.625815153f),
                            new Vector2(-0.035804987f, -0.680332422f),
                            new Vector2(-0.0310842991f, -0.477799356f),
                            new Vector2(0.0958461761f, -0.398417085f),
                            new Vector2(0.114688158f, -0.310526639f),
                            new Vector2(0.0358977318f, -0.242200285f),
                            new Vector2(-0.373823345f, -0.245897442f),
                            new Vector2(-0.513394475f, 0.586770773f)
                        });
                        Cabine.gameObject.AddComponent<Optout>();

                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot
                        Instance.transform.GetChild(6).gameObject.SetActive(false); //Back door

                        //Front door
                        Transform Door = Instance.transform.GetChild(5);
                        Door.localPosition = new Vector3(3.25715f, -0.08572f);
                        Door.gameObject.name = "Door";
                        Door.GetComponent<FixedJoint2D>().connectedBody = Cabine.GetComponent<Rigidbody2D>();
                        Door.GetComponent<HingeJoint2D>().connectedBody = Cabine.GetComponent<Rigidbody2D>();
                        Door.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.942857146f,1.2428571f),
                            new Vector2(-0.942857146f,-0.753494799f),
                            new Vector2(-0.0506865978f,-0.758813918f),
                            new Vector2(0.243927717f,-0.944491565f),
                            new Vector2(0.403524786f,-1.2477659f),
                            new Vector2(0.942857146f,-1.2428571f),
                            new Vector2(0.942857146f,0.325659156f),
                            new Vector2(0.393956035f,1.23979867f)
                        });
                        Door.gameObject.AddComponent<Optout>();

                        //Cabine bottom
                        GameObject CabineBackground = new GameObject("Background");
                        CabineBackground.transform.parent = Cabine;
                        CabineBackground.transform.localRotation = Quaternion.identity;
                        CabineBackground.transform.localScale = Vector3.one;
                        CabineBackground.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer CabineBackgroundRenderer = CabineBackground.AddComponent<SpriteRenderer>();
                        CabineBackgroundRenderer.sprite = ModAPI.LoadSprite("Textures/NSR Truck/CabinBackground.png");
                        CabineBackgroundRenderer.sortingOrder = -1;
                        CabineBackgroundRenderer.sortingLayerName = "Background";
                        CabineBackground.AddComponent<Optout>();

                        //Container top
                        GameObject FrameForeground = new GameObject("Foreground");
                        FrameForeground.transform.parent = Frame;
                        FrameForeground.transform.localRotation = Quaternion.identity;
                        FrameForeground.transform.localScale = Vector3.one;
                        FrameForeground.transform.localPosition = new Vector3(0f, 0f);
                        FrameForeground.gameObject.SetActive(true);
                        SpriteRenderer FrameForegroundRenderer = FrameForeground.AddComponent<SpriteRenderer>();                  
                        //FrameForegroundRenderer.sprite = ModAPI.LoadSprite("Textures/NSR Truck/Banners/Container.png");
                        FrameForegroundRenderer.sortingOrder = 9;
                        FrameForegroundRenderer.sortingLayerName = "Foreground";
                        FrameForeground.AddComponent<RandomCarTextureChildBehaviour>().Sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/NSR Truck/Banners/Container.png"),
                            ModAPI.LoadSprite("Textures/NSR Truck/Banners/ContainerTeaserBanner.png"),
                        };
                        FrameForeground.AddComponent<Optout>();

                        //Inside settings
                        Frame.gameObject.AddComponent<InsideBehaviour>().ForegroundRenderers = new SpriteRenderer[]
                        {
                            FrameForegroundRenderer
                        };

                        //Mirror
                        GameObject Mirror = new GameObject("Mirror");
                        Mirror.transform.parent = Instance.transform;
                        Mirror.transform.localScale = Vector3.one;
                        Mirror.transform.localPosition = new Vector3(4.3852f, 0.3145f);
                        SpriteRenderer MirrorRenderer = Mirror.AddComponent<SpriteRenderer>();
                        MirrorRenderer.sprite = ModAPI.LoadSprite("Textures/NSR Truck/Mirror.png");
                        MirrorRenderer.sortingLayerName = "Foreground";
                        MirrorRenderer.sortingOrder = -9;
                        Mirror.layer = LayerMask.NameToLayer("Objects");
                        Mirror.AddComponent<PolygonCollider2D>();
                        Mirror.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D MirrorFixedJoint = Mirror.AddComponent<FixedJoint2D>();
                        MirrorFixedJoint.connectedBody = Cabine.gameObject.GetComponent<Rigidbody2D>();
                        MirrorFixedJoint.frequency = 8f;
                        MirrorFixedJoint.dampingRatio = 1f;
                        MirrorFixedJoint.breakForce = 500f;
                        MirrorFixedJoint.anchor = new Vector2(-0.0646f, -0.1862f);
                        Mirror.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");
                        Mirror.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Mirror.AddComponent<Optout>();

                        //Sign
                        GameObject Sign = new GameObject("Sign");
                        Sign.transform.parent = Instance.transform;
                        Sign.transform.localScale = Vector3.one;
                        Sign.transform.localPosition = new Vector3(-4.397f, -1.186f);
                        SpriteRenderer SignPartRenderer = Sign.AddComponent<SpriteRenderer>();
                        SignPartRenderer.sprite = ModAPI.LoadSprite("Textures/NSR Truck/BackStuff.png");
                        SignPartRenderer.sortingLayerName = "Foreground";
                        SignPartRenderer.sortingOrder = -9;
                        Sign.layer = LayerMask.NameToLayer("Objects");
                        Sign.AddComponent<PolygonCollider2D>();
                        Sign.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D SignFixedJoint = Sign.AddComponent<FixedJoint2D>();
                        SignFixedJoint.connectedBody = Frame.GetComponent<Rigidbody2D>();
                        SignFixedJoint.frequency = 20f;
                        SignFixedJoint.dampingRatio = 5f;
                        SignFixedJoint.breakForce = 7000f;
                        SignFixedJoint.anchor = new Vector2(0.0262f, 0.2559f);
                        Sign.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");
                        Sign.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Sign.AddComponent<Optout>();

                        //Fence
                        GameObject Fence = new GameObject("Fence");
                        Fence.transform.parent = Instance.transform;
                        Fence.transform.localScale = Vector3.one;
                        Fence.transform.localPosition = new Vector3(-0.9301103f, -1.25f);
                        SpriteRenderer FenceRenderer = Fence.AddComponent<SpriteRenderer>();
                        FenceRenderer.sprite = ModAPI.LoadSprite("Textures/NSR Truck/Fence.png");
                        FenceRenderer.sortingLayerName = "Foreground";
                        FenceRenderer.sortingOrder = -9;
                        Fence.layer = LayerMask.NameToLayer("Objects");
                        Fence.AddComponent<PolygonCollider2D>();
                        Fence.AddComponent<Rigidbody2D>().mass = 3f;
                        FixedJoint2D FenceFixedJoint = Fence.AddComponent<FixedJoint2D>();
                        FenceFixedJoint.connectedBody = Frame.GetComponent<Rigidbody2D>();
                        FenceFixedJoint.breakForce = 3500f;
                        Fence.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");
                        Fence.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Fence.AddComponent<Optout>();

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/NSR Truck/Gas Tank.png");
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        GasTank.gameObject.AddComponent<BoxCollider2D>();

                        //Not colliding objects with wheels in a certain area
                        NoCollideWithObjectsInTrigger noCollideWithObjectsInTrigger = FrameTrigger.gameObject.AddComponent<NoCollideWithObjectsInTrigger>();
                        noCollideWithObjectsInTrigger.Trigger = FrameTrigger;
                        noCollideWithObjectsInTrigger.Colliders = new Collider2D[]
                        {
                            Wheels[0].connectedBody.GetComponent<CircleCollider2D>(),
                            Wheels[1].connectedBody.GetComponent<CircleCollider2D>()
                        };

                        //Serialization
                        CarSerialisationHelper carSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        carSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Frame.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.gameObject.transform.localPosition = new Vector2(3.0142f, -1.4865f);
                            Wheels[1].connectedBody.gameObject.transform.localPosition = new Vector2(-2.8996f, -1.4865f);
                            GasTank.localPosition = new Vector3(1.331f, -1.304f);
                            GasTank.localScale = new Vector3(0.9f, 0.9f);

                            //Engine
                            Transform Engine = Instance.transform.GetChild(7);
                            Engine.localPosition = new Vector3(2.924f, -0.967f);
                            Engine.localScale = new Vector3(0.9f, 0.9f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Frame.gameObject;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Cabine.gameObject;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Door.gameObject;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Mirror;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Sign;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Fence;
                        };

                        //Not colliding car parts with each other
                        NoCollide noCollide = Instance.GetComponent<NoCollide>();
                        noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Farm Truck
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Farm Truck",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Vintage truck. Carry whatever you want in the back. And please close the hatch.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Farm Truck.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/Farm Truck/Wheel.png");

                        //Car speed
                        float MotorSpeed = -2500f;

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 15f;
                        BodyPhysical.InitialMass = 15f;
                        BodyPhysical.rigidbody.mass = 15f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(0.969877958f, -0.81336087f),
                            new Vector2(0.970062256f, -0.928290844f),
                            new Vector2(3.68529797f, -0.92664063f),
                            new Vector2(3.64696407f, -0.028729856f),
                            new Vector2(3.51680231f, 0.27755481f),
                            new Vector2(3.34364033f, 0.0376042426f),
                            new Vector2(2.39259791f, 0.170039326f),
                            new Vector2(2.39135075f, 0.489615977f),
                            new Vector2(2.31592655f, 0.492912531f),
                            new Vector2(1.90802622f, 1.15513027f),
                            new Vector2(0.941876471f, 1.1565963f),
                            new Vector2(0.94317627f, 1.41555917f),
                            new Vector2(0.853999138f, 1.41628504f),
                            new Vector2(0.856516004f, -0.0725044012f),
                            new Vector2(-3.62857151f, -0.0714285746f),
                            new Vector2(-3.62443447f, -0.531727552f),
                            new Vector2(-3.57383204f, -0.532240748f),
                            new Vector2(-3.56462479f, -0.815891743f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(1.26015186f, -0.39981544f),
                            new Vector2(1.1902771f, 0.798715711f),
                            new Vector2(1.06094837f, 0.845597744f),
                            new Vector2(1.06146204f, 1.0625875f),
                            new Vector2(1.84353733f, 1.0658375f),
                            new Vector2(2.25436735f, 0.457060248f),
                            new Vector2(2.25681782f, -0.848223984f),
                            new Vector2(1.57083321f, -0.839994431f),
                            new Vector2(1.577981f, -0.627450585f),
                            new Vector2(1.73081017f, -0.551814973f),
                            new Vector2(1.68850958f, -0.389144957f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Farm Truck/Red/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Farm Truck/Red/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Farm Truck/Red/Bonnet.png"),
                                Background = ModAPI.LoadSprite("Textures/Farm Truck/Red/Background.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Farm Truck/Green/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Farm Truck/Green/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Farm Truck/Green/Bonnet.png"),
                                Background = ModAPI.LoadSprite("Textures/Farm Truck/Green/Background.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Farm Truck/Gray/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Farm Truck/Gray/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Farm Truck/Gray/Bonnet.png"),
                                Background = ModAPI.LoadSprite("Textures/Farm Truck/Gray/Background.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero; //Body bottom position

                        //Wheels
                        WheelJoint2D[] wheels = Body.GetComponents<WheelJoint2D>();
                        wheels[0].connectedBody.gameObject.transform.localScale = new Vector2(1.0f, 1.0f);
                        wheels[0].anchor = new Vector2(2.98579f, -0.97258f);
                        wheels[0].connectedBody.gameObject.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        wheels[0].connectedBody.gameObject.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        wheels[0].connectedBody.gameObject.GetComponent<SpriteRenderer>().sortingOrder = -3;

                        wheels[1].connectedBody.gameObject.transform.localScale = new Vector2(1.0f, 1.0f);
                        wheels[1].anchor = new Vector2(-1.3284f, -0.97258f);
                        wheels[1].connectedBody.gameObject.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        wheels[1].connectedBody.gameObject.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        wheels[1].connectedBody.gameObject.GetComponent<SpriteRenderer>().sortingOrder = -3;

                        //Container upper part
                        GameObject BodyForeground = new GameObject("Foreground");
                        BodyForeground.transform.parent = Body;
                        BodyForeground.transform.localRotation = Quaternion.identity;
                        BodyForeground.transform.localScale = Vector3.one;
                        BodyForeground.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer BodyForegroundRenderer = BodyForeground.AddComponent<SpriteRenderer>();
                        BodyForegroundRenderer.sprite = ModAPI.LoadSprite("Textures/Farm Truck/BackBody.png");
                        BodyForegroundRenderer.sortingOrder = 5;
                        BodyForegroundRenderer.sortingLayerName = "Foreground";
                        BodyForeground.AddComponent<Optout>();

                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot
                        Instance.transform.GetChild(6).gameObject.SetActive(false); //BackDoor

                        //Hatch
                        GameObject Hatch = new GameObject("Hatch");
                        Hatch.transform.parent = Instance.transform;
                        Hatch.transform.localRotation = Body.localRotation;
                        Hatch.transform.localScale = Vector3.one;
                        Hatch.transform.localPosition = new Vector3(-3.583996f, 0.1855299f);
                        SpriteRenderer HatchRenderer = Hatch.AddComponent<SpriteRenderer>();
                        HatchRenderer.sprite = ModAPI.LoadSprite("Textures/Farm Truck/Hatch.png");
                        HatchRenderer.sortingLayerName = "Default";
                        HatchRenderer.sortingOrder = 1;
                        Hatch.layer = LayerMask.NameToLayer("Objects");
                        Hatch.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.042857144f, 0.342857152f),
                            new Vector2(-0.042857144f, -0.342857152f),
                            new Vector2(0.042857144f, -0.342857152f),
                            new Vector2(0.042857144f, 0.342857152f)
                        });
                        Hatch.AddComponent<Rigidbody2D>().mass = 0.2f;
                        FixedJoint2D HatchFixedJoint = Hatch.AddComponent<FixedJoint2D>();
                        HatchFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        HatchFixedJoint.breakForce = 300f;
                        HingeJoint2D HatchHingeJoint = Hatch.AddComponent<HingeJoint2D>();
                        HatchHingeJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        HatchHingeJoint.anchor = new Vector2(0f, -0.303f);
                        HatchHingeJoint.breakForce = 5000f;
                        HatchHingeJoint.breakTorque = 5000f;
                        HatchHingeJoint.useLimits = true;
                        HatchHingeJoint.limits = new JointAngleLimits2D
                        {
                            min = 0,
                            max = -90
                        };
                        Hatch.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Wood");
                        Hatch.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Hatch.AddComponent<Optout>();

                        //Third wheel
                        GameObject Wheel = new GameObject("Wheel3");
                        Wheel.transform.parent = Instance.transform;
                        Wheel.transform.localScale = new Vector3(1f, 1f);
                        Wheel.transform.localPosition = new Vector3(-2.6425f, -0.97258f);
                        SpriteRenderer WheelRenderer = Wheel.AddComponent<SpriteRenderer>();
                        WheelRenderer.sprite = WheelSprite;
                        WheelRenderer.sortingOrder = -3;
                        Wheel.layer = LayerMask.NameToLayer("Objects");
                        Wheel.AddComponent<CircleCollider2D>();
                        Rigidbody2D WheelRigidbody = Wheel.AddComponent<Rigidbody2D>();
                        WheelRigidbody.mass = 1f;
                        //Third wheel settings
                        WheelJoint2D WheelJoint = Body.gameObject.AddComponent<WheelJoint2D>();
                        WheelJoint.connectedBody = WheelRigidbody;
                        WheelJoint.anchor = new Vector3(-2.6425f, -0.97258f);
                        WheelJoint.motor = wheels[0].motor;
                        WheelJoint.suspension = wheels[0].suspension;
                        WheelJoint.breakForce = wheels[0].breakForce;
                        WheelJoint.useMotor = true;
                        Wheel.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                        Wheel.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Wheel.AddComponent<Optout>();

                        if (Body.GetComponent<CarBehaviour>().WheelJoints.Length == 2)
                        {
                            List<WheelJoint2D> wheelList = Body.GetComponent<CarBehaviour>().WheelJoints.ToList();
                            wheelList.Add(WheelJoint);
                            Body.GetComponent<CarBehaviour>().WheelJoints = wheelList.ToArray();

                            List<bool> wheelStateList = Body.GetComponent<CarBehaviour>().WheelJointState.ToList();
                            wheelStateList.Add(true);
                            Body.GetComponent<CarBehaviour>().WheelJointState = wheelStateList.ToArray();
                        }
                        else if (WheelJoint)
                        {
                            List<WheelJoint2D> wheelList = Body.GetComponent<CarBehaviour>().WheelJoints.ToList();
                            wheelList[2] = WheelJoint;
                            Body.GetComponent<CarBehaviour>().WheelJoints = wheelList.ToArray();

                            List<bool> wheelStateList = Body.GetComponent<CarBehaviour>().WheelJointState.ToList();
                            wheelStateList[2] = true;
                            Body.GetComponent<CarBehaviour>().WheelJointState = wheelStateList.ToArray();
                        }

                        //Hood
                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 500f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.542f, 0.098f);
                        Bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65,
                            max = 0
                        };
                        Bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.557142854f, 0.128571436f),
                            new Vector2(-0.553106904f, 0.068466112f),
                            new Vector2(0.559160829f, -0.129330948f),
                            new Vector2(0.555015981f, -0.0793172866f)
                        });

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.628571451f, 0.985714316f),
                            new Vector2(-0.628571451f, -0.985714316f),
                            new Vector2(0.628571451f, -0.985714316f),
                            new Vector2(0.625524461f, 0.351455122f),
                            new Vector2(0.196012348f, 0.983606339f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.614f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.GetComponent<BoxCollider2D>().size = new Vector3(1.4f, 0.4f);
                        GasTank.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Farm Truck/Gas Tank.png");
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Serialization
                        CarSerialisationHelper carSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        carSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            wheels[0].connectedBody.gameObject.transform.localPosition = new Vector2(2.98579f, -0.97258f);
                            wheels[1].connectedBody.gameObject.transform.localPosition = new Vector2(-1.3284f, -0.97258f);
                            FrontDoor.localPosition = new Vector3(1.6572f, 0.1143f);
                            Bonnet.localPosition = new Vector3(2.954f, 0.364f);
                            Instance.transform.GetChild(7).localPosition = new Vector3(2.826f, -0.044f);
                            GasTank.localPosition = new Vector3(0.2001419f, -0.7524748f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Hatch;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Wheel;
                        };

                        //Not colliding car parts with each other
                        NoCollide noCollide = Instance.GetComponent<NoCollide>();
                        noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Military Truck
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "MSR Military Truck",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Military truck. There is enough room in the back. There is a crate and a flask between the cabin and the back of the truck.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Military Truck.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/MSR Truck/Wheel.png");

                        //Car speed
                        float MotorSpeed = -2500f;

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 15f;
                        BodyPhysical.InitialMass = 15f;
                        BodyPhysical.rigidbody.mass = 15f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyColllider = Body.GetComponent<PolygonCollider2D>();
                        BodyColllider.pathCount = 3;
                        BodyColllider.SetPath(0, new Vector2[]
                        {
                            new Vector2(1.46530807f,1.51185596f),
                            new Vector2(1.47822034f,-0.399953902f),
                            new Vector2(1.58825696f,-0.39514491f),
                            new Vector2(1.58636296f,-0.681076705f),
                            new Vector2(2.64808655f,-0.681019902f),
                            new Vector2(3.07994795f,0.187701166f),
                            new Vector2(3.24031782f,0.258581102f),
                            new Vector2(4.69692707f,0.254394352f),
                            new Vector2(4.70195913f,0.671811283f),
                            new Vector2(4.45947742f,0.348631054f),
                            new Vector2(2.80276608f,0.395947158f),
                            new Vector2(2.92404723f,0.84687835f),
                            new Vector2(2.82750869f,0.86785543f),
                            new Vector2(2.4976635f,1.50942647f)
                        });
                        BodyColllider.SetPath(1, new Vector2[]
                        {
                            new Vector2(2.64369893f,0.930906653f),
                            new Vector2(2.41354251f,1.39550471f),
                            new Vector2(1.64103484f,1.4026258f),
                            new Vector2(1.67031074f,1.25701487f),
                            new Vector2(1.79813623f,0.0876212716f),
                            new Vector2(2.23330998f,0.118583359f),
                            new Vector2(2.30504465f,-0.0428237393f),
                            new Vector2(2.15716767f,-0.128844932f),
                            new Vector2(2.14916968f,-0.288752735f),
                            new Vector2(2.6499474f,-0.293266594f)
                        });
                        BodyColllider.SetPath(2, new Vector2[]
                        {
                            new Vector2(0.9527601f,0.0290977955f),
                            new Vector2(0.956498027f,1.37364304f),
                            new Vector2(0.872243166f,1.37595451f),
                            new Vector2(0.8752352f,0.116682172f),
                            new Vector2(-4.70261478f,0.113238446f),
                            new Vector2(-4.70337343f,0.0277739689f),
                            new Vector2(-4.47266769f,0.0232719332f),
                            new Vector2(-4.47752285f,-0.170859754f),
                            new Vector2(1.46347892f,-0.163661122f),
                            new Vector2(1.46313381f,0.034203738f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/MSR Truck/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/MSR Truck/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/MSR Truck/Bonnet.png"),
                                Background = ModAPI.LoadSprite("Textures/MSR Truck/Background.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero; //Body bottom position

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].anchor = new Vector2(3.714f, -0.7871f);
                        Wheels[0].connectedBody.gameObject.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.gameObject.GetComponent<CircleCollider2D>().radius = 0.7285714f;
                        Wheels[0].connectedBody.gameObject.GetComponent<SpriteRenderer>().sortingOrder = -3;

                        Wheels[1].anchor = new Vector2(-0.942f, -0.7871f);
                        Wheels[1].connectedBody.gameObject.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.gameObject.GetComponent<CircleCollider2D>().radius = 0.7285714f;
                        Wheels[1].connectedBody.gameObject.GetComponent<SpriteRenderer>().sortingOrder = -3;

                        //Container upper part
                        GameObject BackBodyTop = new GameObject("BackBody");
                        BackBodyTop.transform.parent = Body;
                        BackBodyTop.transform.localRotation = Quaternion.identity;
                        BackBodyTop.transform.localScale = Vector3.one;
                        BackBodyTop.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer BackBodyTopRenderer = BackBodyTop.AddComponent<SpriteRenderer>();
                        BackBodyTopRenderer.sprite = ModAPI.LoadSprite("Textures/MSR Truck/BackBody.png");
                        BackBodyTopRenderer.sortingOrder = 5;
                        BackBodyTopRenderer.sortingLayerName = "Foreground";
                        BackBodyTop.AddComponent<Optout>();

                        //Hatch
                        GameObject Hatch = new GameObject("Hatch");
                        Hatch.transform.parent = Instance.transform;
                        Hatch.transform.localRotation = Body.localRotation;
                        Hatch.transform.localScale = Vector3.one;
                        Hatch.transform.localPosition = new Vector3(-4.684f, 0.601f);
                        SpriteRenderer HatchRenderer = Hatch.AddComponent<SpriteRenderer>();
                        HatchRenderer.sprite = ModAPI.LoadSprite("Textures/MSR Truck/Hatch.png");
                        HatchRenderer.sortingLayerName = "Default";
                        HatchRenderer.sortingOrder = 1;
                        Hatch.layer = LayerMask.NameToLayer("Objects");
                        Hatch.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.0142949f,0.558966279f),
                            new Vector2(-0.0142989159f,0.448911011f),
                            new Vector2(-0.0689935684f,0.413397193f),
                            new Vector2(-0.0723423958f,0.278105855f),
                            new Vector2(-0.0143065453f,0.241511166f),
                            new Vector2(-0.0143349543f,-0.555103779f),
                            new Vector2(0.0714285746f,-0.557142854f),
                            new Vector2(0.0714285746f,0.557142854f)
                        });
                        Hatch.AddComponent<Rigidbody2D>().mass = 0.2f;
                        FixedJoint2D HatchFixedJoint = Hatch.AddComponent<FixedJoint2D>();
                        HatchFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        HatchFixedJoint.breakForce = 300f;
                        HingeJoint2D HatchHingeJoint = Hatch.AddComponent<HingeJoint2D>();
                        HatchHingeJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        HatchHingeJoint.anchor = new Vector2(0.0287f, -0.515f);
                        HatchHingeJoint.breakForce = 5000f;
                        HatchHingeJoint.breakTorque = 5000f;
                        HatchHingeJoint.useLimits = true;
                        HatchHingeJoint.limits = new JointAngleLimits2D
                        {
                            min = 0,
                            max = -90
                        };
                        Hatch.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Hatch.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Hatch.AddComponent<Optout>();

                        //Third wheel
                        GameObject Wheel = new GameObject("Wheel3");
                        Wheel.transform.parent = Instance.transform;
                        Wheel.transform.rotation = Instance.transform.rotation;
                        Wheel.transform.localScale = new Vector3(1f, 1f);
                        Wheel.transform.localPosition = new Vector3(-2.8853f, -0.7871f);
                        SpriteRenderer WheelRenderer = Wheel.AddComponent<SpriteRenderer>();
                        WheelRenderer.sprite = WheelSprite;
                        WheelRenderer.sortingOrder = -3;
                        Wheel.layer = LayerMask.NameToLayer("Objects");
                        Wheel.AddComponent<CircleCollider2D>();
                        Rigidbody2D WheelRigidbody = Wheel.AddComponent<Rigidbody2D>();
                        WheelRigidbody.mass = 1f;
                        //Third wheel settings
                        WheelJoint2D WheelJoint = Body.gameObject.AddComponent<WheelJoint2D>();
                        WheelJoint.connectedBody = Wheel.GetComponent<Rigidbody2D>();
                        WheelJoint.anchor = new Vector3(-2.8853f, -0.7871f);
                        WheelJoint.motor = Wheels[0].motor;
                        WheelJoint.suspension = Wheels[0].suspension;
                        WheelJoint.breakForce = Wheels[0].breakForce;
                        WheelJoint.useMotor = true;
                        Wheel.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                        Wheel.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Wheel.AddComponent<Optout>();

                        if (Body.GetComponent<CarBehaviour>().WheelJoints.Length == 2)
                        {
                            List<WheelJoint2D> wheelList = Body.GetComponent<CarBehaviour>().WheelJoints.ToList();
                            wheelList.Add(WheelJoint);
                            Body.GetComponent<CarBehaviour>().WheelJoints = wheelList.ToArray();

                            List<bool> wheelStateList = Body.GetComponent<CarBehaviour>().WheelJointState.ToList();
                            wheelStateList.Add(true);
                            Body.GetComponent<CarBehaviour>().WheelJointState = wheelStateList.ToArray();
                        }
                        else if (WheelJoint)
                        {
                            List<WheelJoint2D> wheelList = Body.GetComponent<CarBehaviour>().WheelJoints.ToList();
                            wheelList[2] = WheelJoint;
                            Body.GetComponent<CarBehaviour>().WheelJoints = wheelList.ToArray();

                            List<bool> wheelStateList = Body.GetComponent<CarBehaviour>().WheelJointState.ToList();
                            wheelStateList[2] = true;
                            Body.GetComponent<CarBehaviour>().WheelJointState = wheelStateList.ToArray();
                        }

                        //Crate with bottles
                        GameObject CrateWithBottles = new GameObject("Crate With Bottles");
                        CrateWithBottles.transform.parent = Instance.transform;
                        CrateWithBottles.transform.localScale = Vector3.one;
                        CrateWithBottles.transform.localPosition = new Vector3(1.217f, 0.398f);
                        SpriteRenderer CrateWithBottlesRenderer = CrateWithBottles.AddComponent<SpriteRenderer>();
                        CrateWithBottlesRenderer.sprite = ModAPI.LoadSprite("Textures/MSR Truck/CrateWithBottles.png");
                        CrateWithBottlesRenderer.sortingLayerName = "Default";
                        CrateWithBottlesRenderer.sortingOrder = 0;
                        CrateWithBottles.layer = LayerMask.NameToLayer("Objects");
                        CrateWithBottles.AddComponent<Rigidbody2D>().mass = 0.5f;
                        PolygonCollider2D CrateWithBottlesCollider = CrateWithBottles.AddComponent<PolygonCollider2D>();
                        CrateWithBottlesCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.22857143f, 0.371428579f),
                            new Vector2(-0.22857143f, -0.371428579f),
                            new Vector2(0.22857143f, -0.371428579f),
                            new Vector2(0.22857143f, 0.371428579f)
                        });
                        CrateWithBottles.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Wood");
                        CrateWithBottles.AddComponent<AudioSourceTimeScaleBehaviour>();
                        CrateWithBottles.AddComponent<Optout>();

                        //Flask
                        GameObject Flask = new GameObject("Flask");
                        Flask.transform.parent = Instance.transform;
                        Flask.transform.localScale = Vector3.one;
                        Flask.transform.localPosition = new Vector3(1.3173f, 0.8973f);
                        SpriteRenderer FlaskRenderer = Flask.AddComponent<SpriteRenderer>();
                        FlaskRenderer.sprite = ModAPI.LoadSprite("Textures/MSR Truck/Flask.png");
                        FlaskRenderer.sortingLayerName = "Default";
                        FlaskRenderer.sortingOrder = 0;
                        Flask.layer = LayerMask.NameToLayer("Objects");
                        Flask.AddComponent<Rigidbody2D>().mass = 0.1f;
                        PolygonCollider2D FlaskCollider = Flask.AddComponent<PolygonCollider2D>();
                        FlaskCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.0291287377f,0.128571436f),
                            new Vector2(-0.0853238106f,0.0389609337f),
                            new Vector2(-0.0574215129f,-0.125034854f),
                            new Vector2(0.0562426522f,-0.127392575f),
                            new Vector2(0.0865358114f,0.0383110642f),
                            new Vector2(0.0350230411f,0.127982065f)
                        });
                        Flask.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Flask.GetComponent<PhysicalBehaviour>().HoldingPositions = new Vector3[]
                        {
                            new Vector2(0f, -0.0119f)
                        };
                        Flask.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Flask.AddComponent<Optout>();

                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Back door
                        Instance.transform.GetChild(6).gameObject.SetActive(false); //Boot

                        //Hood
                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 500f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.8654f, 0.0566f);
                        Bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65f,
                            max = 0f
                        };
                        Bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.885714293f,0.0880013108f),
                            new Vector2(-0.887136221f,0.0305252075f),
                            new Vector2(0.885714293f,-0.0914298892f),
                            new Vector2(0.878290057f,-0.0360013768f)
                        });

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.557142854f,0.899999976f),
                            new Vector2(-0.557142854f,-0.899999976f),
                            new Vector2(0.557142854f,-0.899999976f),
                            new Vector2(0.555117488f,0.380925596f),
                            new Vector2(0.278149068f,0.899999976f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.52f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.GetComponent<BoxCollider2D>().size = new Vector3(1.571429f, 0.5428572f);
                        GasTank.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/MSR Truck/Gas Tank.png");
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Serialization
                        CarSerialisationHelper carSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        carSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.gameObject.transform.localPosition = new Vector2(3.714f, -0.7871f);
                            Wheels[1].connectedBody.gameObject.transform.localPosition = new Vector2(-0.942f, -0.7871f);
                            Bonnet.localPosition = new Vector3(3.809f, 0.76f);
                            FrontDoor.localPosition = new Vector3(2.142f, 0.557f);
                            Instance.transform.GetChild(7).localPosition = new Vector3(3.59f, 0.29f);
                            GasTank.localPosition = new Vector3(0.66f, -0.466f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Hatch;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Wheel;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = CrateWithBottles;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Flask;
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>().TakeWhile((collider) => collider != FlaskCollider && collider != CrateWithBottlesCollider).ToArray();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>().TakeWhile((collider) => collider != FlaskCollider && collider != CrateWithBottlesCollider).ToArray();
                    }
                }
            );
        #endregion
        #region Military Truck Flat
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "MSR Military Truck Flat",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Military truck, but with a flat back body. Almost no different from the original.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/MSR Military Truck Flat.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/MSR Truck/Wheel.png");

                        //Car speed
                        float MotorSpeed = -2500f;

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 15f;
                        BodyPhysical.InitialMass = 15f;
                        BodyPhysical.rigidbody.mass = 15f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.offset = new Vector2(0f, -0.36f);
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(1.44236839f,1.50263774f),
                            new Vector2(1.43902588f,1.78895128f),
                            new Vector2(1.35601234f,1.78925645f),
                            new Vector2(1.35777092f,0.102080703f),
                            new Vector2(-4.70127106f,0.100876212f),
                            new Vector2(-4.7024231f,0.0184329748f),
                            new Vector2(-4.47260475f,0.018327117f),
                            new Vector2(-4.4715271f,-0.182109475f),
                            new Vector2(1.47076797f,-0.190062165f),
                            new Vector2(1.47822034f,-0.399953902f),
                            new Vector2(1.58825696f,-0.395144939f),
                            new Vector2(1.58636296f,-0.681076646f),
                            new Vector2(2.64808655f,-0.681019902f),
                            new Vector2(3.07994795f,0.187701166f),
                            new Vector2(3.24031782f,0.258581102f),
                            new Vector2(4.69692707f,0.254394352f),
                            new Vector2(4.70195913f,0.671811283f),
                            new Vector2(4.45947742f,0.348631054f),
                            new Vector2(2.80276608f,0.395947158f),
                            new Vector2(2.92404723f,0.84687835f),
                            new Vector2(2.82750869f,0.86785543f),
                            new Vector2(2.4976635f,1.50942647f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(2.64369893f,0.930906653f),
                            new Vector2(2.41354251f,1.39550471f),
                            new Vector2(1.64103484f,1.4026258f),
                            new Vector2(1.67031074f,1.25701487f),
                            new Vector2(1.79813623f,0.0876212716f),
                            new Vector2(2.23330998f,0.118583359f),
                            new Vector2(2.30504465f,-0.0428237393f),
                            new Vector2(2.15716767f,-0.128844932f),
                            new Vector2(2.14916968f,-0.288752735f),
                            new Vector2(2.6499474f,-0.293266594f)
                        });
                        BoxCollider2D BodyTrigger = Body.gameObject.AddComponent<BoxCollider2D>();
                        BodyTrigger.offset = new Vector2(-1.67964f, 0.4320064f);
                        BodyTrigger.size = new Vector2(6.032009f, 1.380166f);
                        BodyTrigger.isTrigger = true;
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/MSR Truck/MSR Truck Flat/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/MSR Truck/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/MSR Truck/Bonnet.png"),
                                Background = ModAPI.LoadSprite("Textures/MSR Truck/MSR Truck Flat/Background.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero; //Body bottom position

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].anchor = new Vector3(-2.8853f, -1.158549f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.7285714f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -3;

                        Wheels[1].anchor = new Vector2(-0.942f, -1.158549f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.7285714f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -3;

                        //Third wheel
                        GameObject Wheel = new GameObject("Wheel3");
                        Wheel.transform.parent = Instance.transform;
                        Wheel.transform.localScale = new Vector3(1f, 1f);
                        Wheel.transform.localPosition = new Vector2(3.714f, -0.7871f);
                        SpriteRenderer WheelRenderer = Wheel.AddComponent<SpriteRenderer>();
                        WheelRenderer.sprite = WheelSprite;
                        Wheel.GetComponent<SpriteRenderer>().sortingOrder = -3;
                        Wheel.layer = LayerMask.NameToLayer("Objects");
                        Wheel.AddComponent<CircleCollider2D>();
                        Wheel.AddComponent<Rigidbody2D>().mass = 1f;
                        WheelJoint2D WheelJoint = Body.gameObject.AddComponent<WheelJoint2D>();
                        WheelJoint.connectedBody = Wheel.GetComponent<Rigidbody2D>();
                        WheelJoint.anchor = new Vector2(3.714f, -1.158549f);
                        WheelJoint.motor = Wheels[0].motor;
                        WheelJoint.suspension = Wheels[0].suspension;
                        WheelJoint.breakForce = Wheels[0].breakForce;
                        WheelJoint.useMotor = true;
                        Wheel.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                        Wheel.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Wheel.AddComponent<Optout>();

                        if (Body.GetComponent<CarBehaviour>().WheelJoints.Length == 2)
                        {
                            List<WheelJoint2D> wheelList = Body.GetComponent<CarBehaviour>().WheelJoints.ToList();
                            wheelList.Add(WheelJoint);
                            Body.GetComponent<CarBehaviour>().WheelJoints = wheelList.ToArray();

                            List<bool> wheelStateList = Body.GetComponent<CarBehaviour>().WheelJointState.ToList();
                            wheelStateList.Add(true);
                            Body.GetComponent<CarBehaviour>().WheelJointState = wheelStateList.ToArray();
                        }
                        else if (WheelJoint)
                        {
                            List<WheelJoint2D> wheelList = Body.GetComponent<CarBehaviour>().WheelJoints.ToList();
                            wheelList[2] = WheelJoint;
                            Body.GetComponent<CarBehaviour>().WheelJoints = wheelList.ToArray();

                            List<bool> wheelStateList = Body.GetComponent<CarBehaviour>().WheelJointState.ToList();
                            wheelStateList[2] = true;
                            Body.GetComponent<CarBehaviour>().WheelJointState = wheelStateList.ToArray();
                        }

                        //Container upper part
                        GameObject BackBodyTop = new GameObject("BodyTwo");
                        BackBodyTop.transform.parent = Body;
                        BackBodyTop.transform.localRotation = Quaternion.identity;
                        BackBodyTop.transform.localScale = Vector3.one;
                        BackBodyTop.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer BackBodyTopRenderer = BackBodyTop.AddComponent<SpriteRenderer>();
                        BackBodyTopRenderer.sprite = ModAPI.LoadSprite("Textures/MSR Truck/MSR Truck Flat/BackBody.png");
                        BackBodyTopRenderer.sortingOrder = 10;
                        BackBodyTopRenderer.sortingLayerName = "Foreground";
                        BackBodyTop.AddComponent<Optout>();

                        //Hood
                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 500f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.8654f, 0.0566f);
                        Bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65f,
                            max = 0f
                        };
                        Bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.885714293f,0.0880013108f),
                            new Vector2(-0.887136221f,0.0305252075f),
                            new Vector2(0.885714293f,-0.0914298892f),
                            new Vector2(0.878290057f,-0.0360013768f)
                        });

                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Back door
                        Instance.transform.GetChild(6).gameObject.SetActive(false); //Boot

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.557142854f,0.899999976f),
                            new Vector2(-0.557142854f,-0.899999976f),
                            new Vector2(0.557142854f,-0.899999976f),
                            new Vector2(0.555117488f,0.380925596f),
                            new Vector2(0.278149068f,0.899999976f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.52f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.GetComponent<BoxCollider2D>().size = new Vector3(1.571429f, 0.5428572f);
                        GasTank.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/MSR Truck/Gas Tank.png");
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Not colliding objects with wheels in a certain area
                        NoCollideWithObjectsInTrigger noCollideWithObjectsInTrigger = Body.gameObject.AddComponent<NoCollideWithObjectsInTrigger>();
                        noCollideWithObjectsInTrigger.Trigger = BodyTrigger;
                        noCollideWithObjectsInTrigger.Colliders = new Collider2D[]
                        {
                            Wheels[0].connectedBody.GetComponent<CircleCollider2D>(),
                            Wheels[1].connectedBody.GetComponent<CircleCollider2D>()
                        };

                        //Serialization
                        CarSerialisationHelper carSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        carSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.transform.localPosition = new Vector3(-2.8853f, -1.158549f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(-0.942f, -0.7871f);
                            Bonnet.localPosition = new Vector3(3.809f, 0.396f);
                            GasTank.localScale = new Vector3(1f, 1f);
                            GasTank.localPosition = new Vector3(0.66f, -0.845f);
                            FrontDoor.localPosition = new Vector3(2.142f, 0.185f);

                            //Engine
                            Transform Engine = Instance.transform.GetChild(7);
                            Engine.localPosition = new Vector3(3.59f, -0.14f);
                            Engine.localScale = new Vector3(1f, 1f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Wheel;
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region 1986 Fleetwood Bounder
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "1986 Fleetwood Bounder",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Minibus for traveling.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/1986 Fleetwood Bounder.png"),
                    AfterSpawn = (Instance) =>
                    {
                        //Sprites
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/1986 Fleetwood Bounder/Wheel.png");
                        Sprite ConditionerSprite = ModAPI.LoadSprite("Textures/1986 Fleetwood Bounder/Air Conditioner.png");

                        //Car speed
                        float MotorSpeed = -2000f;

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 4;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 30f;
                        BodyPhysical.InitialMass = 30f;
                        BodyPhysical.rigidbody.mass = 30f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(3.95126271f, -1.21406126f),
                            new Vector2(4.20475864f, -0.884922028f),
                            new Vector2(4.31670475f, -0.272265404f),
                            new Vector2(3.54802966f, 1.16612053f),
                            new Vector2(3.41575861f, 1.2631228f),
                            new Vector2(-3.93299937f, 1.2753067f),
                            new Vector2(-4.1485095f, 0.0248128176f),
                            new Vector2(-4.14951372f, -0.394102097f),
                            new Vector2(-3.9275887f, -1.08329582f),
                            new Vector2(-3.1112361f, -1.21891356f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(-3.87268186f, 1.21681595f),
                            new Vector2(-4.09020281f, -0.0152797699f),
                            new Vector2(-4.05875015f, -0.513721466f),
                            new Vector2(-3.90804005f, -0.93362999f),
                            new Vector2(3.05877709f, -0.934898376f),
                            new Vector2(3.00641942f, -0.52107811f),
                            new Vector2(3.00538325f, 0.74625206f),
                            new Vector2(3.14118838f, 0.745500565f),
                            new Vector2(3.10074306f, 0.364789963f),
                            new Vector2(3.20755458f, -0.527545929f),
                            new Vector2(3.6894834f, -0.54388237f),
                            new Vector2(3.53091168f, -0.952850342f),
                            new Vector2(4.08230209f, -0.937379837f),
                            new Vector2(4.25001717f, -0.281492233f),
                            new Vector2(3.43326569f, 1.2063694f)
                        });
                        BoxCollider2D BodyTrigger = Body.gameObject.AddComponent<BoxCollider2D>();
                        BodyTrigger.offset = new Vector2(0.3294941f, -0.2972946f);
                        BodyTrigger.size = new Vector2(7.047341f, 1.297222f);
                        BodyTrigger.isTrigger = true;
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/1986 Fleetwood Bounder/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/1986 Fleetwood Bounder/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/1986 Fleetwood Bounder/Door.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/1986 Fleetwood Bounder/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/1986 Fleetwood Bounder/EmptyBackground.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/1986 Fleetwood Bounder/Door.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero; //Body bottom position

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].anchor = new Vector2(-1.4858f, -1.115f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        Wheels[1].anchor = new Vector2(3.2858f, -1.115f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Body top
                        GameObject BodyForeground = new GameObject("Foreground");
                        BodyForeground.transform.parent = Body;
                        BodyForeground.transform.localRotation = Quaternion.identity;
                        BodyForeground.transform.localScale = Vector3.one;
                        BodyForeground.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer BodyForegroundRenderer = BodyForeground.AddComponent<SpriteRenderer>();
                        BodyForegroundRenderer.sprite = ModAPI.LoadSprite("Textures/1986 Fleetwood Bounder/Switchable Body.png");
                        BodyForegroundRenderer.sortingOrder = 6;
                        BodyForegroundRenderer.sortingLayerName = "Foreground";
                        BodyForeground.AddComponent<Optout>();

                        //Background two
                        GameObject Background2 = new GameObject("Background2");
                        Background2.transform.parent = Body;
                        Background2.transform.localRotation = Quaternion.identity;
                        Background2.transform.localScale = Vector3.one;
                        Background2.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer BodyBackgroundRenderer = Background2.AddComponent<SpriteRenderer>();
                        BodyBackgroundRenderer.sprite = ModAPI.LoadSprite("Textures/1986 Fleetwood Bounder/Background2.png");
                        BodyBackgroundRenderer.sortingOrder = 3;
                        BodyBackgroundRenderer.sortingLayerName = "Foreground";
                        Background2.AddComponent<Optout>();

                        //Conditioner 1
                        GameObject Conditioner = new GameObject("Conditioner");
                        Conditioner.transform.parent = Instance.transform;
                        Conditioner.transform.localRotation = Body.localRotation;
                        Conditioner.transform.localScale = Vector3.one;
                        Conditioner.transform.localPosition = new Vector3(0.973f, 1.406f);
                        SpriteRenderer ConditionerRenderer = Conditioner.AddComponent<SpriteRenderer>();
                        ConditionerRenderer.sprite = ConditionerSprite;
                        ConditionerRenderer.sortingLayerName = "Foreground";
                        ConditionerRenderer.sortingOrder = -5;
                        Conditioner.layer = LayerMask.NameToLayer("Objects");
                        Conditioner.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.402703375f, 0.140614107f),
                            new Vector2(-0.478530407f, -0.0103244781f),
                            new Vector2(-0.438599914f, -0.133883074f),
                            new Vector2(0.276011199f, -0.131640032f),
                            new Vector2(0.463701487f, -0.00764656067f),
                            new Vector2(0.234438509f, 0.139558733f)
                        });
                        Conditioner.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D ConditionerFixedJoint = Conditioner.AddComponent<FixedJoint2D>();
                        ConditionerFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        ConditionerFixedJoint.breakForce = 800f;
                        ConditionerFixedJoint.anchor = new Vector2(0f, -0.138f);
                        Conditioner.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Conditioner.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Conditioner.AddComponent<Optout>();

                        //Conditioner 2
                        GameObject Conditioner2 = new GameObject("Conditioner2");
                        Conditioner2.transform.parent = Instance.transform;
                        Conditioner2.transform.localRotation = Body.localRotation;
                        Conditioner2.transform.localScale = Vector3.one;
                        Conditioner2.transform.localPosition = new Vector3(-2.93f, 1.406f);
                        SpriteRenderer Conditioner2Renderer = Conditioner2.AddComponent<SpriteRenderer>();
                        Conditioner2Renderer.sprite = ConditionerSprite;
                        Conditioner2Renderer.sortingLayerName = "Foreground";
                        Conditioner2Renderer.sortingOrder = -5;
                        Conditioner2.layer = LayerMask.NameToLayer("Objects");
                        Conditioner2.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.402703375f, 0.140614107f),
                            new Vector2(-0.478530407f, -0.0103244781f),
                            new Vector2(-0.438599914f, -0.133883074f),
                            new Vector2(0.276011199f, -0.131640032f),
                            new Vector2(0.463701487f, -0.00764656067f),
                            new Vector2(0.234438509f, 0.139558733f)
                        });
                        Conditioner2.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D Conditioner2FixedJoint = Conditioner2.AddComponent<FixedJoint2D>();
                        Conditioner2FixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        Conditioner2FixedJoint.breakForce = 800f;
                        Conditioner2FixedJoint.anchor = new Vector2(0f, -0.138f);
                        Conditioner2.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Conditioner2.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Conditioner2.AddComponent<Optout>();

                        Instance.transform.GetChild(3).gameObject.SetActive(false); //Hood
                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot
                        Instance.transform.GetChild(6).gameObject.SetActive(false); //Back door

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.400000006f, 1.01428568f),
                            new Vector2(-0.400000006f, -1.01428568f),
                            new Vector2(0.400000006f, -1.01428568f),
                            new Vector2(0.400000006f, 1.01428568f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.375f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Engine
                        Transform Engine = Instance.transform.GetChild(7);

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);

                        //Not colliding objects with parts in a certain area
                        NoCollideWithObjectsInTrigger noCollideWithObjectsInTrigger = Body.gameObject.AddComponent<NoCollideWithObjectsInTrigger>();
                        noCollideWithObjectsInTrigger.Trigger = BodyTrigger;
                        noCollideWithObjectsInTrigger.Colliders = new Collider2D[]
                        {
                            Wheels[0].connectedBody.GetComponent<CircleCollider2D>(),
                            Wheels[1].connectedBody.GetComponent<CircleCollider2D>(),
                            GasTank.GetComponent<BoxCollider2D>(),
                            Engine.GetComponent<BoxCollider2D>()
                        };

                        //Serialization
                        CarSerialisationHelper carSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        carSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Car parts position
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(-1.4858f, -1.115f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(3.2858f, -1.115f);
                            FrontDoor.localPosition = new Vector3(1.1f, -0.0572f);
                            Engine.localPosition = new Vector3(2.158f, -0.796f);
                            GasTank.localPosition = new Vector3(-2.271f, -0.801f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Conditioner;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Conditioner2;
                        };
                        carSerialisationHelper.AfterSpawn = () =>
                        {
                            //Inside settings
                            InsideBehaviour Inside = Body.gameObject.GetOrAddComponent<InsideBehaviour>();
                            Inside.ForegroundRenderers = new SpriteRenderer[]
                            {
                                BodyForegroundRenderer,
                                FrontDoor.GetComponent<SpriteRenderer>()
                            };
                            if (Inside.ShowInside)
                            {
                                Inside.UpdateInside();
                            }

                            ActOnJointBreak FrontDoorActOnJointBreak = FrontDoor.gameObject.AddComponent<ActOnJointBreak>();
                            FrontDoorActOnJointBreak.Joint = FrontDoor.GetComponent<FixedJoint2D>();
                            FrontDoorActOnJointBreak.OnBreak.AddListener(() =>
                            {
                                if (Inside.ShowInside)
                                {
                                    FrontDoor.GetComponent<SpriteRenderer>().enabled = true;
                                }
                                List<SpriteRenderer> List = Inside.ForegroundRenderers.ToList();
                                List.Remove(FrontDoor.GetComponent<SpriteRenderer>());
                                Inside.ForegroundRenderers = List.ToArray();
                            });
                        };

                        //Not colliding car parts with each other
                        NoCollide noCollide = Instance.GetComponent<NoCollide>();
                        noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Minibus
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Minibus",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Minibus for 3 people. There is room in the back for the necessary things. You can also open the side door by clicking on it (F).",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Van.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/Van/Wheel.png");

                        //Car speed
                        float MotorSpeed = -2500f;
                        float Scale = 1.2f; //Car size
                        float SideDoorSpeed = -2f; //Side door slider speed

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 4;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 28f;
                        BodyPhysical.InitialMass = 28f;
                        BodyPhysical.rigidbody.mass = 28f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 4;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(3.15691495f,-0.931653619f),
                            new Vector2(3.62546349f,-0.772631109f),
                            new Vector2(3.62074685f,0.168344378f),
                            new Vector2(2.86787462f,0.809890032f),
                            new Vector2(2.33458424f,1.17634201f),
                            new Vector2(1.83208358f,1.31062424f),
                            new Vector2(-2.57717991f,1.31108975f),
                            new Vector2(-3.44039559f,1.20598054f),
                            new Vector2(-3.44677162f,1.08520389f),
                            new Vector2(2.28238511f,1.08480334f),
                            new Vector2(3.10447192f,0.437182605f),
                            new Vector2(3.11116028f,-0.59101665f),
                            new Vector2(-3.62533212f,-0.589511573f),
                            new Vector2(-3.52640152f,-0.939671993f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(0.222486973f,1.02349257f),
                            new Vector2(0.371715426f,-0.211736381f),
                            new Vector2(0.834664583f,-0.213941306f),
                            new Vector2(0.83528316f,-0.366983503f),
                            new Vector2(0.713599145f,-0.568572283f),
                            new Vector2(0.264045954f,-0.571894884f),
                            new Vector2(0.134879589f,-0.388427258f),
                            new Vector2(0.0875671506f,0.986581862f)
                        });
                        BodyCollider.SetPath(2, new Vector2[]
                        {
                            new Vector2(-1.15413761f,0.980700254f),
                            new Vector2(-0.963344932f,0.99483037f),
                            new Vector2(-0.851079941f,-0.203779608f),
                            new Vector2(-0.347153604f,-0.202479377f),
                            new Vector2(-0.483695835f,-0.581460476f),
                            new Vector2(-0.93742311f,-0.580495834f),
                            new Vector2(-1.04854047f,-0.405968726f)
                        });
                        BodyCollider.SetPath(3, new Vector2[]
                        {
                            new Vector2(2.49370933f,-0.361462563f),
                            new Vector2(2.49317122f,-0.213045105f),
                            new Vector2(2.03016782f,-0.1982878f),
                            new Vector2(1.90141118f,1.0178448f),
                            new Vector2(1.73807299f,1.02446342f),
                            new Vector2(1.77442002f,-0.43494615f)
                        });
                        BoxCollider2D BodyTrigger = Body.gameObject.AddComponent<BoxCollider2D>();
                        BodyTrigger.offset = new Vector2(-0.3564758f, -0.1317928f);
                        BodyTrigger.size = new Vector2(6.101685f, 1.348838f);
                        BodyTrigger.isTrigger = true;
                        Body.GetComponent<RandomCarTextureBehaviour>().OnAfterChange = new UnityEvent();
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Van/Green/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Van/Green/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Van/Green/FrontDoor.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Van/Green/Bonnet.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Van/Red/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Van/Red/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Van/Red/FrontDoor.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Van/Red/Bonnet.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Van/Gray/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Van/Gray/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Van/Gray/FrontDoor.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Van/Gray/Bonnet.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero; //Car bottom position

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].suspension = new JointSuspension2D()
                        {
                            frequency = 14f,
                            angle = 90f,
                            dampingRatio = 0.7f
                        };
                        Wheels[0].anchor = new Vector2(2.0715f, -0.8717f);
                        Wheels[0].connectedBody.gameObject.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.gameObject.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        Wheels[0].connectedBody.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        Wheels[0].connectedBody.gameObject.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        Wheels[1].suspension = new JointSuspension2D()
                        {
                            frequency = 10f,
                            angle = 90f,
                            dampingRatio = 0.7f
                        };
                        Wheels[1].anchor = new Vector2(-1.9853f, -0.8717f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Body top
                        GameObject BackBodyTop = new GameObject("BackBodyTop");
                        BackBodyTop.transform.parent = Body;
                        BackBodyTop.transform.localRotation = Quaternion.identity;
                        BackBodyTop.transform.localScale = Vector3.one;
                        BackBodyTop.transform.localPosition = Vector3.zero;
                        SpriteRenderer BackBodyTopRenderer = BackBodyTop.AddComponent<SpriteRenderer>();
                        BackBodyTopRenderer.sortingOrder = 6;
                        BackBodyTopRenderer.sortingLayerName = "Foreground";
                        BackBodyTop.AddComponent<RandomCarTextureChildBehaviour>().Sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Van/Green/Switchable Body.png"),
                            ModAPI.LoadSprite("Textures/Van/Red/Switchable Body.png"),
                            ModAPI.LoadSprite("Textures/Van/Gray/Switchable Body.png"),
                        };
                        BackBodyTop.AddComponent<Optout>();

                        //Background two
                        GameObject Background2 = new GameObject("Background2");
                        Background2.transform.parent = Body;
                        Background2.transform.localRotation = Quaternion.identity;
                        Background2.transform.localScale = Vector3.one;
                        Background2.transform.localPosition = Vector3.zero;
                        SpriteRenderer bodyBackgroundRenderer = Background2.AddComponent<SpriteRenderer>();
                        bodyBackgroundRenderer.sprite = ModAPI.LoadSprite("Textures/Van/Background2.png");
                        bodyBackgroundRenderer.sortingOrder = 3;
                        bodyBackgroundRenderer.sortingLayerName = "Foreground";
                        Background2.AddComponent<Optout>();

                        //Side door
                        GameObject SideDoor = new GameObject("Side Door");
                        SideDoor.transform.parent = Instance.transform;
                        SideDoor.transform.localRotation = Body.localRotation;
                        SideDoor.transform.localScale = Vector3.one;
                        SideDoor.transform.localPosition = new Vector3(-0.01439f, 0.1428f);
                        SpriteRenderer SideDoorRenderer = SideDoor.AddComponent<SpriteRenderer>();
                        SideDoorRenderer.sprite = ModAPI.LoadSprite("Textures/Van/Green/BackDoor.png");
                        SideDoorRenderer.sortingLayerName = "Foreground";
                        SideDoorRenderer.sortingOrder = 7;
                        SideDoor.AddComponent<RandomCarTextureChildBehaviour>().Sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Van/Green/BackDoor.png"),
                            ModAPI.LoadSprite("Textures/Van/Red/BackDoor.png"),
                            ModAPI.LoadSprite("Textures/Van/Gray/BackDoor.png"),
                        };
                        SideDoor.layer = LayerMask.NameToLayer("Debris");
                        SideDoor.AddComponent<Rigidbody2D>().mass = 0.1f;
                        AudioSource SlideSource = SideDoor.AddComponent<AudioSource>();
                        SlideSource.spatialBlend = 1f;
                        SlideSource.minDistance = 100f;
                        SliderJoint2D SideDoorSlider = SideDoor.AddComponent<SliderJoint2D>();
                        SideDoorSlider.connectedBody = Body.GetComponent<Rigidbody2D>();
                        SideDoorSlider.connectedAnchor = new Vector2(-0.01439f, 0.1428f);
                        SideDoorSlider.autoConfigureAngle = false;
                        SideDoorSlider.useLimits = true;
                        SideDoorSlider.limits = new JointTranslationLimits2D
                        {
                            min = 0f,
                            max = 1.7f
                        };
                        SideDoorSlider.breakForce = 2000f;
                        SideDoorSlider.breakTorque = 1500f;
                        bool Opened = false;
                        JointMotor2D Motor = new JointMotor2D
                        {
                            maxMotorTorque = 25f,
                            motorSpeed = SideDoorSpeed
                        };
                        SideDoorSlider.useMotor = true;
                        SideDoorSlider.motor = Motor;
                        SideDoor.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            if (SideDoorSlider)
                            {
                                Opened = !Opened;
                                Motor.motorSpeed = Opened ? SideDoorSpeed * -1f : SideDoorSpeed;
                                SideDoorSlider.motor = Motor;
                            }
                        };
                        SideDoor.AddComponent<BoxCollider2D>();
                        JointSerialisationHelper SideDoorSliderSerialisationHelper = SideDoor.AddComponent<JointSerialisationHelper>();
                        SideDoorSliderSerialisationHelper.Joint = SideDoorSlider;
                        SideDoor.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        SideDoor.AddComponent<AudioSourceTimeScaleBehaviour>();
                        SideDoor.AddComponent<Optout>();

                        //Boot
                        GameObject Boot = new GameObject("Boot ");
                        Boot.transform.parent = Instance.transform;
                        Boot.transform.localRotation = Body.localRotation;
                        Boot.transform.localScale = Vector3.one;
                        Boot.transform.localPosition = new Vector3(-3.5427f, 0.26f);
                        SpriteRenderer BootRenderer = Boot.AddComponent<SpriteRenderer>();
                        BootRenderer.sortingLayerName = "Default";
                        BootRenderer.sortingOrder = 1;
                        Boot.AddComponent<RandomCarTextureChildBehaviour>().Sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Van/Green/Boot.png"),
                            ModAPI.LoadSprite("Textures/Van/Red/Boot.png"),
                            ModAPI.LoadSprite("Textures/Van/Gray/Boot.png"),
                        };
                        Boot.layer = LayerMask.NameToLayer("Objects");
                        Boot.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.143940926f,0.824017167f),
                            new Vector2(0.0545118377f,0.820781112f),
                            new Vector2(-0.0831313878f,0.0572116077f),
                            new Vector2(-0.0860362053f,-0.505003631f),
                            new Vector2(-0.14198184f,-0.541833818f),
                            new Vector2(-0.143734932f,-0.6005373f),
                            new Vector2(-0.0865507126f,-0.605960727f),
                            new Vector2(-0.0876985341f,-0.828571439f),
                            new Vector2(0.00454795733f,-0.828571439f),
                            new Vector2(0.00488830358f,0.0432980061f)
                        });
                        Boot.AddComponent<Rigidbody2D>().mass = 0.2f;
                        FixedJoint2D BootFixedJoint = Boot.AddComponent<FixedJoint2D>();
                        BootFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        BootFixedJoint.breakForce = 300f;
                        HingeJoint2D BootHingeJoint = Boot.AddComponent<HingeJoint2D>();
                        BootHingeJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        BootHingeJoint.anchor = new Vector2(0.0993f, 0.782f);
                        BootHingeJoint.breakForce = 5000f;
                        BootHingeJoint.breakTorque = 5000f;
                        BootHingeJoint.useMotor = true;
                        BootHingeJoint.limits = new JointAngleLimits2D
                        {
                            min = 0f,
                            max = 120f
                        };
                        BootHingeJoint.motor = new JointMotor2D
                        {
                            maxMotorTorque = 8f
                        };
                        Boot.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Boot.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Boot.AddComponent<Optout>();

                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot
                        Instance.transform.GetChild(6).gameObject.SetActive(false); //Back door

                        //Hood
                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
                        Bonnet.GetComponent<SpriteRenderer>().sortingOrder = -1;
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 500f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.1526f, 0.1444f);
                        Bonnet.gameObject.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65f,
                            max = 0f
                        };
                        Bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.171428576f,0.18571429f),
                            new Vector2(-0.160942212f,0.1414316f),
                            new Vector2(0.0969347954f,-0.109360337f),
                            new Vector2(0.116916776f,-0.184665635f),
                            new Vector2(0.171428576f,-0.18571429f),
                            new Vector2(0.170380011f,-0.130944937f),
                            new Vector2(-0.153620124f,0.183616981f)
                        });

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().pathCount = 1;
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.00447686017f,0.971428573f),
                            new Vector2(-0.885714293f,0.971428573f),
                            new Vector2(-0.762458324f,0.0997464806f),
                            new Vector2(-0.494045854f,-0.542443454f),
                            new Vector2(-0.146143794f,-0.47774595f),
                            new Vector2(0.0845730305f,-0.573304892f),
                            new Vector2(0.335408211f,-0.957002819f),
                            new Vector2(0.885714293f,-0.971428573f),
                            new Vector2(0.866067648f,0.30971542f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.839f, -0.207f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 6;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Engine
                        Transform Engine = Instance.transform.GetChild(7);
                        foreach (Collider2D EngineCollider in Engine.GetComponents<Collider2D>())
                        {
                            NoCollideWithBodyPart NoCollideWithBodyPart = Engine.gameObject.AddComponent<NoCollideWithBodyPart>();
                            NoCollideWithBodyPart.BodyPart = LimbBehaviour.BodyPart.Legs;
                            NoCollideWithBodyPart.Collider = EngineCollider;
                        }

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        UnityEngine.Object.Destroy(GasTank.GetComponent<BoxCollider2D>());
                        GasTank.gameObject.AddComponent<BoxCollider2D>();
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Background";
                        foreach (Collider2D GasTankCollider in GasTank.GetComponents<Collider2D>())
                        {
                            NoCollideWithBodyPart NoCollideWithBodyPart = GasTank.gameObject.AddComponent<NoCollideWithBodyPart>();
                            NoCollideWithBodyPart.BodyPart = LimbBehaviour.BodyPart.Legs;
                            NoCollideWithBodyPart.Collider = GasTankCollider;
                        }

                        //Not colliding objects with wheels in a certain area
                        NoCollideWithObjectsInTrigger NoCollideWithObjectsInTrigger = Body.gameObject.AddComponent<NoCollideWithObjectsInTrigger>();
                        NoCollideWithObjectsInTrigger.Colliders = new Collider2D[]
                        {
                            Wheels[0].connectedBody.GetComponent<CircleCollider2D>(),
                            Wheels[1].connectedBody.GetComponent<CircleCollider2D>()
                        };
                        NoCollideWithObjectsInTrigger.Trigger = BodyTrigger;

                        //Serialization
                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.gameObject.transform.localPosition = new Vector2(2.0715f, -0.8717f);
                            Wheels[1].connectedBody.gameObject.transform.localPosition = new Vector2(-1.9853f, -0.8717f);
                            FrontDoor.localPosition = new Vector3(2.2857f, 0.14285f);
                            Bonnet.localPosition = new Vector3(3.453f, 0.2398f);
                            GasTank.localPosition = new Vector3(-0.771f, -0.699f);
                            GasTank.localScale = new Vector3(0.8f, 0.8f);
                            Engine.localPosition = new Vector3(3.132f, -0.273f);
                            Engine.localScale = new Vector3(0.8f, 0.8f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = SideDoor;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Boot;
                        };
                        CarSerialisationHelper.AfterSpawn = () =>
                        {
                            //Inside settings
                            InsideBehaviour Inside = Body.gameObject.GetOrAddComponent<InsideBehaviour>();
                            Inside.ForegroundRenderers = new SpriteRenderer[]
                            {
                                BackBodyTopRenderer,
                                SideDoorRenderer,
                                FrontDoor.GetComponent<SpriteRenderer>()
                            };
                            if (Inside.ShowInside)
                            {
                                Inside.UpdateInside();
                            }

                            ActOnJointBreak FrontDoorActOnJointBreak = FrontDoor.gameObject.AddComponent<ActOnJointBreak>();
                            FrontDoorActOnJointBreak.Joint = FrontDoor.GetComponent<FixedJoint2D>();
                            FrontDoorActOnJointBreak.OnBreak.AddListener(() =>
                            {
                                if (Inside.ShowInside)
                                {
                                    FrontDoor.GetComponent<SpriteRenderer>().enabled = true;
                                }
                                List<SpriteRenderer> List = Inside.ForegroundRenderers.ToList();
                                List.Remove(FrontDoor.GetComponent<SpriteRenderer>());
                                Inside.ForegroundRenderers = List.ToArray();
                            });

                            ActOnJointBreak SideDoorActOnJointBreak = FrontDoor.gameObject.AddComponent<ActOnJointBreak>();
                            SideDoorActOnJointBreak.Joint = SideDoorSlider;
                            SideDoorActOnJointBreak.OnBreak.AddListener(() =>
                            {
                                if (Inside.ShowInside)
                                {
                                    SideDoorRenderer.enabled = true;
                                }
                                List<SpriteRenderer> List = Inside.ForegroundRenderers.ToList();
                                List.Remove(SideDoorRenderer);
                                Inside.ForegroundRenderers = List.ToArray();
                            });

                            SideDoorSlider.angle = 0f;
                            if (SideDoorSlider.limitState == JointLimitState2D.UpperLimit)
                            {
                                Opened = true;
                                Motor.motorSpeed = SideDoorSpeed * -1f;
                                SideDoorSlider.motor = Motor;
                            }
                        };
                        CarSerialisationHelper.Right = () =>
                        {
                            Instance.transform.localScale = new Vector2(Scale, Scale);
                        };
                        CarSerialisationHelper.Left = () =>
                        {
                            Instance.transform.localScale = new Vector2(-Scale, Scale);
                            SideDoorSpeed *= -1f;
                            Motor.motorSpeed = SideDoorSpeed;
                            SideDoorSlider.motor = Motor;
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Prison Bus
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Prison Bus",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Bus for prisoners. 5 seats for prisoners and one for the bus driver. (School Bus)",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Prison Bus.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/Prison Bus/Wheel.png");

                        //Car speed
                        float MotorSpeed = -2500f;

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 4;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 15f;
                        BodyPhysical.InitialMass = 15f;
                        BodyPhysical.rigidbody.mass = 15f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 8;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(1.92646384f,-1.58190119f),
                            new Vector2(3.52617693f,-1.5763278f),
                            new Vector2(5.24371719f,-1.53138149f),
                            new Vector2(5.18349409f,-0.117641285f),
                            new Vector2(4.78568268f,-0.484850824f),
                            new Vector2(3.66302395f,-0.189893663f),
                            new Vector2(3.72342324f,0.151489004f),
                            new Vector2(3.52707458f,0.168475151f),
                            new Vector2(3.52522159f,1.32854402f),
                            new Vector2(3.6660111f,1.3342551f),
                            new Vector2(3.67170358f,1.87278271f),
                            new Vector2(-5.18504477f,1.87142861f),
                            new Vector2(-5.18516302f,-1.21433306f),
                            new Vector2(1.2360146f,-1.21298563f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(3.38833976f,-0.645581484f),
                            new Vector2(-5.07106829f,-0.639577091f),
                            new Vector2(-5.07117319f,1.78593385f),
                            new Vector2(3.38739538f,1.78507793f)
                        });
                        BodyCollider.SetPath(2, new Vector2[]
                        {
                            new Vector2(2.61324835f,-0.631867111f),
                            new Vector2(2.16114497f,-0.62764585f),
                            new Vector2(2.08881974f,-0.348538339f),
                            new Vector2(1.97265661f,0.948051691f),
                            new Vector2(2.177248f,0.984106719f),
                            new Vector2(2.23954582f,-0.232111514f),
                            new Vector2(2.79276919f,-0.213284463f),
                            new Vector2(2.81643343f,-0.388266206f)
                        });
                        BodyCollider.SetPath(3, new Vector2[]
                        {
                            new Vector2(0.50496465f,-0.635962844f),
                            new Vector2(0.952902615f,-0.640090466f),
                            new Vector2(1.14086306f,-0.0977437496f),
                            new Vector2(0.575563729f,-0.144512981f),
                            new Vector2(0.490500569f,1.10006118f),
                            new Vector2(0.302696466f,1.09605312f)
                        });
                        BodyCollider.SetPath(4, new Vector2[]
                        {
                            new Vector2(-0.969423175f,1.08286488f),
                            new Vector2(-0.781084239f,1.08930707f),
                            new Vector2(-0.723053932f,-0.143876553f),
                            new Vector2(-0.162997231f,-0.105752185f),
                            new Vector2(-0.315603375f,-0.627364576f),
                            new Vector2(-0.77371496f,-0.629081726f)
                        });
                        BodyCollider.SetPath(5, new Vector2[]
                        {
                            new Vector2(-4.52764797f,-0.131507576f),
                            new Vector2(-4.03346157f,-0.0852921903f),
                            new Vector2(-4.1638484f,-0.61765492f),
                            new Vector2(-4.64387417f,-0.615851581f),
                            new Vector2(-4.84482431f,1.09537566f),
                            new Vector2(-4.66132975f,1.10181797f)
                        });
                        BodyCollider.SetPath(6, new Vector2[]
                        {
                            new Vector2(-2.75068712f,-0.0860753059f),
                            new Vector2(-2.91516495f,-0.622458875f),
                            new Vector2(-3.33450603f,-0.623334527f),
                            new Vector2(-3.54108906f,1.08608115f),
                            new Vector2(-3.36196423f,1.11348784f),
                            new Vector2(-3.25724769f,-0.153440952f)
                        });
                        BodyCollider.SetPath(7, new Vector2[]
                        {
                            new Vector2(-1.58723354f,-0.618265212f),
                            new Vector2(-2.04524517f,-0.634842873f),
                            new Vector2(-2.24795341f,1.10597408f),
                            new Vector2(-2.05243897f,1.09409142f),
                            new Vector2(-1.98968267f,-0.156665444f),
                            new Vector2(-1.45910406f,-0.102106571f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Prison Bus/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Prison Bus/RightDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Prison Bus/LeftDoor.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Prison Bus/Bonnet.png"),
                                Background = ModAPI.LoadSprite("Textures/Prison Bus/Background.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Prison Bus/School Bus/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Prison Bus/School Bus/RightDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Prison Bus/School Bus/LeftDoor.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Prison Bus/School Bus/Bonnet.png"),
                                Background = ModAPI.LoadSprite("Textures/Prison Bus/School Bus/Background.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero;

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].connectedBody.transform.localScale = new Vector2(1.0f, 1.0f);
                        Wheels[0].anchor = new Vector2(4.2282f, -1.2868f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.5857143f;

                        Wheels[1].connectedBody.transform.localScale = new Vector2(1.0f, 1.0f);
                        Wheels[1].anchor = new Vector2(-2.428f, -1.2868f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.5857143f;

                        //Hood
                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 500f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.695f, 0.069f);
                        Bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65f,
                            max = 0f
                        };
                        Bonnet.gameObject.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.402864933f,0.0502298474f),
                            new Vector2(-0.728571415f,0.114285715f),
                            new Vector2(-0.730054617f,0.0303720124f),
                            new Vector2(0.378301144f,-0.0336759128f),
                            new Vector2(0.728571415f,-0.114285715f),
                            new Vector2(0.724121571f,-0.0299833417f)
                        });

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.317f, 0f);
                        FrontDoor.gameObject.AddComponent<BoxCollider2D>().size = new Vector3(0.6571429f, 2.514286f);
                        UnityEngine.Object.Destroy(FrontDoor.GetComponent<PolygonCollider2D>());

                        //Mirror
                        GameObject Mirror = new GameObject("Mirror");
                        Mirror.transform.parent = Instance.transform;
                        Mirror.transform.localRotation = Body.localRotation;
                        Mirror.transform.localScale = Vector3.one;
                        Mirror.transform.localPosition = new Vector3(3.395f, 1.011f);
                        SpriteRenderer MirrorRenderer = Mirror.AddComponent<SpriteRenderer>();
                        MirrorRenderer.sprite = ModAPI.LoadSprite("Textures/Prison Bus/Mirror.png");
                        MirrorRenderer.sortingLayerName = "Foreground";
                        MirrorRenderer.sortingOrder = 6;
                        Mirror.layer = LayerMask.NameToLayer("Objects");
                        Mirror.AddComponent<PolygonCollider2D>();
                        Mirror.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D MirrorFixedJoint1 = Mirror.AddComponent<FixedJoint2D>();
                        MirrorFixedJoint1.connectedBody = FrontDoor.GetComponent<Rigidbody2D>();
                        MirrorFixedJoint1.breakForce = 500f;
                        MirrorFixedJoint1.anchor = new Vector2(-0.041f, 0.3f);
                        FixedJoint2D MirrorFixedJoint2 = Mirror.AddComponent<FixedJoint2D>();
                        MirrorFixedJoint2.connectedBody = FrontDoor.GetComponent<Rigidbody2D>();
                        MirrorFixedJoint2.frequency = 8f;
                        MirrorFixedJoint2.dampingRatio = 1f;
                        MirrorFixedJoint2.breakForce = 1500f;
                        MirrorFixedJoint2.anchor = new Vector2(-0.041f, -0.298f);
                        Mirror.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");
                        Mirror.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Mirror.AddComponent<Optout>();

                        //Back door
                        Transform BackDoor = Instance.transform.GetChild(6);
                        BackDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.326f, 0f);
                        BackDoor.gameObject.AddComponent<BoxCollider2D>().size = new Vector3(0.6857143f, 2.514286f);
                        UnityEngine.Object.Destroy(BackDoor.GetComponent<PolygonCollider2D>());

                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot

                        //Serialization
                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(4.2282f, -1.2868f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(-2.428f, -1.2868f);
                            Bonnet.localPosition = new Vector3(4.45f, 0.006f);
                            FrontDoor.localPosition = new Vector3(3.086f, 0.128f);
                            BackDoor.localPosition = new Vector3(2.415f, 0.128f);
                            Instance.transform.GetChild(7).localPosition = new Vector3(4.235f, -0.375f);
                            Instance.transform.GetChild(8).localPosition = new Vector3(1.608f, -0.941f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Mirror;
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
#endregion
#region Light Vehicles
        #region Mitsubishi Fuzion
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Mitsubishi Fuzion",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "An ordinary family car with a trunk in the back.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Mitsubishi Fuzion.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite Wheel = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Wheel.png");

                        AudioClip Horn = ModAPI.LoadSound("Sfx/Truck Horn.mp3");

                        //Car speed
                        float MotorSpeed = -3000f;

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 18f;
                        BodyPhysical.InitialMass = 18f;
                        BodyPhysical.rigidbody.mass = 18f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 3;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(1.8311367f,0.415661991f),
                            new Vector2(0.968008041f,1.1301384f),
                            new Vector2(0.795776367f,1.24012637f),
                            new Vector2(0.511253834f,1.28947353f),
                            new Vector2(-2.49411678f,1.29037333f),
                            new Vector2(-2.84219599f,1.18727517f),
                            new Vector2(-3.00706768f,1.04164135f),
                            new Vector2(-2.57092571f,1.17816389f),
                            new Vector2(0.715112209f,1.15423727f),
                            new Vector2(1.58381796f,0.345743299f),
                            new Vector2(1.58915329f,-0.654538095f),
                            new Vector2(-1.05548954f,-0.656983256f),
                            new Vector2(-1.18567896f,-0.657103658f),
                            new Vector2(-1.36712551f,-0.386009574f),
                            new Vector2(-1.69235516f,-0.307964325f),
                            new Vector2(-1.99795437f,-0.419532329f),
                            new Vector2(-2.19279766f,-0.658034801f),
                            new Vector2(-2.95733929f,-0.658741713f),
                            new Vector2(-2.76852036f,-0.938400805f),
                            new Vector2(2.86924076f,-0.920236111f),
                            new Vector2(3.13775158f,-0.703185081f),
                            new Vector2(3.06544566f,0.0845822692f),
                            new Vector2(2.12985063f,0.0622216463f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(0.555399418f,-0.281830072f),
                            new Vector2(1.00259542f,-0.250593603f),
                            new Vector2(1.04066038f,-0.369113028f),
                            new Vector2(0.868457794f,-0.644539237f),
                            new Vector2(0.505991936f,-0.643406034f),
                            new Vector2(0.342801571f,-0.464379847f),
                            new Vector2(0.304600716f,0.957809448f),
                            new Vector2(0.458066225f,0.973966599f)
                        });
                        BodyCollider.SetPath(2, new Vector2[]
                        {
                            new Vector2(-0.784033775f,-0.270721316f),
                            new Vector2(-0.363283157f,-0.255542904f),
                            new Vector2(-0.297355175f,-0.387181163f),
                            new Vector2(-0.463256359f,-0.641720235f),
                            new Vector2(-0.835574627f,-0.6439448f),
                            new Vector2(-0.986845493f,-0.461124897f),
                            new Vector2(-1.04435968f,0.993759036f),
                            new Vector2(-0.873331547f,0.963154554f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().OnAfterChange = new UnityEvent();
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Red/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Red/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Red/Front Door.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Red/Back Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Red/Bonnet.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Green/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Green/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Green/Front Door.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Green/Back Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Green/Bonnet.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Gray/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Gray/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Gray/Front Door.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Gray/Back Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Gray/Bonnet.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero; //Car bottom position

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].anchor = new Vector2(-1.6859f, -0.8436f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = Wheel;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        Wheels[1].anchor = new Vector2(2.2286f, -0.8436f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = Wheel;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        //Boot
                        GameObject Boot = new GameObject("NotBootDuplicate");
                        Boot.transform.parent = Instance.transform;
                        Boot.transform.localRotation = Body.localRotation;
                        Boot.transform.localScale = Vector3.one;
                        Boot.transform.localPosition = new Vector3(-3.015f, 0.199f);
                        SpriteRenderer BootRenderer = Boot.AddComponent<SpriteRenderer>();
                        BootRenderer.sortingLayerName = "Default";
                        BootRenderer.sortingOrder = 1;
                        Boot.AddComponent<RandomCarTextureChildBehaviour>().Sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Red/Boot.png"),
                            ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Green/Boot.png"),
                            ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/Gray/Boot.png")
                        };
                        Boot.layer = LayerMask.NameToLayer("Objects");
                        Boot.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.00437899679f,0.858460844f),
                            new Vector2(-0.112232782f,0.393628418f),
                            new Vector2(-0.112160683f,-0.173596323f),
                            new Vector2(-0.0583295822f,-0.359005928f),
                            new Vector2(-0.118124008f,-0.404360294f),
                            new Vector2(-0.116869926f,-0.455391884f),
                            new Vector2(-0.0596796051f,-0.461553484f),
                            new Vector2(0.0298156999f,-0.858190596f),
                            new Vector2(0.114204027f,-0.854316652f),
                            new Vector2(-0.0286197662f,-0.165166542f),
                            new Vector2(-0.0252494812f,0.387189329f),
                            new Vector2(0.0883775726f,0.858190596f)
                        });
                        Boot.AddComponent<Rigidbody2D>().mass = 0.2f;
                        FixedJoint2D BootFixedJoint = Boot.AddComponent<FixedJoint2D>();
                        BootFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        BootFixedJoint.breakForce = 300f;
                        HingeJoint2D BootHingeJoint = Boot.AddComponent<HingeJoint2D>();
                        BootHingeJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        BootHingeJoint.anchor = new Vector2(0.041f, 0.807f);
                        BootHingeJoint.breakForce = 5000f;
                        BootHingeJoint.breakTorque = 5000f;
                        BootHingeJoint.useMotor = true;
                        BootHingeJoint.limits = new JointAngleLimits2D
                        {
                            min = 0f,
                            max = 120f
                        };
                        BootHingeJoint.motor = new JointMotor2D
                        {
                            maxMotorTorque = 8f
                        };
                        Boot.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Boot.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Boot.AddComponent<Optout>();

                        //Steering wheel
                        GameObject SteeringWheel = new GameObject("Steering Wheel");
                        SteeringWheel.transform.parent = Instance.transform;
                        SteeringWheel.transform.localRotation = Body.localRotation;
                        SteeringWheel.transform.localScale = Vector3.one;
                        SteeringWheel.transform.localPosition = new Vector3(1.286f, 0.229f);
                        SpriteRenderer SteeringWheelRenderer = SteeringWheel.AddComponent<SpriteRenderer>();
                        SteeringWheelRenderer.sprite = ModAPI.LoadSprite("Textures/Mitsubishi Fuzion/SteeringWheel.png");
                        SteeringWheelRenderer.sortingLayerName = "Default";
                        SteeringWheelRenderer.sortingOrder = 1;
                        SteeringWheel.layer = 10;
                        SteeringWheel.AddComponent<Rigidbody2D>().mass = 0.2f;
                        FixedJoint2D SteeringWheelFixedJoint = SteeringWheel.AddComponent<FixedJoint2D>();
                        SteeringWheelFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        SteeringWheelFixedJoint.breakForce = 152000f;
                        AudioSource HornSource = SteeringWheel.AddComponent<AudioSource>();
                        HornSource.spatialBlend = 1f;
                        HornSource.minDistance = 100f;
                        SteeringWheel.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            HornSource.PlayOneShot(Horn);
                        };
                        SteeringWheel.AddComponent<PolygonCollider2D>().pathCount = 1;
                        SteeringWheel.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.0463324487f,0.22857143f),
                            new Vector2(-0.100000001f,-0.22857143f),
                            new Vector2(-0.0420635641f,-0.226863876f),
                            new Vector2(0.100000001f,0.22857143f),
                        });
                        SteeringWheel.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        SteeringWheel.AddComponent<AudioSourceTimeScaleBehaviour>();
                        SteeringWheel.AddComponent<Optout>();

                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot

                        //Hood
                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";
                        Bonnet.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 500f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.57f, 0.126f);
                        Bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65f,
                            max = 0f
                        };
                        Bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.60437727f,-0.121885993f),
                            new Vector2(0.337554693f,0.0142583549f),
                            new Vector2(-0.600000024f,0.157142863f),
                            new Vector2(-0.600000024f,0.096889019f),
                            new Vector2(0.228675604f,-0.0248475224f),
                            new Vector2(0.587595582f,-0.163000911f)
                        });

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.356814861f,1.04376006f),
                            new Vector2(-0.828571439f,1.04285717f),
                            new Vector2(-0.668408036f,0.0750686601f),
                            new Vector2(-0.600356102f,-0.291737825f),
                            new Vector2(-0.608723521f,-1.03557706f),
                            new Vector2(0.741209626f,-1.04649723f),
                            new Vector2(0.8264184f,-0.804066181f),
                            new Vector2(0.825346112f,0.192139477f),
                            new Vector2(0.141904846f,0.893809617f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.8f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Back door
                        Transform BackDoor = Instance.transform.GetChild(6);
                        BackDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.668017149f,1.03913713f),
                            new Vector2(-0.842734814f,0.583417773f),
                            new Vector2(-0.924851179f,0.0686337948f),
                            new Vector2(-0.924851179f,-0.389588505f),
                            new Vector2(-0.674237728f,-0.471336842f),
                            new Vector2(-0.46423769f,-0.661635756f),
                            new Vector2(-0.343606055f,-1.04285717f),
                            new Vector2(0.928571463f,-1.04285717f),
                            new Vector2(0.929633141f,-0.271559596f),
                            new Vector2(0.839291334f,0.0827427953f),
                            new Vector2(0.69896549f,1.04285717f)
                        });
                        BackDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.8583f, 0f);
                        BackDoor.GetComponent<SpriteRenderer>().sortingOrder = 6;
                        BackDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.gameObject.AddComponent<BoxCollider2D>();
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        //Serialization
                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(-1.6859f, -0.8436f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(2.2286f, -0.8436f);
                            Bonnet.localPosition = new Vector3(2.441f, 0.243f);
                            FrontDoor.localPosition = new Vector3(0.78569f, 0.15715f);
                            BackDoor.localPosition = new Vector3(-0.7149f, 0.157f);
                            GasTank.localScale = new Vector3(0.9f, 0.9f);
                            GasTank.localPosition = new Vector3(-0.795f, -0.633f);

                            //Engine
                            Transform Engine = Instance.transform.GetChild(7);
                            Engine.localScale = new Vector3(0.9f, 0.9f);
                            Engine.localPosition = new Vector3(2.268f, -0.263f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Boot;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = SteeringWheel;
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Toyota Revo 2000
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Toyota Revo 2000",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "A family car, but only faster. And heavier.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Toyota Revo 2000.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/Toyota Revo 2000/Wheel.png");

                        AudioClip Horn = ModAPI.LoadSound("Sfx/Truck Horn.mp3");

                        //Car speed
                        float MotorSpeed = -3500f;

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 22f;
                        BodyPhysical.InitialMass = 22f;
                        BodyPhysical.rigidbody.mass = 22f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 3;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(2.82170081f,-1.07129943f),
                            new Vector2(2.99307418f,-0.977823913f),
                            new Vector2(3.0338974f,-0.503300369f),
                            new Vector2(2.9792819f,-0.49780944f),
                            new Vector2(2.91361332f,-0.0426378176f),
                            new Vector2(1.94697714f,0.0888080001f),
                            new Vector2(1.88824177f,0.307202131f),
                            new Vector2(1.59909379f,0.375730336f),
                            new Vector2(1.59909379f,0.375730336f),
                            new Vector2(0.648338735f,1.15740693f),
                            new Vector2(-2.43492842f,1.16357052f),
                            new Vector2(-3.03610945f,1.06566489f),
                            new Vector2(-2.88893104f,0.988269448f),
                            new Vector2(-2.7911129f,0.822994709f),
                            new Vector2(-2.38681579f,1.05177855f),
                            new Vector2(0.721058667f,1.06867301f),
                            new Vector2(1.51006806f,0.322280705f),
                            new Vector2(1.68957508f,-0.368269026f),
                            new Vector2(1.19214964f,-0.781287909f),
                            new Vector2(-1.08993983f,-0.777921438f),
                            new Vector2(-1.23253095f,-0.541026056f),
                            new Vector2(-1.59278977f,-0.439583302f),
                            new Vector2(-1.90182102f,-0.511364162f),
                            new Vector2(-2.07388735f,-0.780359983f),
                            new Vector2(-2.76807213f,-0.782760501f),
                            new Vector2(-2.753649f,-0.498915195f),
                            new Vector2(-2.89995861f,-0.496296287f),
                            new Vector2(-2.9929285f,-0.540768802f),
                            new Vector2(-3.00796342f,-0.783590019f),
                            new Vector2(-2.9944315f,-0.915023088f),
                            new Vector2(-2.83554077f,-1.06771421f),
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(0.203713924f,0.815220952f),
                            new Vector2(0.237790942f,-0.556316495f),
                            new Vector2(0.404628724f,-0.770644248f),
                            new Vector2(0.765821576f,-0.76600045f),
                            new Vector2(0.942205429f,-0.54088521f),
                            new Vector2(0.892956078f,-0.394966811f),
                            new Vector2(0.452523977f,-0.405065626f),
                            new Vector2(0.383435011f,0.830627441f)
                        });
                        BodyCollider.SetPath(2, new Vector2[]
                        {
                            new Vector2(-1.17651582f,0.822243929f),
                            new Vector2(-1.00487411f,0.837704182f),
                            new Vector2(-0.91116184f,-0.401181817f),
                            new Vector2(-0.489073157f,-0.373962134f),
                            new Vector2(-0.424105376f,-0.478737235f),
                            new Vector2(-0.610542059f,-0.761080623f),
                            new Vector2(-0.992741406f,-0.76062578f),
                            new Vector2(-1.08560658f,-0.468387604f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().OnAfterChange = new UnityEvent();
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Toyota Revo 2000/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Toyota Revo 2000/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Toyota Revo 2000/Front Door.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Toyota Revo 2000/Back Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Toyota Revo 2000/Bonnet.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero; //Car bottom position

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].anchor = new Vector2(-1.5855f, -0.9725f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        Wheels[1].anchor = new Vector2(1.9003f, -0.9725f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4428571f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        //Boot
                        GameObject Boot = new GameObject("NotBootDuplicate");
                        Boot.transform.parent = Instance.transform;
                        Boot.transform.localRotation = Body.localRotation;
                        Boot.transform.localScale = Vector3.one;
                        Boot.transform.localPosition = new Vector3(-2.869f, 0.152f);
                        SpriteRenderer BootRenderer = Boot.AddComponent<SpriteRenderer>();
                        BootRenderer.sprite = ModAPI.LoadSprite("Textures/Toyota Revo 2000/Boot.png");
                        BootRenderer.sortingLayerName = "Default";
                        BootRenderer.sortingOrder = 1;
                        Boot.layer = LayerMask.NameToLayer("Objects");
                        Boot.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.0726545155f,0.654921472f),
                            new Vector2(-0.0372638553f,0.142932758f),
                            new Vector2(-0.0452077389f,-0.362253457f),
                            new Vector2(-0.0984768867f,-0.397991061f),
                            new Vector2(-0.102345467f,-0.463537633f),
                            new Vector2(-0.0394191742f,-0.459718943f),
                            new Vector2(-0.0126733631f,-0.659939647f),
                            new Vector2(0.074779436f,-0.657142878f),
                            new Vector2(0.0436501503f,-0.482729137f),
                            new Vector2(0.0535264015f,0.125624165f),
                            new Vector2(0.157142863f,0.657142878f)
                        });
                        Boot.AddComponent<Rigidbody2D>().mass = 0.2f;
                        FixedJoint2D BootFixedJoint = Boot.AddComponent<FixedJoint2D>();
                        BootFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        BootFixedJoint.breakForce = 300f;
                        HingeJoint2D BootHingeJoint = Boot.AddComponent<HingeJoint2D>();
                        BootHingeJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        BootHingeJoint.anchor = new Vector2(0.115f, 0.608f);
                        BootHingeJoint.breakForce = 5000f;
                        BootHingeJoint.breakTorque = 5000f;
                        BootHingeJoint.useMotor = true;
                        BootHingeJoint.limits = new JointAngleLimits2D()
                        {
                            min = 0f,
                            max = 120f
                        };
                        BootHingeJoint.motor = new JointMotor2D
                        {
                            maxMotorTorque = 8f
                        };
                        Boot.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Boot.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Boot.AddComponent<Optout>();

                        //Steering wheel
                        GameObject SteeringWheel = new GameObject("Steering Wheel");
                        SteeringWheel.transform.parent = Instance.transform;
                        SteeringWheel.transform.localRotation = Body.localRotation;
                        SteeringWheel.transform.localScale = Vector3.one;
                        SteeringWheel.transform.localPosition = new Vector3(1.0719f, 0.2718f);
                        SpriteRenderer SteeringWheelRenderer = SteeringWheel.AddComponent<SpriteRenderer>();
                        SteeringWheelRenderer.sprite = ModAPI.LoadSprite("Textures/Toyota Revo 2000/SteeringWheel.png");
                        SteeringWheelRenderer.sortingLayerName = "Default";
                        SteeringWheelRenderer.sortingOrder = 1;
                        SteeringWheel.layer = 10;
                        SteeringWheel.AddComponent<Rigidbody2D>().mass = 0.2f;
                        PolygonCollider2D SteeringWheelCollider = SteeringWheel.AddComponent<PolygonCollider2D>();
                        SteeringWheelCollider.pathCount = 1;
                        SteeringWheelCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(0.0463324487f,0.22857143f),
                            new Vector2(-0.100000001f,-0.22857143f),
                            new Vector2(-0.0420635641f,-0.226863876f),
                            new Vector2(0.100000001f,0.22857143f),
                        });
                        FixedJoint2D SteeringWheelFixedJoint = SteeringWheel.AddComponent<FixedJoint2D>();
                        SteeringWheelFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        SteeringWheelFixedJoint.breakForce = 152000f;
                        AudioSource HornSource = SteeringWheel.AddComponent<AudioSource>();
                        HornSource.spatialBlend = 1f;
                        HornSource.minDistance = 100f;
                        SteeringWheel.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            HornSource.PlayOneShot(Horn);
                        };
                        SteeringWheel.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        SteeringWheel.AddComponent<AudioSourceTimeScaleBehaviour>();
                        SteeringWheel.AddComponent<Optout>();

                        //Hood
                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";
                        Bonnet.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 500f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.47f, 0.152f);
                        Bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65f,
                            max = 0f
                        };
                        Bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.0695784092f,0.0762012303f),
                            new Vector2(-0.5f,0.18571429f),
                            new Vector2(-0.496985435f,0.131719917f),
                            new Vector2(0.0538766533f,0.00996474177f),
                            new Vector2(0.5f,-0.18571429f),
                            new Vector2(0.495478153f,-0.128973201f)
                        });

                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-1.02857149f,1.05714285f),
                            new Vector2(-0.841279268f,-0.0694906339f),
                            new Vector2(-0.82893151f,-1.05714285f),
                            new Vector2(0.39918825f,-1.05714285f),
                            new Vector2(0.475878298f,-0.735037684f),
                            new Vector2(0.688392699f,-0.448924214f),
                            new Vector2(1.03668344f,-0.386840373f),
                            new Vector2(0.95826906f,0.191888198f),
                            new Vector2(0.655377567f,0.330031157f),
                            new Vector2(-0.0209161043f,0.992904544f),
                            new Vector2(-0.209909618f,1.05714285f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.98f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Back door
                        Transform BackDoor = Instance.transform.GetChild(6);
                        BackDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.569396734f,1.05456209f),
                            new Vector2(-0.798392713f,-0.0613634586f),
                            new Vector2(-0.805161536f,-0.371977419f),
                            new Vector2(-0.573842764f,-0.40810594f),
                            new Vector2(-0.256559134f,-0.59445405f),
                            new Vector2(-0.139296308f,-1.05714285f),
                            new Vector2(0.800000012f,-1.05714285f),
                            new Vector2(0.792539775f,-0.0799939334f),
                            new Vector2(0.571350217f,1.04968262f)
                        });
                        BackDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.745f, 0f);
                        BackDoor.GetComponent<SpriteRenderer>().sortingOrder = 6;
                        BackDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.gameObject.AddComponent<BoxCollider2D>();
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        //Serialization
                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(-1.5855f, -0.9725f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(1.9003f, -0.9725f);
                            Bonnet.localPosition = new Vector3(2.3837f, 0.1107f);
                            FrontDoor.localPosition = new Vector3(0.8286f, 0.04277f);
                            BackDoor.localPosition = new Vector3(-0.7716f, 0.0429f);
                            GasTank.localScale = new Vector3(0.9f, 0.9f);
                            GasTank.localPosition = new Vector3(-0.795f, -0.633f);

                            //Engine
                            Transform Engine = Instance.transform.GetChild(7);
                            Engine.localScale = new Vector3(0.9f, 0.9f);
                            Engine.localPosition = new Vector3(2.268f, -0.263f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Boot;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = SteeringWheel;
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Vintage Pickup
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Vintage Pickup",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Vintage truck. Carry whatever you want in the back. And please close the hatch.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Vintage Pickup.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/Vintage Pickup/Wheel.png");

                        //Car speed
                        float MotorSpeed = -2500f;

                        Instance.transform.GetChild(4).gameObject.SetActive(false);
                        Instance.transform.GetChild(6).gameObject.SetActive(false);

                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 15f;
                        BodyPhysical.InitialMass = 15f;
                        BodyPhysical.rigidbody.mass = 15f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 2;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(2.85526419f, -0.930820346f),
                            new Vector2(3.00170135f, -0.598134518f),
                            new Vector2(3.00502825f, 0.0356935412f),
                            new Vector2(2.69627357f, -0.163579881f),
                            new Vector2(1.63785851f, -0.0129931197f),
                            new Vector2(1.66480207f, 0.305818886f),
                            new Vector2(1.14037788f, 1.23816311f),
                            new Vector2(-0.149903193f, 1.23849773f),
                            new Vector2(-0.213276297f, 0.215841651f),
                            new Vector2(-0.22051996f, -0.327417016f),
                            new Vector2(-3.00023508f, -0.329512477f),
                            new Vector2(-2.99662709f, -0.668191552f),
                            new Vector2(-2.50924659f, -0.924483538f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(0.10535197f, 1.13346052f),
                            new Vector2(1.02330375f, 1.1297667f),
                            new Vector2(1.4846493f, 0.295497179f),
                            new Vector2(1.4862591f, -0.75677067f),
                            new Vector2(0.635522127f, -0.753368855f),
                            new Vector2(0.634070039f, -0.562847376f),
                            new Vector2(0.782717824f, -0.502800941f),
                            new Vector2(0.725698471f, -0.324557215f),
                            new Vector2(0.322816044f, -0.324059904f),
                            new Vector2(0.210455745f, 0.897625625f),
                            new Vector2(0.0867402256f, 0.886726975f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().OnAfterChange = new UnityEvent();
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Vintage Pickup/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Vintage Pickup/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Vintage Pickup/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Vintage Pickup/Bonnet.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Vintage Pickup/Green/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Vintage Pickup/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Vintage Pickup/Green/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Vintage Pickup/Green/Bonnet.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Vintage Pickup/Brown/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Vintage Pickup/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Vintage Pickup/Brown/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Vintage Pickup/Brown/Bonnet.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero; //Car bottom position

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].suspension = new JointSuspension2D()
                        {
                            frequency = 5f,
                            angle = 90f,
                            dampingRatio = 0.7f
                        };
                        Wheels[0].anchor = new Vector2(2.3285f, -0.8852f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.3857143f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        Wheels[1].suspension = new JointSuspension2D()
                        {
                            frequency = 5f,
                            angle = 90f,
                            dampingRatio = 0.7f
                        };
                        Wheels[1].anchor = new Vector2(-1.871f, -0.8852f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.3857143f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        //Container upper top
                        GameObject BodyTwoSPart = new GameObject("Background");
                        BodyTwoSPart.transform.parent = Body;
                        BodyTwoSPart.transform.localRotation = Quaternion.identity;
                        BodyTwoSPart.transform.localScale = Vector3.one;
                        BodyTwoSPart.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer bodyBackgroundRenderer = BodyTwoSPart.AddComponent<SpriteRenderer>();
                        BodyTwoSPart.AddComponent<RandomCarTextureChildBehaviour>().Sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Vintage Pickup/Body2.png"),
                            ModAPI.LoadSprite("Textures/Vintage Pickup/Green/Body2.png"),
                            ModAPI.LoadSprite("Textures/Vintage Pickup/Brown/Body2.png")
                        };
                        bodyBackgroundRenderer.sortingOrder = 7;
                        bodyBackgroundRenderer.sortingLayerName = "Foreground";
                        BodyTwoSPart.AddComponent<Optout>();

                        //Hatch
                        GameObject Hatch = new GameObject("Hatch");
                        Hatch.transform.parent = Instance.transform;
                        Hatch.transform.localRotation = Body.localRotation;
                        Hatch.transform.localScale = Vector3.one;
                        Hatch.transform.localPosition = new Vector3(-2.967f, -0.157f);
                        SpriteRenderer HatchRenderer = Hatch.AddComponent<SpriteRenderer>();
                        HatchRenderer.sortingLayerName = "Default";
                        HatchRenderer.sortingOrder = 1;
                        Hatch.AddComponent<RandomCarTextureChildBehaviour>().Sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Vintage Pickup/Trunk Hatch.png"),
                            ModAPI.LoadSprite("Textures/Vintage Pickup/Green/Trunk Hatch.png"),
                            ModAPI.LoadSprite("Textures/Vintage Pickup/Brown/Trunk Hatch.png")
                        };
                        Hatch.layer = LayerMask.NameToLayer("Objects");
                        Hatch.AddComponent<Rigidbody2D>().mass = 0.2f;
                        FixedJoint2D HatchFixedJoint = Hatch.AddComponent<FixedJoint2D>();
                        HatchFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        HatchFixedJoint.breakForce = 300f;
                        HingeJoint2D HatchHingeJoint = Hatch.AddComponent<HingeJoint2D>();
                        HatchHingeJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        HatchHingeJoint.anchor = new Vector2(0.01370001f, -0.2154f);
                        HatchHingeJoint.breakForce = 5000f;
                        HatchHingeJoint.breakTorque = 5000f;
                        HatchHingeJoint.useMotor = true;
                        HatchHingeJoint.limits = new JointAngleLimits2D()
                        {
                            min = 0,
                            max = -90
                        };
                        HatchHingeJoint.motor = new JointMotor2D
                        {
                            maxMotorTorque = 15f
                        };
                        Hatch.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.0571428575f,0.257142872f),
                            new Vector2(-0.0284725465f,-0.257142872f),
                            new Vector2(0.0571428575f,-0.257142872f),
                            new Vector2(0.0571428575f,0.257142872f)
                        });
                        Hatch.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Hatch.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Hatch.AddComponent<Optout>();

                        //Hood
                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Bonnet.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 500f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.65f, 0.092f);
                        Bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65f,
                            max = 0f
                        };
                        Bonnet.gameObject.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.671428561f,0.128571436f),
                            new Vector2(-0.669849753f,-0.0130094811f),
                            new Vector2(-0.616640329f,-0.0154280066f),
                            new Vector2(-0.60582912f,0.0690621138f),
                            new Vector2(0.575072527f,-0.0730053633f),
                            new Vector2(0.608204842f,-0.129942149f),
                            new Vector2(0.668270946f,-0.1317292f),
                            new Vector2(0.622482896f,-0.0439820401f)
                        });

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.656511307f,0.967503726f),
                            new Vector2(-0.741649508f,0.427599728f),
                            new Vector2(-0.738792956f,-0.858775973f),
                            new Vector2(-0.626993179f,-0.962233007f),
                            new Vector2(0.630671442f,-0.964072108f),
                            new Vector2(0.736115575f,-0.854466915f),
                            new Vector2(0.735007584f,0.105849594f),
                            new Vector2(0.256511241f,0.963578999f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.698f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 6;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.gameObject.AddComponent<BoxCollider2D>();
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        //Serialization
                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(2.3285f, -0.8852f);
                            Wheels[0].connectedBody.transform.localScale = new Vector2(1.1f, 1.1f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(1.228f, -0.8727002f);
                            Wheels[1].connectedBody.transform.localScale = new Vector2(1.1f, 1.1f);
                            Bonnet.localPosition = new Vector3(2.3f, 0.17f);
                            FrontDoor.localPosition = new Vector3(0.7714076f, 0.1856998f);
                            GasTank.localScale = new Vector3(0.9f, 0.9f);
                            GasTank.localPosition = new Vector3(-0.795f, -0.633f);

                            //Engine
                            Transform Engine = Instance.transform.GetChild(7);
                            Engine.localScale = new Vector3(0.9f, 0.9f);
                            Engine.localPosition = new Vector3(2.25f, -0.355f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Hatch;
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Peel P50
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Peel P50",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "A single small car. Easy to turn over, but very lightweight.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Peel P50.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/Peel P50/Wheel.png");

                        float MotorSpeed = -4000f;

                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 18f;
                        BodyPhysical.InitialMass = 18f;
                        BodyPhysical.rigidbody.mass = 18f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 2;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(0.535120368f, 0.077945888f),
                            new Vector2(0.130013004f, 1.12725377f),
                            new Vector2(-0.0491130278f, 1.19740605f),
                            new Vector2(-0.932607055f, 1.19481397f),
                            new Vector2(-1.05203843f, 1.1012764f),
                            new Vector2(-1.08855712f, 0.104615808f),
                            new Vector2(-1.20020008f, -0.055727005f),
                            new Vector2(-1.17349422f, -0.558544517f),
                            new Vector2(-1.08180511f, -0.814851761f),
                            new Vector2(-0.715750813f, -0.866295457f),
                            new Vector2(0.709959984f, -0.997200012f),
                            new Vector2(0.832304835f, -0.722269833f),
                            new Vector2(1.19917297f, -0.691555977f),
                            new Vector2(1.19911861f, -0.5987854f),
                            new Vector2(1.13810253f, -0.597414017f),
                            new Vector2(1.20131111f, -0.398326874f),
                            new Vector2(1.20297909f, -0.280597687f),
                            new Vector2(1.14859009f, -0.0843486786f),
                            new Vector2(1.0273627f, -0.0446084701f),
                            new Vector2(1.02782154f, -0.239442825f),
                            new Vector2(0.533593178f, -0.236851543f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(-0.206746101f, -0.598035812f),
                            new Vector2(-0.201748848f, -0.378677368f),
                            new Vector2(-0.562030792f, -0.381916046f),
                            new Vector2(-0.606800079f, 0.123495102f),
                            new Vector2(-0.70598793f, 0.234489441f),
                            new Vector2(-0.851586342f, 0.133693695f),
                            new Vector2(-0.858656883f, 0.942453384f),
                            new Vector2(-0.733301997f, 1.05434871f),
                            new Vector2(0.0830484629f, 1.06488109f),
                            new Vector2(0.454452753f, 0.10799551f),
                            new Vector2(0.450418562f, -0.769970477f),
                            new Vector2(-0.377611041f, -0.767501414f)
                        });
                        BoxCollider2D BodyTrigger = Body.gameObject.AddComponent<BoxCollider2D>();
                        BodyTrigger.offset = new Vector2(0.04886436f, -0.277637f);
                        BodyTrigger.size = new Vector2(1.808479f, 0.9866743f);
                        BodyTrigger.isTrigger = true;
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Peel P50/Red/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Peel P50/Red/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Peel P50/Red/Bonnet.png"),
                                Background = ModAPI.LoadSprite("Textures/Peel P50/Background.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Peel P50/Green/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Peel P50/Green/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Peel P50/Green/Bonnet.png"),
                                Background = ModAPI.LoadSprite("Textures/Peel P50/Background.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Peel P50/Grey/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Peel P50/Grey/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Peel P50/Grey/Bonnet.png"),
                                Background = ModAPI.LoadSprite("Textures/Peel P50/Background.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero; //Car bottom position

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].anchor = new Vector2(1.0139f, -0.9587f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.2428571f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        Wheels[1].anchor = new Vector2(-1.0149f, -0.9587f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.2428571f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        //Hood
                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";
                        Bonnet.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 500f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.31f, 0.049f);
                        Bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65f,
                            max = 0f
                        };
                        Bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.0847015381f,0.0284118652f),
                            new Vector2(-0.0130739212f,0.0841293335f),
                            new Vector2(-0.328571439f,0.0857142881f),
                            new Vector2(-0.326112866f,0.0267359596f),
                            new Vector2(-0.0259545222f,0.0248985812f),
                            new Vector2(0.302761197f,-0.0857142881f),
                            new Vector2(0.329667091f,-0.0328731537f),
                            new Vector2(0.296616673f,0.0279483274f)
                        });

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.564616442f,-0.941110969f),
                            new Vector2(0.619236946f,-0.948701859f),
                            new Vector2(0.683968127f,-0.864297271f),
                            new Vector2(0.694088519f,-0.0362074673f),
                            new Vector2(0.373811722f,0.84702301f),
                            new Vector2(0.281033456f,0.938670516f),
                            new Vector2(-0.534899712f,0.943271637f),
                            new Vector2(-0.689149439f,0.800288558f),
                            new Vector2(-0.682222903f,-0.713522017f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.658f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot
                        Instance.transform.GetChild(6).gameObject.SetActive(false); //Back door

                        //Engine
                        Transform Engine = Instance.transform.GetChild(7);
                        Engine.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Peel P50/Engine.png");
                        Engine.gameObject.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.103981018f, 0.170022964f),
                            new Vector2(-0.351393193f, 0.171121448f),
                            new Vector2(-0.426885337f, -0.0192716569f),
                            new Vector2(-0.376283377f, -0.253769726f),
                            new Vector2(0.0797317326f, -0.253768772f),
                            new Vector2(0.210047454f, -0.0620719939f),
                            new Vector2(0.426916122f, -0.116644859f),
                            new Vector2(0.430258483f, 0.2183474f)
                        });
                        UnityEngine.Object.Destroy(Engine.GetComponent<BoxCollider2D>());
                        
                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.GetComponent<BoxCollider2D>().size = new Vector3(0.5142857f, 0.3428572f);
                        GasTank.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Peel P50/Gas Tank.png");
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        //Not colliding objects with wheels in a certain area
                        NoCollideWithObjectsInTrigger NoCollideWithObjectsInTrigger = Body.gameObject.AddComponent<NoCollideWithObjectsInTrigger>();
                        NoCollideWithObjectsInTrigger.Colliders = new Collider2D[]
                        {
                            Instance.transform.GetChild(7).gameObject.GetComponent<PolygonCollider2D>(),
                            Instance.transform.GetChild(8).gameObject.GetComponent<BoxCollider2D>()
                        };
                        NoCollideWithObjectsInTrigger.Trigger = BodyTrigger;

                        //Serialization
                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(1.0139f, -0.9587f);
                            Wheels[0].connectedBody.transform.localScale = new Vector2(1f, 1f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(-1.0149f, -0.9587f);
                            Wheels[1].connectedBody.transform.localScale = new Vector2(1f, 1f);
                            Bonnet.localPosition = new Vector3(0.8649998f, -0.008001328f);
                            FrontDoor.localPosition = new Vector3(-0.1997f, 0.1425f);
                            Engine.localScale = new Vector3(1f, 1f);
                            Engine.localPosition = new Vector3(0.3659992f, -0.6340008f);
                            GasTank.localScale = new Vector3(1f, 1f);
                            GasTank.localPosition = new Vector3(-0.7780008f, -0.6420002f);
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Motorcycle
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Motorcycle",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Fast motorcycle! Unfortunately, you can't put it in the water!",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Motorcycle.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/Motorcycle/Wheel.png");

                        //Car speed
                        float MotorSpeed = -4300f;

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        var SmoothBraking = Body.gameObject.AddComponent<CarSmoothBrakingBehaviour>();
                        SmoothBraking.DefaultTorque = 150f;
                        SmoothBraking.SmoothBrakingTorque = 25f;
                        SmoothBraking.SpeedForFullBraking = 50f;
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        foreach (Collider2D Collider in Body.GetComponentsInChildren<Collider2D>())
                        {
                            NoCollideWithBodyPart noCollideWithBodyPart = Body.gameObject.AddComponent<NoCollideWithBodyPart>();
                            noCollideWithBodyPart.BodyPart = LimbBehaviour.BodyPart.Legs;
                            noCollideWithBodyPart.Collider = Collider;
                        }
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 2f;
                        BodyPhysical.InitialMass = 2f;
                        BodyPhysical.rigidbody.mass = 2f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 1;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(0.938556433f,0.175993994f),
                            new Vector2(0.677660406f,0.700242639f),
                            new Vector2(0.581141472f,0.670809746f),
                            new Vector2(0.700066805f,0.400308609f),
                            new Vector2(0.469714284f,0.512446523f),
                            new Vector2(0.0465953201f,0.514757454f),
                            new Vector2(-0.227331698f,0.377121061f),
                            new Vector2(-1.12366378f,0.398630768f),
                            new Vector2(-1.22082663f,0.172279134f),
                            new Vector2(-0.815399289f,0.136743248f),
                            new Vector2(-1.17459643f,-0.543898046f),
                            new Vector2(-1.04201734f,-0.598376989f),
                            new Vector2(-0.709159791f,0.106118843f),
                            new Vector2(-0.610299826f,-0.139732212f),
                            new Vector2(-0.328336179f,-0.142214149f),
                            new Vector2(-0.144120365f,0.113676205f),
                            new Vector2(0.0182590485f,0.200406641f),
                            new Vector2(0.797033846f,0.204700604f),
                            new Vector2(1.19773507f,-0.606110096f),
                            new Vector2(1.31558096f,-0.553987563f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().OnAfterChange = new UnityEvent();
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Motorcycle/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Motorcycle/None.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Motorcycle/Texture2/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Motorcycle/None.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Motorcycle/Texture3/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Motorcycle/None.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Motorcycle/Texture4/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Motorcycle/None.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Motorcycle/Texture5/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Motorcycle/None.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Motorcycle/Texture6/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Motorcycle/None.png")
                            }
                        };

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].suspension = new JointSuspension2D
                        {
                            frequency = 8f,
                            angle = Instance.transform.localScale.x > 0 ? 65f : -65f,
                            dampingRatio = 0.7f
                        };
                        Wheels[0].anchor = new Vector2(-1.0864f, -0.5307f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.3857143f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -1;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Default";

                        Wheels[1].suspension = new JointSuspension2D
                        {
                            frequency = 10f,
                            angle = Instance.transform.localScale.x > 0 ? 114f : -114f,
                            dampingRatio = 0.7f
                        };
                        Wheels[1].anchor = new Vector2(1.228f, -0.5307f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.3857143f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -1;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Default";

                        Instance.transform.GetChild(3).gameObject.SetActive(false); //Hood
                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot
                        Instance.transform.GetChild(5).gameObject.SetActive(false); //Front door
                        Instance.transform.GetChild(6).gameObject.SetActive(false); //Back door

                        //Light
                        GameObject Light = new GameObject("Light");
                        Light.transform.parent = Instance.transform;
                        Light.transform.localRotation = Body.localRotation;
                        Light.transform.localScale = Vector3.one;
                        Light.transform.localPosition = new Vector3(0.954f, 0.418f);
                        SpriteRenderer LightRenderer = Light.AddComponent<SpriteRenderer>();
                        LightRenderer.sortingLayerName = "Default";
                        LightRenderer.sortingOrder = 1;
                        Light.AddComponent<RandomCarTextureChildBehaviour>().Sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Motorcycle/Light.png"),
                            ModAPI.LoadSprite("Textures/Motorcycle/Texture2/Light.png"),
                            ModAPI.LoadSprite("Textures/Motorcycle/Texture3/Light.png"),
                            ModAPI.LoadSprite("Textures/Motorcycle/Texture4/Light.png"),
                            ModAPI.LoadSprite("Textures/Motorcycle/Texture5/Light.png"),
                            ModAPI.LoadSprite("Textures/Motorcycle/Texture6/Light.png"),
                        };
                        Light.layer = LayerMask.NameToLayer("Objects");
                        Light.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D LightFixedJoint = Light.AddComponent<FixedJoint2D>();
                        LightFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        LightFixedJoint.anchor = new Vector2(-0.101f, -0.05f);
                        LightFixedJoint.frequency = 25f;
                        LightFixedJoint.dampingRatio = 5f;
                        LightFixedJoint.breakForce = 5000f;
                        Light.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.0276281834f,0.214452267f),
                            new Vector2(-0.0275788307f,0.214558363f),
                            new Vector2(-0.115581043f,0.100296602f),
                            new Vector2(-0.112990148f,-0.0990012735f),
                            new Vector2(-0.0332603455f,-0.214041591f),
                            new Vector2(0.0299363136f,-0.213851094f),
                            new Vector2(0.116876371f,-0.0977058262f),
                            new Vector2(0.114285715f,0.0964106172f)
                        });
                        Light.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Light.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Light.AddComponent<Optout>();

                        //Front fender part 1
                        GameObject FrontFender = new GameObject("Front Fender");
                        FrontFender.transform.parent = Instance.transform;
                        FrontFender.transform.localRotation = Body.localRotation;
                        FrontFender.transform.localScale = Vector3.one;
                        FrontFender.transform.localPosition = new Vector3(0.827f, -0.356f);
                        SpriteRenderer FrontFenderRenderer = FrontFender.AddComponent<SpriteRenderer>();
                        FrontFenderRenderer.sprite = ModAPI.LoadSprite("Textures/Motorcycle/FrontFender.png");
                        FrontFenderRenderer.sortingLayerName = "Default";
                        FrontFenderRenderer.sortingOrder = 1;
                        FrontFender.layer = LayerMask.NameToLayer("Objects");
                        FrontFender.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D FrontFenderFixedJoint = FrontFender.AddComponent<FixedJoint2D>();
                        FrontFenderFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        FrontFenderFixedJoint.frequency = 8f;
                        FrontFenderFixedJoint.dampingRatio = 1f;
                        FrontFenderFixedJoint.breakForce = 1500f;
                        FrontFenderFixedJoint.anchor = new Vector2(0.139f, 0.214f);
                        FrontFender.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.0772174895f,0.24789159f),
                            new Vector2(-0.0916937441f,0.033542484f),
                            new Vector2(-0.153367028f,-0.216425821f),
                            new Vector2(-0.088545233f,-0.241598532f),
                            new Vector2(0.0321439505f,0.0721812248f),
                            new Vector2(0.147994861f,0.192959234f)
                        });
                        FrontFender.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                        FrontFender.AddComponent<AudioSourceTimeScaleBehaviour>();
                        FrontFender.AddComponent<Optout>();

                        //Front fender part 2
                        GameObject FrontFender2 = new GameObject("Front Fender 2");
                        FrontFender2.transform.parent = Instance.transform;
                        FrontFender2.transform.localRotation = Body.localRotation;
                        FrontFender2.transform.localScale = Vector3.one;
                        FrontFender2.transform.localPosition = new Vector3(1.314f, -0.099f);
                        SpriteRenderer FrontFender2Renderer = FrontFender2.AddComponent<SpriteRenderer>();
                        FrontFender2Renderer.sprite = ModAPI.LoadSprite("Textures/Motorcycle/FrontFender2.png");
                        FrontFender2Renderer.sortingLayerName = "Default";
                        FrontFender2Renderer.sortingOrder = 1;
                        FrontFender2.layer = LayerMask.NameToLayer("Objects");
                        FrontFender2.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D FrontFender2FixedJoint = FrontFender2.AddComponent<FixedJoint2D>();
                        FrontFender2FixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        FrontFender2FixedJoint.frequency = 8f;
                        FrontFender2FixedJoint.dampingRatio = 2f;
                        FrontFender2FixedJoint.breakForce = 1500f;
                        FrontFender2FixedJoint.anchor = new Vector2(-0.271f, 0.023f);
                        FrontFender2.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.268486023f,0.069592692f),
                            new Vector2(-0.273390293f,-0.0166251436f),
                            new Vector2(-0.0821105242f,0.0187972784f),
                            new Vector2(0.0853084326f,-0.0164974928f),
                            new Vector2(0.203747749f,-0.101961829f),
                            new Vector2(0.272409439f,-0.0356132053f),
                            new Vector2(0.0179837197f,0.100980856f)
                        });
                        FrontFender2.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                        FrontFender2.AddComponent<AudioSourceTimeScaleBehaviour>();
                        FrontFender2.AddComponent<Optout>();

                        //Back fender part 1
                        GameObject BackFender = new GameObject("Back Fender");
                        BackFender.transform.parent = Instance.transform;
                        BackFender.transform.localRotation = Body.localRotation;
                        BackFender.transform.localScale = Vector3.one;
                        BackFender.transform.localPosition = new Vector3(-1.258746f, -0.2714922f);
                        SpriteRenderer BackFenderRenderer = BackFender.AddComponent<SpriteRenderer>();
                        BackFenderRenderer.sprite = ModAPI.LoadSprite("Textures/Motorcycle/BackFender.png");
                        BackFenderRenderer.sortingLayerName = "Default";
                        BackFenderRenderer.sortingOrder = 1;
                        BackFender.layer = LayerMask.NameToLayer("Objects");
                        BackFender.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D BackFenderFixedJoint = BackFender.AddComponent<FixedJoint2D>();
                        BackFenderFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        BackFenderFixedJoint.frequency = 8f;
                        BackFenderFixedJoint.dampingRatio = 2f;
                        BackFenderFixedJoint.breakForce = 1500f;
                        BackFenderFixedJoint.anchor = new Vector2(0.3416f, 0.1953f);
                        BackFender.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.357142866f,0.269001245f),
                            new Vector2(0.0742130205f,0.275069475f),
                            new Vector2(-0.163211271f,0.157242879f),
                            new Vector2(-0.275726438f,0.0143856183f),
                            new Vector2(-0.349020362f,-0.114428401f),
                            new Vector2(-0.359638214f,-0.268691778f),
                            new Vector2(-0.268901348f,-0.270214915f),
                            new Vector2(-0.229310691f,-0.0786367655f),
                            new Vector2(-0.139594451f,0.0625332743f),
                            new Vector2(0.0717877746f,0.172480941f),
                            new Vector2(0.336510301f,0.164881974f)
                        });
                        BackFender.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                        BackFender.AddComponent<AudioSourceTimeScaleBehaviour>();
                        BackFender.AddComponent<Optout>();

                        //Back fender part 2
                        GameObject BackFender2 = new GameObject("Back Fender 2");
                        BackFender2.transform.parent = Instance.transform;
                        BackFender2.transform.localRotation = Body.localRotation;
                        BackFender2.transform.localScale = Vector3.one;
                        BackFender2.transform.localPosition = new Vector3(-0.785f, -0.126f);
                        SpriteRenderer BackFender2Renderer = BackFender2.AddComponent<SpriteRenderer>();
                        BackFender2Renderer.sprite = ModAPI.LoadSprite("Textures/Motorcycle/BackFender2.png");
                        BackFender2Renderer.sortingLayerName = "Default";
                        BackFender2Renderer.sortingOrder = 1;
                        BackFender2.layer = LayerMask.NameToLayer("Objects");
                        BackFender2.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D BackFender2FixedJoint = BackFender2.AddComponent<FixedJoint2D>();
                        BackFender2FixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        BackFender2FixedJoint.frequency = 8f;
                        BackFender2FixedJoint.dampingRatio = 2f;
                        BackFender2FixedJoint.breakForce = 1500f;
                        BackFender2FixedJoint.anchor = new Vector2(-0.0469f, 0.0208f);
                        BackFender2.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.0523113981f,0.0787608996f),
                            new Vector2(-0.0873437002f,0.0231340192f),
                            new Vector2(0.0498672798f,-0.0787608996f),
                            new Vector2(0.0897877887f,-0.0231339f)
                        });
                        BackFender2.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                        BackFender2.AddComponent<AudioSourceTimeScaleBehaviour>();
                        BackFender2.AddComponent<Optout>();

                        //Engine
                        Transform Engine = Instance.transform.GetChild(7);
                        Engine.gameObject.AddComponent<CircleCollider2D>().radius = 0.2f;
                        Engine.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Motorcycle/Engine.png");
                        Engine.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
                        foreach (Collider2D collider in Engine.GetComponents<Collider2D>())
                        {
                            NoCollideWithBodyPart noCollideWithBodyPart = Engine.gameObject.AddComponent<NoCollideWithBodyPart>();
                            noCollideWithBodyPart.BodyPart = LimbBehaviour.BodyPart.Legs;
                            noCollideWithBodyPart.Collider = collider;
                        }
                        UnityEngine.Object.Destroy(Engine.GetComponent<BoxCollider2D>());

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        UnityEngine.Object.Destroy(GasTank.GetComponent<BoxCollider2D>());
                        GasTank.gameObject.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.300000012f,0.157142863f),
                            new Vector2(-0.300000012f,-0.157142863f),
                            new Vector2(0.130043849f,-0.157142863f),
                            new Vector2(0.298886597f,0.0116999f),
                            new Vector2(0.271032572f,0.127796769f),
                            new Vector2(0.0706378222f,0.157142863f)
                        });
                        GasTank.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Motorcycle/Gas Tank.png");
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
                        foreach (Collider2D collider in GasTank.GetComponents<Collider2D>())
                        {
                            NoCollideWithBodyPart noCollideWithBodyPart = GasTank.gameObject.AddComponent<NoCollideWithBodyPart>();
                            noCollideWithBodyPart.BodyPart = LimbBehaviour.BodyPart.Legs;
                            noCollideWithBodyPart.Collider = collider;
                        }

                        //Serialization
                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(-1.0864f, -0.5307f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(1.228f, -0.5307f);
                            Engine.localPosition = new Vector3(-0.007f, 0.003f);
                            GasTank.localPosition = new Vector3(0.431f, 0.034f);

                            //Serializing object
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Light;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = FrontFender;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = FrontFender2;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = BackFender;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = BackFender2;
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region TukTuk
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "TukTuk",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Tricicle. Uh, just.... Uh... Enjoy the ride!",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Filipino Trycicle.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/Motorcycle/Wheel.png");

                        //Car speed
                        float MotorSpeed = -3400f;

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        var SmoothBraking = Body.gameObject.AddComponent<CarSmoothBrakingBehaviour>();
                        SmoothBraking.DefaultTorque = 150f;
                        SmoothBraking.SmoothBrakingTorque = 25f;
                        SmoothBraking.SpeedForFullBraking = 50f;
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        foreach (Collider2D Collider in Body.GetComponents<Collider2D>())
                        {
                            NoCollideWithBodyPart NoCollideWithBodyPart = Body.gameObject.AddComponent<NoCollideWithBodyPart>();
                            NoCollideWithBodyPart.BodyPart = LimbBehaviour.BodyPart.Legs;
                            NoCollideWithBodyPart.Collider = Collider;
                        }
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 4f;
                        BodyPhysical.InitialMass = 4f;
                        BodyPhysical.rigidbody.mass = 4f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 1;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(0.938556433f,0.175993994f),
                            new Vector2(0.677660406f,0.700242639f),
                            new Vector2(0.581141472f,0.670809746f),
                            new Vector2(0.700066805f,0.400308609f),
                            new Vector2(0.469714284f,0.512446523f),
                            new Vector2(0.0465953201f,0.514757454f),
                            new Vector2(-0.227331698f,0.377121061f),
                            new Vector2(-1.04691327f,0.398630768f),
                            new Vector2(-0.977782965f,0.152177781f),
                            new Vector2(-0.577837825f,0.129433662f),
                            new Vector2(-0.487864256f,-0.14338699f),
                            new Vector2(-0.328336179f,-0.142214149f),
                            new Vector2(-0.144120365f,0.113676205f),
                            new Vector2(0.0182590485f,0.200406641f),
                            new Vector2(0.797033846f,0.204700604f),
                            new Vector2(1.19773507f,-0.606110096f),
                            new Vector2(1.31558096f,-0.553987563f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Motorcycle/Filipino Trycicle/Motorcycle Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Motorcycle/None.png")
                            }
                        };

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].suspension = new JointSuspension2D()
                        {
                            frequency = 13f,
                            angle = 65f,
                            dampingRatio = 1f
                        };
                        Wheels[0].anchor = new Vector2(-1.0864f, -0.5307f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.3857143f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -9;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";

                        foreach (Collider2D Collider in Wheels[0].connectedBody.GetComponents<Collider2D>())
                        {
                            NoCollideWithBodyPart noCollide = Wheels[0].connectedBody.gameObject.AddComponent<NoCollideWithBodyPart>();
                            noCollide.BodyPart = LimbBehaviour.BodyPart.Legs;
                            noCollide.Collider = Collider;
                        }

                        foreach (Collider2D Collider in Wheels[0].connectedBody.GetComponents<Collider2D>())
                        {
                            NoCollideWithBodyPart noCollide = Wheels[0].connectedBody.gameObject.AddComponent<NoCollideWithBodyPart>();
                            noCollide.BodyPart = LimbBehaviour.BodyPart.Torso;
                            noCollide.Collider = Collider;
                        }

                        Wheels[1].suspension = new JointSuspension2D()
                        {
                            frequency = 13f,
                            angle = 114f,
                            dampingRatio = 1f
                        };
                        Wheels[1].anchor = new Vector2(1.228f, -0.5307f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.3857143f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -1;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Default";

                        Instance.transform.GetChild(3).gameObject.SetActive(false); //Hood
                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot
                        Instance.transform.GetChild(5).gameObject.SetActive(false); //Front door
                        Instance.transform.GetChild(6).gameObject.SetActive(false); //Back door

                        //Light
                        GameObject Light = new GameObject("Light");
                        Light.transform.parent = Instance.transform;
                        Light.transform.localRotation = Body.localRotation;
                        Light.transform.localScale = Vector3.one;
                        Light.transform.localPosition = new Vector3(0.954f, 0.418f);
                        SpriteRenderer LightRenderer = Light.AddComponent<SpriteRenderer>();
                        LightRenderer.sprite = ModAPI.LoadSprite("Textures/Motorcycle/Light.png");
                        LightRenderer.sortingOrder = 1;
                        Light.layer = LayerMask.NameToLayer("Objects");
                        Light.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D LightFixedJoint = Light.AddComponent<FixedJoint2D>();
                        LightFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        LightFixedJoint.anchor = new Vector2(-0.101f, -0.05f);
                        LightFixedJoint.frequency = 25f;
                        LightFixedJoint.dampingRatio = 5f;
                        LightFixedJoint.breakForce = 5000f;
                        Light.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.0276281834f,0.214452267f),
                            new Vector2(-0.0275788307f,0.214558363f),
                            new Vector2(-0.115581043f,0.100296602f),
                            new Vector2(-0.112990148f,-0.0990012735f),
                            new Vector2(-0.0332603455f,-0.214041591f),
                            new Vector2(0.0299363136f,-0.213851094f),
                            new Vector2(0.116876371f,-0.0977058262f),
                            new Vector2(0.114285715f,0.0964106172f)
                        });
                        Light.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Light.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Light.AddComponent<Optout>();

                        //Front fender part 1
                        GameObject FrontFender = new GameObject("Front Fender");
                        FrontFender.transform.parent = Instance.transform;
                        FrontFender.transform.localRotation = Body.localRotation;
                        FrontFender.transform.localScale = Vector3.one;
                        FrontFender.transform.localPosition = new Vector3(0.827f, -0.356f);
                        SpriteRenderer FrontFenderRenderer = FrontFender.AddComponent<SpriteRenderer>();
                        FrontFenderRenderer.sprite = ModAPI.LoadSprite("Textures/Motorcycle/FrontFender.png");
                        FrontFenderRenderer.sortingLayerName = "Default";
                        FrontFenderRenderer.sortingOrder = 1;
                        FrontFender.layer = LayerMask.NameToLayer("Objects");
                        FrontFender.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D FrontFenderFixedJoint = FrontFender.AddComponent<FixedJoint2D>();
                        FrontFenderFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        FrontFenderFixedJoint.frequency = 8f;
                        FrontFenderFixedJoint.dampingRatio = 1f;
                        FrontFenderFixedJoint.breakForce = 1500f;
                        FrontFenderFixedJoint.anchor = new Vector2(0.139f, 0.214f);
                        FrontFender.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.0772174895f,0.24789159f),
                            new Vector2(-0.0916937441f,0.033542484f),
                            new Vector2(-0.153367028f,-0.216425821f),
                            new Vector2(-0.088545233f,-0.241598532f),
                            new Vector2(0.0321439505f,0.0721812248f),
                            new Vector2(0.147994861f,0.192959234f)
                        });
                        FrontFender.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                        FrontFender.AddComponent<AudioSourceTimeScaleBehaviour>();
                        FrontFender.AddComponent<Optout>();

                        //Front fender part 2
                        GameObject FrontFender2 = new GameObject("Front Fender 2");
                        FrontFender2.transform.parent = Instance.transform;
                        FrontFender2.transform.localRotation = Body.localRotation;
                        FrontFender2.transform.localScale = Vector3.one;
                        FrontFender2.transform.localPosition = new Vector3(1.314f, -0.099f);
                        SpriteRenderer FrontFender2Renderer = FrontFender2.AddComponent<SpriteRenderer>();
                        FrontFender2Renderer.sprite = ModAPI.LoadSprite("Textures/Motorcycle/FrontFender2.png");
                        FrontFender2Renderer.sortingLayerName = "Default";
                        FrontFender2Renderer.sortingOrder = 1;
                        FrontFender2.layer = LayerMask.NameToLayer("Objects");
                        FrontFender2.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D FrontFender2FixedJoint = FrontFender2.AddComponent<FixedJoint2D>();
                        FrontFender2FixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        FrontFender2FixedJoint.frequency = 8f;
                        FrontFender2FixedJoint.dampingRatio = 2f;
                        FrontFender2FixedJoint.breakForce = 1500f;
                        FrontFender2FixedJoint.anchor = new Vector2(-0.271f, 0.023f);
                        FrontFender2.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.268486023f,0.069592692f),
                            new Vector2(-0.273390293f,-0.0166251436f),
                            new Vector2(-0.0821105242f,0.0187972784f),
                            new Vector2(0.0853084326f,-0.0164974928f),
                            new Vector2(0.203747749f,-0.101961829f),
                            new Vector2(0.272409439f,-0.0356132053f),
                            new Vector2(0.0179837197f,0.100980856f)
                        });
                        FrontFender2.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                        FrontFender2.AddComponent<AudioSourceTimeScaleBehaviour>();
                        FrontFender2.AddComponent<Optout>();

                        //Tricycle body
                        GameObject TrycicleBody = new GameObject("Trycicle Body");
                        TrycicleBody.transform.parent = Body;
                        TrycicleBody.transform.localRotation = Quaternion.identity;
                        TrycicleBody.transform.localScale = Vector3.one;
                        TrycicleBody.transform.localPosition = new Vector3(-1.075f, 0.596f);
                        SpriteRenderer TrycicleBodyRenderer = TrycicleBody.AddComponent<SpriteRenderer>();
                        TrycicleBodyRenderer.sprite = ModAPI.LoadSprite("Textures/Motorcycle/Filipino Trycicle/Body.png");
                        TrycicleBodyRenderer.sortingLayerName = "Default";
                        TrycicleBodyRenderer.sortingOrder = 6;
                        TrycicleBody.layer = LayerMask.NameToLayer("Objects");
                        TrycicleBody.AddComponent<Rigidbody2D>().mass = 1f;
                        TrycicleBody.AddComponent<FixedJoint2D>().connectedBody = Body.GetComponent<Rigidbody2D>();
                        TrycicleBody.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.856763244f,1.16979635f),
                            new Vector2(-1.1269033f,0.204314411f),
                            new Vector2(-1.12608671f,-0.207771033f),
                            new Vector2(-0.725970387f,-1.16276741f),
                            new Vector2(0.677913368f,-1.15264821f),
                            new Vector2(0.665659845f,-1.07439184f),
                            new Vector2(-0.699467063f,-1.07142222f),
                            new Vector2(-1.01060963f,-0.137491316f),
                            new Vector2(-1.00834894f,0.248396337f),
                            new Vector2(-0.776093066f,0.925756514f),
                            new Vector2(0.570355356f,0.931275606f),
                            new Vector2(0.758990645f,1.08659768f),
                            new Vector2(0.489776373f,1.15888596f)
                        });
                        TrycicleBody.GetComponent<PolygonCollider2D>().pathCount = 1;
                        TrycicleBody.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        TrycicleBody.AddComponent<AudioSourceTimeScaleBehaviour>();
                        TrycicleBody.AddComponent<Optout>();

                        //Tricycle body background
                        GameObject TrycicleBackground = new GameObject("Trycicle Background");
                        TrycicleBackground.transform.parent = TrycicleBody.transform;
                        TrycicleBackground.transform.localRotation = Quaternion.identity;
                        TrycicleBackground.transform.localScale = Vector3.one;
                        TrycicleBackground.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer TrycicleBackgroundRenderer = TrycicleBackground.AddComponent<SpriteRenderer>();
                        TrycicleBackgroundRenderer.sprite = ModAPI.LoadSprite("Textures/Motorcycle/Filipino Trycicle/Background.png");
                        TrycicleBackgroundRenderer.sortingLayerName = "Bottom";
                        TrycicleBackgroundRenderer.sortingOrder = -7;
                        TrycicleBackground.AddComponent<Optout>();

                        //Body 2
                        GameObject MotorcycleBody2 = new GameObject("Motorcycle Body 2");
                        MotorcycleBody2.transform.parent = Body;
                        MotorcycleBody2.transform.localRotation = Quaternion.identity;
                        MotorcycleBody2.transform.localScale = Vector3.one;
                        MotorcycleBody2.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer MotorcycleBody2Renderer = MotorcycleBody2.AddComponent<SpriteRenderer>();
                        MotorcycleBody2Renderer.sprite = ModAPI.LoadSprite("Textures/Motorcycle/Filipino Trycicle/Motorcycle Body2.png");
                        MotorcycleBody2Renderer.sortingLayerName = "Bottom";
                        MotorcycleBody2Renderer.sortingOrder = -8;
                        MotorcycleBody2.layer = LayerMask.NameToLayer("Objects");
                        MotorcycleBody2.AddComponent<Rigidbody2D>().mass = 0f;
                        MotorcycleBody2.AddComponent<FixedJoint2D>().connectedBody = Body.GetComponent<Rigidbody2D>();
                        MotorcycleBody2.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        MotorcycleBody2.AddComponent<AudioSourceTimeScaleBehaviour>();
                        MotorcycleBody2.AddComponent<Optout>();

                        //Chair
                        GameObject Chair = new GameObject("Chair");
                        Chair.transform.parent = Instance.transform;
                        Chair.transform.localRotation = Body.localRotation;
                        Chair.transform.localScale = Vector3.one;
                        Chair.transform.localPosition = new Vector3(-1.588f, 0.024f);
                        SpriteRenderer ChairRenderer = Chair.AddComponent<SpriteRenderer>();
                        ChairRenderer.sprite = ModAPI.LoadSprite("Textures/Motorcycle/Filipino Trycicle/Chair.png");
                        ChairRenderer.sortingLayerName = "Default";
                        ChairRenderer.sortingOrder = 5;
                        Chair.layer = LayerMask.NameToLayer("Objects");
                        Chair.AddComponent<Rigidbody2D>().mass = 0.2f;
                        FixedJoint2D ChairFixedJoint = Chair.AddComponent<FixedJoint2D>();
                        ChairFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        ChairFixedJoint.breakForce = 2000f;
                        Chair.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.177391648f,0.505213141f),
                            new Vector2(-0.307724714f,0.494143993f),
                            new Vector2(-0.326942801f,0.24908267f),
                            new Vector2(-0.245497823f,-0.195290446f),
                            new Vector2(-0.183290958f,-0.500948668f),
                            new Vector2(0.21572268f,-0.500665426f),
                            new Vector2(0.326414347f,-0.185063392f),
                            new Vector2(0.297913194f,-0.0695647448f),
                            new Vector2(-0.0751104504f,-0.0705380142f)
                        });
                        Chair.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Chair.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Chair.AddComponent<Optout>();

                        //Engine
                        Transform Engine = Instance.transform.GetChild(7);
                        UnityEngine.Object.Destroy(Engine.GetComponent<BoxCollider2D>());
                        Engine.gameObject.AddComponent<CircleCollider2D>();
                        Engine.GetComponent<CircleCollider2D>().radius = 0.2f;
                        Engine.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Motorcycle/Engine.png");
                        Engine.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
                        foreach (Collider2D Collider in Engine.GetComponents<Collider2D>())
                        {
                            NoCollideWithBodyPart NoCollideWithBodyPart = Engine.gameObject.AddComponent<NoCollideWithBodyPart>();
                            NoCollideWithBodyPart.BodyPart = LimbBehaviour.BodyPart.Legs;
                            NoCollideWithBodyPart.Collider = Collider;
                        }

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        UnityEngine.Object.Destroy(GasTank.GetComponent<BoxCollider2D>());
                        GasTank.gameObject.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.300000012f,0.157142863f),
                            new Vector2(-0.300000012f,-0.157142863f),
                            new Vector2(0.130043849f,-0.157142863f),
                            new Vector2(0.298886597f,0.0116999f),
                            new Vector2(0.271032572f,0.127796769f),
                            new Vector2(0.0706378222f,0.157142863f)
                        });
                        GasTank.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Motorcycle/Gas Tank.png");
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Default";
                        foreach (Collider2D Collider in GasTank.GetComponents<Collider2D>())
                        {
                            NoCollideWithBodyPart NoCollideWithBodyPart = GasTank.gameObject.AddComponent<NoCollideWithBodyPart>();
                            NoCollideWithBodyPart.BodyPart = LimbBehaviour.BodyPart.Legs;
                            NoCollideWithBodyPart.Collider = Collider;
                        }

                        //Serialization
                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(-1.0864f, -0.5307f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(1.228f, -0.5307f);
                            Engine.localPosition = new Vector3(-0.007f, 0.003f);
                            GasTank.localPosition = new Vector3(0.431f, 0.034f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = FrontFender;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = FrontFender2;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Light;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Chair;
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
#endregion
#region Extra Heavy Vehicles
        #region Tractor
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Tractor",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Slow tractor. Maximum height of the driver - 0.8m",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Tractor.png"),
                    AfterSpawn = (Instance) =>
                    {
                        //Car speed
                        float MotorSpeed = -250f;

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        Body.GetComponent<CarBehaviour>().WaterProof = true;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        Body.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.671237946f, 0.586121202f),
                            new Vector2(0.276311755f, 0.595907271f),
                            new Vector2(0.278191626f, 1.35058725f),
                            new Vector2(0.333479762f, 1.35465777f),
                            new Vector2(0.299787372f, 1.49792409f),
                            new Vector2(-1.27561867f, 1.5011512f),
                            new Vector2(-1.30210316f, 1.35262835f),
                            new Vector2(-1.27505136f, 1.32455063f),
                            new Vector2(-1.27067149f, 1.01681781f),
                            new Vector2(-1.30069339f, 1.01364172f),
                            new Vector2(-1.30230045f, 0.673768401f),
                            new Vector2(-1.32668662f, 0.670561671f),
                            new Vector2(-1.33252764f, 0.331430942f),
                            new Vector2(-1.58562398f, 0.331357718f),
                            new Vector2(-1.58713007f, 0.239980459f),
                            new Vector2(-1.32973599f, 0.244569495f),
                            new Vector2(-1.32872987f, -0.0690284371f),
                            new Vector2(-1.15337753f, -0.414788365f),
                            new Vector2(-0.92498517f, -0.529495776f),
                            new Vector2(-0.445107877f, -0.530214489f),
                            new Vector2(-0.443457484f, -0.584004641f),
                            new Vector2(-0.358813107f, -0.584108472f),
                            new Vector2(-0.355927974f, -0.524684429f),
                            new Vector2(0.18240568f, -0.52665025f),
                            new Vector2(0.185097665f, -0.58128345f),
                            new Vector2(0.306797624f, -0.671058655f),
                            new Vector2(0.819960713f, -0.67579633f),
                            new Vector2(0.816046178f, -0.954868853f),
                            new Vector2(0.904327989f, -0.953218341f),
                            new Vector2(0.905614495f, -0.676502287f),
                            new Vector2(1.0197506f, -0.452649951f),
                            new Vector2(1.29654419f, -0.228157222f),
                            new Vector2(1.67730415f, -0.128672898f),
                            new Vector2(2.04187536f, -0.214909613f),
                            new Vector2(2.24207473f, -0.498955667f),
                            new Vector2(2.21824884f, -0.27132386f),
                            new Vector2(2.21526909f, 0.186229259f),
                            new Vector2(0.671206951f, 0.186790615f)
                        });
                        Body.GetComponent<PolygonCollider2D>().SetPath(1, new Vector2[]
                        {
                            new Vector2(-1.08345056f, -0.315188438f),
                            new Vector2(-1.22759151f, -0.151488289f),
                            new Vector2(-1.23919761f, 0.328147829f),
                            new Vector2(-1.18459749f, 1.42672908f),
                            new Vector2(0.186343372f, 1.42446315f),
                            new Vector2(0.187714219f, -0.440303802f),
                            new Vector2(-0.918489218f, -0.436653674f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Tractor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Tractor/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Tractor/Door.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Tractor/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Tractor/Bonnet.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Tractor/Yellow/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Tractor/Yellow/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Tractor/Door.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Tractor/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Tractor/Yellow/Bonnet.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Tractor/Broken/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Tractor/Broken/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Tractor/Broken/Door.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Tractor/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Tractor/Broken/Bonnet.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero; //Car bottom position

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].anchor = new Vector2(-1.428f, -0.6592f);
                        Wheels[0].suspension = new JointSuspension2D()
                        {
                            frequency = 14f,
                            angle = 90f,
                            dampingRatio = 1f
                        };
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Tractor/Big Wheel.png");
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.8428571f;

                        Wheels[1].anchor = new Vector2(1.6289f, -0.915f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Tractor/Small Wheel.png");
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6428571f;

                        //Chair
                        GameObject Chair = new GameObject("Chair");
                        Chair.transform.parent = Instance.transform;
                        Chair.transform.localRotation = Body.localRotation;
                        Chair.transform.localScale = Vector3.one;
                        Chair.transform.localPosition = new Vector3(-0.676f, 0.129f);
                        SpriteRenderer ChairRenderer = Chair.AddComponent<SpriteRenderer>();
                        ChairRenderer.sprite = ModAPI.LoadSprite("Textures/Tractor/Chair2.png");
                        ChairRenderer.sortingLayerName = "Default";
                        ChairRenderer.sortingOrder = 4;
                        Chair.layer = LayerMask.NameToLayer("Objects");
                        Chair.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D ChairFixedJoint = Chair.AddComponent<FixedJoint2D>();
                        ChairFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        ChairFixedJoint.breakForce = 4000f;
                        Chair.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");
                        Chair.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Chair.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.371428579f,0.571428597f),
                            new Vector2(-0.252832115f,-0.570130467f),
                            new Vector2(0.143671647f,-0.572653174f),
                            new Vector2(0.163793921f,-0.403702319f),
                            new Vector2(0.273466289f,-0.263672799f),
                            new Vector2(0.28938517f,-0.0555137806f),
                            new Vector2(-0.117217183f,-0.0584296882f),
                            new Vector2(-0.193288937f,0.571428597f)
                        });
                        Chair.AddComponent<Optout>();

                        //Body top
                        GameObject Bodyup = new GameObject("Body2");
                        Bodyup.transform.parent = Instance.transform;
                        Bodyup.transform.localRotation = Body.localRotation;
                        Bodyup.transform.localScale = Vector3.one;
                        Bodyup.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer BodyupRenderer = Bodyup.AddComponent<SpriteRenderer>();
                        BodyupRenderer.sprite = ModAPI.LoadSprite("Textures/Tractor/Background2.png");
                        BodyupRenderer.sortingLayerName = "Foreground";
                        BodyupRenderer.sortingOrder = 2;
                        Bodyup.layer = LayerMask.NameToLayer("Objects");
                        Bodyup.AddComponent<Rigidbody2D>().mass = 3f;
                        FixedJoint2D BodyupFixedJoint = Bodyup.AddComponent<FixedJoint2D>();
                        BodyupFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        BodyupFixedJoint.breakForce = 10000f;
                        Bodyup.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Bodyup.AddComponent<AudioSourceTimeScaleBehaviour>();
                        PolygonCollider2D Body2Collider = Bodyup.AddComponent<PolygonCollider2D>();
                        Body2Collider.pathCount = 1;
                        Body2Collider.SetPath(0, new Vector2[]
                        {
                            new Vector2(-1.32989931f, 0.245258451f),
                            new Vector2(-1.31401157f, -0.119970784f),
                            new Vector2(-1.1847837f, -0.373874247f),
                            new Vector2(-0.935878336f, -0.51474005f),
                            new Vector2(-0.444180042f, -0.527248561f),
                            new Vector2(-0.445675671f, -0.44495818f),
                            new Vector2(-0.89308852f, -0.432906836f),
                            new Vector2(-1.14483106f, -0.268992245f),
                            new Vector2(-1.2331506f, -0.0274081677f),
                            new Vector2(-1.23865211f, 0.245118275f)
                        });
                        Bodyup.AddComponent<Optout>();

                        //Hood
                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Bonnet.GetComponent<SpriteRenderer>().sortingOrder = 2;
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 300f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.7719f, 0.1835f);
                        Bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65f,
                            max = 0f
                        };
                        Bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.744668961f,0.199194193f),
                            new Vector2(-0.771428585f,0.200000003f),
                            new Vector2(-0.771428585f,-0.200000003f),
                            new Vector2(-0.743995667f,-0.200000003f),
                            new Vector2(-0.745420456f,0.16996865f),
                            new Vector2(0.742544174f,0.0808310658f),
                            new Vector2(0.74133873f,-0.20101735f),
                            new Vector2(0.770750046f,-0.201362044f),
                            new Vector2(0.769486904f,0.0803557485f),
                            new Vector2(0.771428585f,0.113803282f)
                        });

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().pathCount = 2;
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.54285717f, 0.942857146f),
                            new Vector2(-0.544883609f, -0.164130837f),
                            new Vector2(-0.470748156f, -0.209769085f),
                            new Vector2(-0.327474087f, -0.485711157f),
                            new Vector2(-0.166135326f, -0.940544665f),
                            new Vector2(0.54285717f, -0.942857146f),
                            new Vector2(0.54285717f, 0.942857146f)
                        });
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(1, new Vector2[]
                        {
                            new Vector2(0.455191106f,-0.857136786f),
                            new Vector2(-0.112480193f,-0.853988945f),
                            new Vector2(-0.220769942f,-0.495190233f),
                            new Vector2(-0.453954071f,-0.1187874f),
                            new Vector2(-0.456170231f,0.886436403f),
                            new Vector2(0.461254686f,0.882796228f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.228f, 0.413f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 6;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot
                        Instance.transform.GetChild(6).gameObject.SetActive(false); //Back door

                        //Engine
                        Transform Engine = Instance.transform.GetChild(7);
                        Engine.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Engine.GetComponent<SpriteRenderer>().sortingOrder = 1;

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        UnityEngine.Object.Destroy(GasTank.GetComponent<BoxCollider2D>());
                        GasTank.gameObject.AddComponent<BoxCollider2D>();

                        //Serialization
                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(-1.428f, -0.6592f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(1.6289f, -0.915f);
                            FrontDoor.localPosition = new Vector3(-0.271f, 0.413f);
                            Bonnet.localPosition = new Vector3(1.4425f, 0.3857f);
                            Engine.localScale = new Vector3(0.9f, 0.9f);
                            Engine.localPosition = new Vector3(1.132f, 0.158f);
                            GasTank.localScale = new Vector3(0.9f, 0.9f);
                            GasTank.localPosition = new Vector3(0.773f, -0.355f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Chair;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Bodyup;
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Roadroller
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Roadroller",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Whoa. The front wheel is VERY heavy... It could probably run over a human being...",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Roadroller.png"),
                    AfterSpawn = (Instance) =>
                    {
                        //Car speed
                        float MotorSpeed = -800f;

                        //Body
                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 5;
                        Body.GetComponent<PolygonCollider2D>().pathCount = 2;
                        Body.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(2.7951932f, -0.709461331f),
                            new Vector2(0.754169703f, -0.714874983f),
                            new Vector2(0.839129925f, -0.32577914f),
                            new Vector2(1.11050236f, -0.0859619975f),
                            new Vector2(1.10548794f, 0.0751149654f),
                            new Vector2(0.934939682f, 0.0603026748f),
                            new Vector2(0.922068715f, 1.49682581f),
                            new Vector2(1.19047606f, 1.66316593f),
                            new Vector2(1.10769784f, 1.84907091f),
                            new Vector2(-1.1404984f, 1.83109808f),
                            new Vector2(-1.20491207f, 1.62361598f),
                            new Vector2(-0.972346663f, 1.48676133f),
                            new Vector2(-0.979294419f, 0.805262327f),
                            new Vector2(-1.45116949f, 0.579909027f),
                            new Vector2(-1.70350659f, 0.241274953f),
                            new Vector2(-2.19937778f, -0.00113485008f),
                            new Vector2(-2.39267015f, -0.167541683f),
                            new Vector2(-2.53399587f, -0.381219834f),
                            new Vector2(-2.70220041f, -0.985095024f),
                            new Vector2(-2.79982591f, -1.02423334f),
                            new Vector2(-2.80363297f, -1.2055614f),
                            new Vector2(-2.5525589f, -1.20469368f),
                            new Vector2(-2.19107175f, -1.19901979f),
                            new Vector2(-2.12617612f, -0.878451526f),
                            new Vector2(-1.78801632f, -0.528783619f),
                            new Vector2(-0.723331869f, -0.48689276f),
                            new Vector2(-0.684034765f, -0.601267338f),
                            new Vector2(-0.374614179f, -0.592255592f),
                            new Vector2(-0.353236079f, -1.08832479f),
                            new Vector2(2.71553254f, -1.08814991f)
                        });
                        Body.GetComponent<PolygonCollider2D>().SetPath(1, new Vector2[]
                        {
                            new Vector2(-0.888522089f,1.5546174f),
                            new Vector2(-0.878435135f,0.43902117f),
                            new Vector2(-0.627556264f,-0.519756794f),
                            new Vector2(0.533927321f,-0.514439225f),
                            new Vector2(0.827720702f,-0.0468912721f),
                            new Vector2(0.832945585f,1.54831314f)
                        });
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Roadroller/Original/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Roadroller/Original/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Roadroller/Original/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Roadroller/Original/Bonnet.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Roadroller/Orange/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Roadroller/Orange/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Roadroller/Orange/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Roadroller/Orange/Bonnet.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Roadroller/Rusty/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Roadroller/Rusty/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Roadroller/Rusty/Door.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Roadroller/Rusty/Bonnet.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero; //Car bottom position

                        //Wheels
                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].anchor = new Vector2(1.669f, -0.9f);
                        Wheels[0].suspension = new JointSuspension2D()
                        {
                            frequency = 35f,
                            angle = 90f,
                            dampingRatio = 0f
                        };
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Roadroller/Wheel2.png");
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        Wheels[0].connectedBody.GetComponent<PhysicalBehaviour>().InitialMass = 80;
                        Wheels[0].connectedBody.GetComponent<PhysicalBehaviour>().TrueInitialMass = 90;
                        Wheels[0].connectedBody.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Concrete");
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.9285714f;

                        Wheels[1].anchor = new Vector2(-1.5002f, -1.1858f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Roadroller/Wheel.png");
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6428571f;
                       
                        //Tube
                        GameObject Bodyup = new GameObject("Body2");
                        Bodyup.transform.parent = Instance.transform;
                        Bodyup.transform.localRotation = Body.localRotation;
                        Bodyup.transform.localScale = Vector3.one;
                        Bodyup.transform.localPosition = new Vector3(-1.34285f, 1.0141f);
                        SpriteRenderer BodyupRenderer = Bodyup.AddComponent<SpriteRenderer>();
                        BodyupRenderer.sprite = ModAPI.LoadSprite("Textures/Roadroller/Tube.png");
                        BodyupRenderer.sortingLayerName = "Foreground";
                        BodyupRenderer.sortingOrder = 6;
                        Bodyup.layer = LayerMask.NameToLayer("Objects");
                        PolygonCollider2D BodyupCollider = Bodyup.AddComponent<PolygonCollider2D>();
                        BodyupCollider.pathCount = 1;
                        BodyupCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(0.200000003f, -0.528571427f),
                            new Vector2(0.201300502f, 0.0743412971f),
                            new Vector2(-0.0567095727f, 0.0748302341f),
                            new Vector2(-0.0562729836f, 0.441311836f),
                            new Vector2(-0.00480055809f, 0.463728786f),
                            new Vector2(0.00167798996f, 0.528571486f),
                            new Vector2(-0.200000003f, 0.528571427f),
                            new Vector2(-0.200361729f, 0.470871329f),
                            new Vector2(-0.140716791f, 0.43632102f),
                            new Vector2(-0.150383353f, -0.0173110366f),
                            new Vector2(0.120381355f, -0.012593627f),
                            new Vector2(0.113251373f, -0.529860497f)
                        });
                        Bodyup.AddComponent<Rigidbody2D>().mass = 3f;
                        FixedJoint2D BodyupFixedJoint = Bodyup.AddComponent<FixedJoint2D>();
                        BodyupFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        BodyupFixedJoint.breakForce = 3000000f;
                        Bodyup.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Bodyup.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Bodyup.AddComponent<Optout>();

                        //Smoke
                        GameObject Smoke = UnityEngine.Object.Instantiate(ModAPI.FindSpawnable("Particle Projector").Prefab.transform.Find("smoke").gameObject, Bodyup.transform);
                        Smoke.transform.localRotation = Quaternion.identity;
                        Smoke.transform.localScale = Vector3.one;
                        Smoke.transform.localPosition = new Vector3(-0.101f, 0.513f);

                        //Smoke controller
                        SmokeController SmokeController = Body.gameObject.AddComponent<SmokeController>();
                        SmokeController.Smoke = Smoke.GetComponent<ParticleSystem>();

                        ActOnJointBreak BodyupActOnJointBreak = Bodyup.AddComponent<ActOnJointBreak>();
                        BodyupActOnJointBreak.Joint = BodyupFixedJoint;
                        BodyupActOnJointBreak.OnBreak.AddListener(() =>
                        {
                            SmokeController.enabled = false;
                        });

                        //Chair
                        GameObject Chair = new GameObject("Chair");
                        Chair.transform.parent = Instance.transform;
                        Chair.transform.localRotation = Body.localRotation;
                        Chair.transform.localScale = Vector3.one;
                        Chair.transform.localPosition = new Vector3(-0.256f, 0.056f);
                        SpriteRenderer ChairRenderer = Chair.AddComponent<SpriteRenderer>();
                        ChairRenderer.sprite = ModAPI.LoadSprite("Textures/Roadroller/Chair.png");
                        ChairRenderer.sortingLayerName = "Default";
                        ChairRenderer.sortingOrder = 4;
                        Chair.layer = LayerMask.NameToLayer("Objects");
                        Chair.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.371428579f, 0.571428597f),
                            new Vector2(-0.250261158f, -0.571160555f),
                            new Vector2(0.0768283159f, -0.571428597f),
                            new Vector2(0.123448327f, -0.309277505f),
                            new Vector2(0.28686288f, -0.254461765f),
                            new Vector2(0.286862969f, -0.0586752892f),
                            new Vector2(-0.112807915f, -0.0586890578f),
                            new Vector2(-0.142857149f, 0.571428597f)
                        });
                        Chair.AddComponent<Rigidbody2D>().mass = 1f;
                        FixedJoint2D ChairFixedJoint = Chair.AddComponent<FixedJoint2D>();
                        ChairFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        ChairFixedJoint.breakForce = 4000f;
                        Chair.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");
                        Chair.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Chair.AddComponent<Optout>();

                        //Lamp
                        GameObject Lamp = new GameObject("Lamp");
                        Lamp.transform.parent = Instance.transform;
                        Lamp.transform.localRotation = Body.localRotation;
                        Lamp.transform.localScale = Vector3.one;
                        Lamp.transform.localPosition = new Vector3(2.7003f, -0.6436f);
                        SpriteRenderer LampRenderer = Lamp.AddComponent<SpriteRenderer>();
                        LampRenderer.sprite = ModAPI.LoadSprite("Textures/Roadroller/Lamp.png");
                        LampRenderer.sortingLayerName = "Foreground";
                        LampRenderer.sortingOrder = 2;
                        Lamp.layer = LayerMask.NameToLayer("Objects");
                        Lamp.AddComponent<PolygonCollider2D>();
                        Lamp.AddComponent<Rigidbody2D>().mass = 3f;
                        FixedJoint2D LampFixedJoint = Lamp.AddComponent<FixedJoint2D>();
                        LampFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        LampFixedJoint.breakForce = 3000f;
                        Lamp.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Lamp.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Lamp.AddComponent<Optout>();

                        //Hood
                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Bonnet.GetComponent<SpriteRenderer>().sortingOrder = -3;
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 300f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(0.3281f, 0.2996f);
                        Bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = 65f,
                            max = 0f
                        };
                        Bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.269399315f,0.312141925f),
                            new Vector2(0.0229640007f,0.195811376f),
                            new Vector2(-0.126249552f,0.0596507043f),
                            new Vector2(-0.274755716f,-0.148065522f),
                            new Vector2(-0.302155256f,-0.256331474f),
                            new Vector2(-0.328064203f,-0.257774532f),
                            new Vector2(-0.328832626f,-0.285708934f),
                            new Vector2(-0.300384521f,-0.286672115f),
                            new Vector2(-0.299744725f,-0.314285725f),
                            new Vector2(-0.242114589f,-0.314285725f),
                            new Vector2(-0.213366508f,-0.233014286f),
                            new Vector2(-0.0965764523f,-0.0249013752f),
                            new Vector2(0.0214059353f,0.121877357f),
                            new Vector2(0.146088839f,0.195813805f),
                            new Vector2(0.324811459f,0.252640307f),
                            new Vector2(0.324811459f,0.252640307f)
                        });

                        //Front door
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.714285731f,1.11428571f),
                            new Vector2(-0.712338448f,-0.0786612034f),
                            new Vector2(-0.421453029f,-1.11428571f),
                            new Vector2(0.59872359f,-1.11233842f),
                            new Vector2(0.712550998f,-0.456690103f),
                            new Vector2(0.714285731f,1.11428571f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.668f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 6;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        Instance.transform.GetChild(6).gameObject.SetActive(false); //Back door
                        Instance.transform.GetChild(4).gameObject.SetActive(false); //Boot

                        //Gastank
                        Transform GasTank = Instance.transform.GetChild(8);
                        UnityEngine.Object.Destroy(GasTank.GetComponent<BoxCollider2D>());
                        GasTank.gameObject.AddComponent<BoxCollider2D>();
                        
                        //Serialization
                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            //Parts position
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(1.669f, -0.9f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(-1.5002f, -1.1858f);
                            Bonnet.localPosition = new Vector3(-2.242f, -0.203f);
                            FrontDoor.localPosition = new Vector3(-0.25714f, 0.51429f);
                            GasTank.localScale = new Vector3(0.9f, 0.9f);
                            GasTank.localPosition = new Vector3(-0.576f, -0.791f);

                            //Engine
                            Transform Engine = Instance.transform.GetChild(7);
                            Engine.localScale = new Vector3(0.9f, 0.9f);
                            Engine.localPosition = new Vector3(-1.558f, -0.354f);

                            //Serializing objects
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Bodyup;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Chair;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Lamp;
                        };

                        //Not colliding car parts with each other
                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Excavator
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Excavator",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Construction vehicle. Always been on the construction site. Has an impressive scoop with which you can rake up anything and anyone.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Excavator.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/Excavator/Wheel.png");
                        Sprite SteeringWheelSprite = ModAPI.LoadSprite("Textures/Excavator/SteeringWheel.png");
                        Sprite ScoopStartSprite = ModAPI.LoadSprite("Textures/Excavator/Scoop.png");
                        Sprite ScoopStartContSprite = ModAPI.LoadSprite("Textures/Excavator/Scoop2.png");
                        Sprite ScoopTwoSprite = ModAPI.LoadSprite("Textures/Excavator/Scoop3.png");
                        Sprite ScoopEndSprite = ModAPI.LoadSprite("Textures/Excavator/Scoop4.png");

                        AudioClip Horn = ModAPI.LoadSound("Sfx/Truck Horn.mp3");

                        float MotorSpeed = -700f;

                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 38f;
                        BodyPhysical.InitialMass = 38f;
                        BodyPhysical.rigidbody.mass = 38f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 4;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(0.457311898f, -1.51544213f),
                            new Vector2(-0.457762837f, -1.51680744f),
                            new Vector2(-0.724932015f, -1.26937604f),
                            new Vector2(-0.622572422f, -1.05809438f),
                            new Vector2(0.312453747f, -1.05424762f),
                            new Vector2(0.375598073f, -1.1175164f),
                            new Vector2(0.714863658f, -1.11870968f),
                            new Vector2(0.715001881f, -0.972522259f),
                            new Vector2(0.598261714f, -0.927787602f),
                            new Vector2(0.597571969f, -0.807503104f),
                            new Vector2(0.496329308f, -0.680097163f),
                            new Vector2(0.58307457f, -0.441309601f),
                            new Vector2(0.584983468f, 0.511314034f),
                            new Vector2(0.407286584f, 1.18443429f),
                            new Vector2(0.0665832162f, 1.59031355f),
                            new Vector2(-0.482873082f, 1.91175067f),
                            new Vector2(-2.24993849f, 1.91552901f),
                            new Vector2(-2.45217443f, 1.71056044f),
                            new Vector2(-2.45491529f, 0.712240219f),
                            new Vector2(-2.63507652f, 0.793397963f),
                            new Vector2(-2.69178653f, 1.03712142f),
                            new Vector2(-2.84420347f, 1.15482247f),
                            new Vector2(-2.85318971f, 1.00966918f),
                            new Vector2(-2.76415205f, 0.78576225f),
                            new Vector2(-3.97651505f, 0.687210798f),
                            new Vector2(-4.1353035f, 0.409050196f),
                            new Vector2(-4.19179726f, -0.778176665f),
                            new Vector2(-4.31794834f, -0.941690922f),
                            new Vector2(-4.25430155f, -1.1228528f),
                            new Vector2(-3.76415873f, -1.11113226f),
                            new Vector2(-3.70195556f, -1.0577625f),
                            new Vector2(-1.89632559f, -1.06646752f),
                            new Vector2(-1.79048669f, -1.28220987f),
                            new Vector2(-2.06068516f, -1.51092625f),
                            new Vector2(-2.97097683f, -1.51609337f),
                            new Vector2(-2.97274804f, -1.77184391f),
                            new Vector2(0.456664383f, -1.76886761f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(-2.37668729f, 0.573726714f),
                            new Vector2(-2.1795764f, 0.573500514f),
                            new Vector2(-2.26531482f, 0.768477023f),
                            new Vector2(-2.13666534f, 0.872724712f),
                            new Vector2(-1.83476901f, 0.885102451f),
                            new Vector2(-1.65210438f, 0.752197444f),
                            new Vector2(-1.74525607f, 0.572363079f),
                            new Vector2(-1.37034714f, 0.57224232f),
                            new Vector2(-1.36972284f, 1.30156052f),
                            new Vector2(-1.25429189f, 1.50557113f),
                            new Vector2(-1.05752587f, 1.56385279f),
                            new Vector2(-1.0457685f, 1.83073938f),
                            new Vector2(-2.24171877f, 1.8285569f),
                            new Vector2(-2.37008286f, 1.70683837f)
                        });
                        BodyCollider.SetPath(2, new Vector2[]
                        {
                            new Vector2(-0.889126301f, 0.421543092f),
                            new Vector2(-0.816859365f, -0.213629067f),
                            new Vector2(-0.364483058f, -0.19806394f),
                            new Vector2(-0.326066762f, -0.288375139f),
                            new Vector2(-0.560663402f, -0.588514686f),
                            new Vector2(-0.563557565f, -0.717738211f),
                            new Vector2(-0.97105962f, -0.724698186f),
                            new Vector2(-1.08719444f, 0.421228409f)
                        });
                        BodyCollider.SetPath(3, new Vector2[]
                        {
                            new Vector2(-0.985927582f, -0.729322493f),
                            new Vector2(0.248799384f, -0.743390799f),
                            new Vector2(0.370417684f, -0.650829017f),
                            new Vector2(0.487420112f, -0.365957975f),
                            new Vector2(0.483238995f, 0.442987204f),
                            new Vector2(0.23895523f, 1.26471043f),
                            new Vector2(-0.0395359993f, 1.50455284f),
                            new Vector2(-1.12748742f, 1.47489667f),
                            new Vector2(-1.27080142f, 1.30958498f),
                            new Vector2(-1.25246418f, -0.138777316f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Excavator/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Excavator/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Excavator/Door.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero;

                        GameObject Smoke = UnityEngine.Object.Instantiate(ModAPI.FindSpawnable("Particle Projector").Prefab.transform.Find("smoke").gameObject, Body);
                        Smoke.transform.localRotation = Quaternion.identity;
                        Smoke.transform.localScale = Vector3.one;
                        Smoke.transform.localPosition = new Vector3(-2.855f, 1.076f);

                        SmokeController SmokeController = Body.gameObject.AddComponent<SmokeController>();
                        SmokeController.Smoke = Smoke.GetComponent<ParticleSystem>();

                        Instance.transform.GetChild(3).gameObject.SetActive(false);
                        Instance.transform.GetChild(4).gameObject.SetActive(false);
                        Instance.transform.GetChild(6).gameObject.SetActive(false);

                        GameObject ScoopStart = new GameObject("Scoop Start");
                        ScoopStart.transform.parent = Instance.transform;
                        ScoopStart.transform.localRotation = Body.localRotation;
                        ScoopStart.transform.localScale = Vector3.one;
                        ScoopStart.transform.localPosition = new Vector3(1.679f, 0.276f);
                        SpriteRenderer ScoopStartRenderer = ScoopStart.AddComponent<SpriteRenderer>();
                        ScoopStartRenderer.sprite = ScoopStartSprite;
                        ScoopStartRenderer.sortingLayerName = "Default";
                        ScoopStartRenderer.sortingOrder = 1;
                        ScoopStart.layer = 10;
                        ScoopStart.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-1.03515446f, 0.483374864f),
                            new Vector2(-2.08454442f, -0.582055807f),
                            new Vector2(-2.15177941f, -0.856707335f),
                            new Vector2(-1.96931446f, -1.10862005f),
                            new Vector2(-1.59536624f, -1.13298476f),
                            new Vector2(-0.578546762f, 0.0278372616f)
                        });
                        ScoopStart.AddComponent<Rigidbody2D>().mass = 0.5f;
                        HingeJoint2D ScoopStartHingeJoint = ScoopStart.AddComponent<HingeJoint2D>();
                        ScoopStartHingeJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        ScoopStartHingeJoint.anchor = new Vector2(-1.768f, -0.79f);
                        ScoopStartHingeJoint.useMotor = true;
                        ScoopStartHingeJoint.limits = new JointAngleLimits2D()
                        {
                            min = -35f,
                            max = 45f
                        };
                        ScoopStartHingeJoint.motor = new JointMotor2D
                        {
                            maxMotorTorque = 455f
                        };
                        ScoopStart.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        ScoopStart.AddComponent<AudioSourceTimeScaleBehaviour>();
                        ScoopStart.AddComponent<Optout>();

                        GameObject ScoopStartCont = new GameObject("Scoop Cont Start");
                        ScoopStartCont.transform.parent = ScoopStart.transform;
                        ScoopStartCont.transform.localRotation = Quaternion.identity;
                        ScoopStartCont.transform.localScale = Vector3.one;
                        ScoopStartCont.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer ScoopStartContRenderer = ScoopStartCont.AddComponent<SpriteRenderer>();
                        ScoopStartContRenderer.sprite = ScoopStartContSprite;
                        ScoopStartContRenderer.sortingLayerName = "Default";
                        ScoopStartContRenderer.sortingOrder = 1;
                        ScoopStartCont.layer = LayerMask.NameToLayer("Objects");
                        ScoopStartCont.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(1.91027439f, 0.946746171f),
                            new Vector2(2.16322184f, 1.12273777f),
                            new Vector2(1.94723666f, 1.24905837f),
                            new Vector2(1.47236383f, 1.25633383f),
                            new Vector2(1.03661549f, 1.28931177f),
                            new Vector2(-0.532656908f, 0.954496384f),
                            new Vector2(-1.06642199f, 0.509148002f),
                            new Vector2(-0.58346355f, 0.0243760329f),
                            new Vector2(-0.202725321f, 0.453437537f),
                            new Vector2(0.512472987f, 0.758141518f),
                            new Vector2(1.61635292f, 0.922308981f)
                        });
                        ScoopStartCont.AddComponent<Rigidbody2D>().mass = 0.2f;
                        ScoopStartCont.AddComponent<FixedJoint2D>().connectedBody = ScoopStart.GetComponent<Rigidbody2D>();
                        ScoopStartCont.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        ScoopStartCont.AddComponent<AudioSourceTimeScaleBehaviour>();
                        ScoopStartCont.AddComponent<Optout>();

                        GameObject ScoopTwo = new GameObject("Scoop Two");
                        ScoopTwo.transform.parent = ScoopStartCont.transform;
                        ScoopTwo.transform.localRotation = Quaternion.identity;
                        ScoopTwo.transform.localScale = Vector3.one;
                        ScoopTwo.transform.localPosition = new Vector3(2.216f, 0.355f);
                        SpriteRenderer ScoopTwoRenderer = ScoopTwo.AddComponent<SpriteRenderer>();
                        ScoopTwoRenderer.sprite = ScoopTwoSprite;
                        ScoopTwoRenderer.sortingLayerName = "Default";
                        ScoopTwoRenderer.sortingOrder = 1;
                        ScoopTwo.layer = LayerMask.NameToLayer("Objects");
                        ScoopTwo.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.264154375f,1.2799691f),
                            new Vector2(0.184764296f,1.42978394f),
                            new Vector2(0.0965173244f,1.38739228f),
                            new Vector2(0.0859553814f,1.27734923f),
                            new Vector2(-0.158473253f,0.959566414f),
                            new Vector2(-0.28339529f,0.568615913f),
                            new Vector2(-0.396046638f,-1.3735826f),
                            new Vector2(-0.154084951f,-1.41428578f),
                            new Vector2(0.386239201f,0.876161575f)
                        });
                        ScoopTwo.AddComponent<Rigidbody2D>().mass = 0.5f;
                        HingeJoint2D ScoopTwoHingeJoint = ScoopTwo.AddComponent<HingeJoint2D>();
                        ScoopTwoHingeJoint.connectedBody = ScoopStartCont.GetComponent<Rigidbody2D>();
                        ScoopTwoHingeJoint.anchor = new Vector2(-0.1747f, 0.7438f);
                        ScoopTwoHingeJoint.useMotor = true;
                        ScoopTwoHingeJoint.limits = new JointAngleLimits2D()
                        {
                            min = -90f,
                            max = 25f
                        };
                        ScoopTwoHingeJoint.motor = new JointMotor2D
                        {
                            maxMotorTorque = 355f
                        };
                        ScoopTwo.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        ScoopTwo.AddComponent<AudioSourceTimeScaleBehaviour>();
                        ScoopTwo.AddComponent<Optout>();

                        GameObject ScoopEnd = new GameObject("Scoop End");
                        ScoopEnd.transform.parent = ScoopTwo.transform;
                        ScoopEnd.transform.localRotation = Quaternion.identity;
                        ScoopEnd.transform.localScale = Vector3.one;
                        ScoopEnd.transform.localPosition = new Vector3(-0.705f, -1.539f);
                        SpriteRenderer ScoopEndRenderer = ScoopEnd.AddComponent<SpriteRenderer>();
                        ScoopEndRenderer.sprite = ScoopEndSprite;
                        ScoopEndRenderer.sortingLayerName = "Foreground";
                        ScoopEndRenderer.sortingOrder = 7;
                        ScoopEnd.layer = LayerMask.NameToLayer("Objects");
                        ScoopEnd.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.442710876f,0.323504388f),
                            new Vector2(-0.46084556f,0.448286206f),
                            new Vector2(-0.514285743f,0.45714286f),
                            new Vector2(-0.463836432f,-0.0384716392f),
                            new Vector2(-0.414648712f,-0.241431147f),
                            new Vector2(-0.287077814f,-0.410645515f),
                            new Vector2(0.0305790901f,-0.443275213f),
                            new Vector2(0.306701422f,-0.399604559f),
                            new Vector2(0.424143285f,-0.255654365f),
                            new Vector2(0.483625889f,-0.109167099f),
                            new Vector2(0.52267313f,0.138899386f),
                            new Vector2(0.499839067f,0.253913939f),
                            new Vector2(0.343293905f,0.259788215f),
                            new Vector2(0.464371681f,0.103437066f),
                            new Vector2(0.425692797f,-0.0790711641f),
                            new Vector2(0.349113464f,-0.282663584f),
                            new Vector2(0.193963766f,-0.379346013f),
                            new Vector2(-0.136676848f,-0.387565643f),
                            new Vector2(-0.329308808f,-0.316749722f),
                            new Vector2(-0.420815259f,-0.0271200836f)
                        });
                        ScoopEnd.AddComponent<Rigidbody2D>().mass = 0.5f;
                        HingeJoint2D ScoopEndHingeJoint = ScoopEnd.AddComponent<HingeJoint2D>();
                        ScoopEndHingeJoint.connectedBody = ScoopTwo.GetComponent<Rigidbody2D>();
                        ScoopEndHingeJoint.anchor = new Vector2(0.423f, 0.178f);
                        ScoopEndHingeJoint.useMotor = true;
                        ScoopEndHingeJoint.limits = new JointAngleLimits2D()
                        {
                            min = -90f,
                            max = 30f
                        };
                        ScoopEndHingeJoint.motor = new JointMotor2D
                        {
                            maxMotorTorque = 455f
                        };
                        ScoopEnd.GetComponent<Rigidbody2D>().rotation = ScoopEnd.transform.rotation.z;
                        ScoopEnd.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        ScoopEnd.AddComponent<AudioSourceTimeScaleBehaviour>();
                        ScoopEnd.AddComponent<Optout>();

                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";
                        Bonnet.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 500f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.57f, 0.126f);
                        Bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65f,
                            max = 0f
                        };
                        Bonnet.localPosition = new Vector3(2.441f, 0.243f);
                        Bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.60437727f,-0.121885993f),
                            new Vector2(0.337554693f,0.0142583549f),
                            new Vector2(-0.600000024f,0.157142863f),
                            new Vector2(-0.600000024f,0.096889019f),
                            new Vector2(0.228675604f,-0.0248475224f),
                            new Vector2(0.587595582f,-0.163000911f)
                        });

                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.gameObject.AddComponent<BoxCollider2D>();
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.671428561f,1.14285719f),
                            new Vector2(-0.671428561f,-1.14285719f),
                            new Vector2(0.415916562f,-1.14209235f),
                            new Vector2(0.546264589f,-1.0791961f),
                            new Vector2(0.671428561f,-0.754637837f),
                            new Vector2(0.668222725f,0.0746456981f),
                            new Vector2(0.508560777f,0.694531262f),
                            new Vector2(0.227234617f,1.13579857f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.675f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].suspension = new JointSuspension2D()
                        {
                            frequency = 35f,
                            angle = 90f,
                            dampingRatio = 0f
                        };
                        Wheels[0].anchor = new Vector2(-2.8134f, -1.6425f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4142857f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[0].connectedBody.GetComponent<PhysicalBehaviour>().InitialMass = 1;
                        Wheels[0].connectedBody.GetComponent<PhysicalBehaviour>().TrueInitialMass = 1;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";
                        Wheels[0].connectedBody.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");

                        Wheels[1].suspension = new JointSuspension2D()
                        {
                            frequency = 35f,
                            angle = 90f,
                            dampingRatio = 0f
                        };
                        Wheels[1].anchor = new Vector2(0.301f, -1.6425f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4142857f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[1].connectedBody.GetComponent<PhysicalBehaviour>().InitialMass = 1;
                        Wheels[1].connectedBody.GetComponent<PhysicalBehaviour>().TrueInitialMass = 1;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";
                        Wheels[1].connectedBody.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");

                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            GasTank.localScale = new Vector3(0.9f, 0.9f);
                            GasTank.localPosition = new Vector3(-1.83f, -0.48f);
                            FrontDoor.localPosition = new Vector3(-0.1581f, 0.371f);
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(-2.8134f, -1.6425f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(0.301f, -1.6425f);

                            Transform Engine = Instance.transform.GetChild(7);
                            Engine.localScale = new Vector3(0.9f, 0.9f);
                            Engine.localPosition = new Vector3(-3.5f, -0.26f);

                            Instance.AddComponent<ObjectSerialisationHelper>().Object = ScoopStart;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = ScoopStartCont;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = ScoopTwo;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = ScoopEnd;
                        };

                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Bulldozer
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Bulldozer",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "It's a very heavy machine. It's supposed to be a garbage scraper, but that's a lie here.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Bulldozer.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite WheelSprite = ModAPI.LoadSprite("Textures/Bulldozer/Wheel.png");
                        Sprite SharpenerSprite = ModAPI.LoadSprite("Textures/Bulldozer/Sharpener.png");
                        Sprite SpoonSprite = ModAPI.LoadSprite("Textures/Bulldozer/Spoon.png");

                        float MotorSpeed = -400f;
                        
                        Transform Body = Instance.transform.GetChild(0);
                        CarSmoothBrakingBehaviour SmoothBraking = Body.gameObject.AddComponent<CarSmoothBrakingBehaviour>();
                        float defaultMotorTorque = 15050f; //это сила, ее меняй
                        SmoothBraking.DefaultTorque = defaultMotorTorque; 
                        SmoothBraking.SmoothBrakingTorque = 50f;
                        SmoothBraking.SpeedForFullBraking = 400f;

                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 25;
                        BodyPhysical.InitialMass = 25;
                        BodyPhysical.rigidbody.mass = 25;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 2;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(3.64924979f,-0.411028504f),
                            new Vector2(3.11318755f,0.828426778f),
                            new Vector2(3.10256481f,1.08424938f),
                            new Vector2(2.73824072f,1.02582657f),
                            new Vector2(2.65629053f,0.59914583f),
                            new Vector2(1.77953076f,0.686564565f),
                            new Vector2(1.77199864f,0.769411623f),
                            new Vector2(1.48863769f,0.77520752f),
                            new Vector2(1.49559319f,0.686176717f),
                            new Vector2(1.26151252f,0.687855422f),
                            new Vector2(1.25522482f,1.17133677f),
                            new Vector2(1.1475482f,1.20383704f),
                            new Vector2(1.13849413f,0.685657859f),
                            new Vector2(0.771498799f,0.692498982f),
                            new Vector2(0.768940747f,1.85739243f),
                            new Vector2(-1.62763786f,1.86022818f),
                            new Vector2(-1.69048464f,1.7428571f),
                            new Vector2(-1.60212648f,1.74288416f),
                            new Vector2(-1.60477042f,0.742390335f),
                            new Vector2(-1.78335524f,0.602066934f),
                            new Vector2(-2.72522712f,0.487399369f),
                            new Vector2(-2.90470862f,0.246678367f),
                            new Vector2(-2.94762778f,-0.315686077f),
                            new Vector2(-3.45260572f,0.28216213f),
                            new Vector2(-3.64956188f,0.262365133f),
                            new Vector2(-4.26746941f,-0.162320524f),
                            new Vector2(-4.38119364f,-0.10945946f),
                            new Vector2(-4.57537365f,-0.43561852f),
                            new Vector2(-4.52747488f,-0.54406935f),
                            new Vector2(-4.16376925f,-0.718613744f),
                            new Vector2(-3.95607352f,-0.593076706f),
                            new Vector2(-3.1269629f,-1.06694746f),
                            new Vector2(-2.55529642f,-0.495184511f),
                            new Vector2(-2.06854773f,-0.252972573f),
                            new Vector2(-1.5647074f,-0.233599663f),
                            new Vector2(-0.321561009f,-0.711336136f),
                            new Vector2(-0.0726725012f,-0.942294121f),
                            new Vector2(-0.857166708f,-0.917110741f),
                            new Vector2(-1.20505738f,-1.17246556f),
                            new Vector2(-2.43257213f,-1.17485356f),
                            new Vector2(-2.27573252f,-1.59970748f),
                            new Vector2(3.64584708f,-1.55086899f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(-0.30536285f,0.768217146f),
                            new Vector2(-0.23300989f,0.125136822f),
                            new Vector2(0.211628705f,0.137213886f),
                            new Vector2(0.276048064f,0.0168052912f),
                            new Vector2(0.0260516256f,-0.21435377f),
                            new Vector2(0.03046529f,-0.425805926f),
                            new Vector2(0.716168225f,-0.42977649f),
                            new Vector2(0.721524656f,1.68469477f),
                            new Vector2(-0.39767009f,1.68582892f),
                            new Vector2(-0.412457258f,0.77954483f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Bulldozer/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Bulldozer/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Bulldozer/Door.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero;

                        GameObject Smoke = UnityEngine.Object.Instantiate(ModAPI.FindSpawnable("Particle Projector").Prefab.transform.Find("smoke").gameObject, Body);
                        Smoke.transform.localRotation = Quaternion.identity;
                        Smoke.transform.localScale = Vector3.one;
                        Smoke.transform.localPosition = new Vector3(1.202f, 1.163f);

                        SmokeController SmokeController = Body.gameObject.AddComponent<SmokeController>();
                        SmokeController.Smoke = Smoke.GetComponent<ParticleSystem>();

                        GameObject Sharpener = new GameObject("Sharpener");
                        Sharpener.transform.parent = Instance.transform;
                        Sharpener.transform.localRotation = Body.localRotation;
                        Sharpener.transform.localScale = Vector3.one;
                        Sharpener.transform.localPosition = new Vector3(-4.242f, -0.985f);
                        SpriteRenderer SharpenerRenderer = Sharpener.AddComponent<SpriteRenderer>();
                        SharpenerRenderer.sprite = SharpenerSprite;
                        SharpenerRenderer.sortingLayerName = "Bottom";
                        SharpenerRenderer.sortingOrder = 1;
                        Sharpener.layer = LayerMask.NameToLayer("Objects");
                        Sharpener.AddComponent<Rigidbody2D>().mass = 0.5f;
                        FixedJoint2D SharpenerFixedJoint = Sharpener.AddComponent<FixedJoint2D>();
                        SharpenerFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        SharpenerFixedJoint.breakForce = 3900f;
                        Sharpener.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.236291707f,0.428571433f),
                            new Vector2(-0.303747475f,0.0679633915f),
                            new Vector2(-0.157128334f,-0.271174759f),
                            new Vector2(0.236917257f,-0.429259777f),
                            new Vector2(0.30878526f,-0.413581342f),
                            new Vector2(0.0348849297f,-0.244985104f),
                            new Vector2(-0.0123038292f,-0.108023763f),
                            new Vector2(-0.0136146545f,0.0592569113f),
                            new Vector2(0.0357983708f,0.300733089f)
                        });
                        Sharpener.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Sharpener.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Sharpener.AddComponent<Optout>();

                        GameObject Spoon = new GameObject("Spoon");
                        Spoon.transform.parent = Instance.transform;
                        Spoon.transform.localRotation = Body.localRotation;
                        Spoon.transform.localScale = Vector3.one;
                        Spoon.transform.localPosition = new Vector3(4.094958f, -0.8699999f);
                        SpriteRenderer SpoonRenderer = Spoon.AddComponent<SpriteRenderer>();
                        SpoonRenderer.sprite = SpoonSprite;
                        SpoonRenderer.sortingLayerName = "Foreground";
                        SpoonRenderer.sortingOrder = 5;
                        Spoon.layer = LayerMask.NameToLayer("Objects");
                        Spoon.AddComponent<Rigidbody2D>().mass = 50f;
                        FixedJoint2D SpoonFixedJoint = Spoon.AddComponent<FixedJoint2D>();
                        SpoonFixedJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        SpoonFixedJoint.breakForce = Mathf.Infinity;
                        Spoon.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.208636343f,0.88033998f),
                            new Vector2(-0.315330505f,0.729499698f),
                            new Vector2(-0.468741596f,0.628296375f),
                            new Vector2(-0.474928081f,-0.730588555f),
                            new Vector2(-0.0254853368f,-0.819225788f),
                            new Vector2(0.344180584f,-0.830120087f),
                            new Vector2(0.361623943f,-0.804614723f),
                            new Vector2(-0.0601143241f,-0.757765472f),
                            new Vector2(-0.401963711f,-0.637595892f),
                            new Vector2(-0.412717581f,0.576529741f),
                            new Vector2(-0.192993402f,0.733778775f),
                            new Vector2(-0.184289724f,0.885714293f)
                        });
                        Spoon.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        Spoon.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Spoon.AddComponent<Optout>();

                        Instance.transform.GetChild(3).gameObject.SetActive(false);
                        Instance.transform.GetChild(4).gameObject.SetActive(false);
                        Instance.transform.GetChild(6).gameObject.SetActive(false);

                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.614285707f,1.11428571f),
                            new Vector2(-0.607073903f,-0.166480035f),
                            new Vector2(-0.495646954f,-1.12108326f),
                            new Vector2(0.614285707f,-1.11428571f),
                            new Vector2(0.614285707f,1.11428571f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.557f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].suspension = new JointSuspension2D()
                        {
                            frequency = 50f,
                            angle = 90f,
                            dampingRatio = 15f
                        };
                        JointMotor2D motor = Wheels[0].motor;
                        motor.maxMotorTorque = defaultMotorTorque;
                        Wheels[0].motor = motor;
                        Wheels[0].breakForce = 50000f;
                        Wheels[0].anchor = new Vector2(-2.672f, -1.357f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4142857f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[0].connectedBody.GetComponent<PhysicalBehaviour>().InitialMass = 20;
                        Wheels[0].connectedBody.GetComponent<PhysicalBehaviour>().TrueInitialMass = 20;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";
                        Wheels[0].connectedBody.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");

                        Wheels[1].suspension = new JointSuspension2D()
                        {
                            frequency = 50f,
                            angle = 90f,
                            dampingRatio = 15f
                        };
                        Wheels[1].motor = motor;
                        Wheels[1].anchor = new Vector2(2.413f, -1.357f);
                        Wheels[1].breakForce = 50000f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = WheelSprite;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4142857f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = -2;
                        Wheels[1].connectedBody.GetComponent<PhysicalBehaviour>().InitialMass = 20;
                        Wheels[1].connectedBody.GetComponent<PhysicalBehaviour>().TrueInitialMass = 20;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Background";
                        Wheels[1].connectedBody.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");

                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            GasTank.localScale = new Vector3(0.9f, 0.9f);
                            GasTank.localPosition = new Vector3(-1.83f, -0.48f);
                            FrontDoor.localPosition = new Vector3(0.157f, 0.6285f);
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(-2.672f, -1.357f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(2.413f, -1.357f);

                            Transform Engine = Instance.transform.GetChild(7);
                            Engine.localScale = new Vector3(0.9f, 0.9f);
                            Engine.localPosition = new Vector3(-1.38f, 0.17f);

                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Sharpener;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = Spoon;
                        };

                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Wheel Loader
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Wheel Loader",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Fast, lighter version of the excavator. Has a small bucket at the front.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Wheel Loader.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite Wheel = ModAPI.LoadSprite("Textures/Wheel Loader/Wheel.png");
                        Sprite ScoopTwoSprite = ModAPI.LoadSprite("Textures/Wheel Loader/ScoopHand.png");
                        Sprite ScoopEndSprite = ModAPI.LoadSprite("Textures/Wheel Loader/Scoop.png");

                        float MotorSpeed = -900f;

                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 38f;
                        BodyPhysical.InitialMass = 38f;
                        BodyPhysical.rigidbody.mass = 38f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 2;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(1.12565517f,0.0400791168f),
                            new Vector2(0.748205543f,0.0384307876f),
                            new Vector2(0.675253689f,1.675614f),
                            new Vector2(0.907101035f,1.87561393f),
                            new Vector2(0.849958181f,1.97916257f),
                            new Vector2(-1.02458298f,1.98548353f),
                            new Vector2(-1.07874012f,1.876297f),
                            new Vector2(-0.836788774f,1.70303464f),
                            new Vector2(-0.924376488f,0.352225006f),
                            new Vector2(-2.11714745f,0.295965195f),
                            new Vector2(-3.16496086f,0.131784558f),
                            new Vector2(-3.18642449f,-0.758746684f),
                            new Vector2(-3.3282938f,-0.783864975f),
                            new Vector2(-3.17238855f,-1.20227313f),
                            new Vector2(-2.53564191f,-1.35653698f),
                            new Vector2(2.10425973f,-1.35289204f),
                            new Vector2(2.10360527f,-0.924911499f),
                            new Vector2(1.9322865f,-0.574242711f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(-0.749691963f,1.79417801f),
                            new Vector2(-0.821878433f,-0.440877914f),
                            new Vector2(-0.443644524f,-0.442975998f),
                            new Vector2(-0.602838516f,0.697378159f),
                            new Vector2(-0.46991539f,0.739988327f),
                            new Vector2(-0.362183571f,0.0527839661f),
                            new Vector2(0.111651421f,0.0543460846f),
                            new Vector2(-0.152323723f,-0.440231323f),
                            new Vector2(0.670475006f,-0.430030823f),
                            new Vector2(0.582745552f,1.79129028f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Wheel Loader/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Wheel Loader/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Wheel Loader/Door.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero;

                        GameObject Smoke = UnityEngine.Object.Instantiate(ModAPI.FindSpawnable("Particle Projector").Prefab.transform.Find("smoke").gameObject, Body);
                        Smoke.transform.localRotation = Quaternion.identity;
                        Smoke.transform.localScale = Vector3.one;
                        Smoke.transform.localPosition = new Vector3(-1.58f, 0.627f);

                        SmokeController SmokeController = Body.gameObject.AddComponent<SmokeController>();
                        SmokeController.Smoke = Smoke.GetComponent<ParticleSystem>();

                        Instance.transform.GetChild(3).gameObject.SetActive(false);
                        Instance.transform.GetChild(4).gameObject.SetActive(false);
                        Instance.transform.GetChild(6).gameObject.SetActive(false);

                        GameObject ScoopTwo = new GameObject("Scoop Two");
                        ScoopTwo.transform.parent = Instance.transform;
                        ScoopTwo.transform.localRotation = Body.localRotation;
                        ScoopTwo.transform.localScale = Vector3.one;
                        ScoopTwo.transform.localPosition = new Vector3(1.779f, -0.827f);
                        SpriteRenderer ScoopTwoRenderer = ScoopTwo.AddComponent<SpriteRenderer>();
                        ScoopTwoRenderer.sprite = ScoopTwoSprite;
                        ScoopTwoRenderer.sortingLayerName = "Default";
                        ScoopTwoRenderer.sortingOrder = 1;
                        ScoopTwo.layer = LayerMask.NameToLayer("Objects");
                        ScoopTwo.AddComponent<Rigidbody2D>().mass = 5.5f;
                        HingeJoint2D ScoopTwoHingeJoint = ScoopTwo.AddComponent<HingeJoint2D>();
                        ScoopTwoHingeJoint.connectedBody = Body.GetComponent<Rigidbody2D>();
                        ScoopTwoHingeJoint.anchor = new Vector2(-0.758f, 0.804f);
                        ScoopTwoHingeJoint.useMotor = true;
                        ScoopTwoHingeJoint.limits = new JointAngleLimits2D()
                        {
                            min = -90f,
                            max = 25f
                        };
                        ScoopTwoHingeJoint.motor = new JointMotor2D
                        {
                            maxMotorTorque = 355f
                        };
                        ScoopTwo.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.279713631f,0.581938803f),
                            new Vector2(-0.754420698f,0.886992514f),
                            new Vector2(-0.85091728f,0.792827487f),
                            new Vector2(-0.290934622f,0.401159674f),
                            new Vector2(0.150115967f,-0.062374115f),
                            new Vector2(0.488174558f,-0.774749517f),
                            new Vector2(0.553447723f,-0.41532135f),
                            new Vector2(0.268375099f,0.0603149533f)
                        });
                        ScoopTwo.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        ScoopTwo.AddComponent<AudioSourceTimeScaleBehaviour>();
                        ScoopTwo.AddComponent<Optout>();

                        GameObject ScoopEnd = new GameObject("Scoop End");
                        ScoopEnd.transform.parent = ScoopTwo.transform;
                        ScoopEnd.transform.localRotation = Quaternion.identity;
                        ScoopEnd.transform.localScale = Vector3.one;
                        ScoopEnd.transform.localPosition = new Vector3(0.985f, -0.348f);
                        SpriteRenderer ScoopEndRenderer = ScoopEnd.AddComponent<SpriteRenderer>();
                        ScoopEndRenderer.sprite = ScoopEndSprite;
                        ScoopEndRenderer.sortingLayerName = "Foreground";
                        ScoopEndRenderer.sortingOrder = 7;
                        ScoopEnd.layer = LayerMask.NameToLayer("Objects");
                        ScoopEnd.AddComponent<Rigidbody2D>().mass = 5.5f;
                        HingeJoint2D ScoopEndHingeJoint = ScoopEnd.AddComponent<HingeJoint2D>();
                        ScoopEndHingeJoint.connectedBody = ScoopTwo.GetComponent<Rigidbody2D>();
                        ScoopEndHingeJoint.anchor = new Vector2(-0.44f, -0.347f);
                        ScoopEndHingeJoint.breakForce = Mathf.Infinity;
                        ScoopEndHingeJoint.breakTorque = Mathf.Infinity;
                        ScoopEndHingeJoint.useMotor = true;
                        ScoopEndHingeJoint.limits = new JointAngleLimits2D()
                        {
                            min = -10f,
                            max = 50f
                        };
                        ScoopEndHingeJoint.motor = new JointMotor2D
                        {
                            maxMotorTorque = 455f
                        };
                        ScoopEnd.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.231230736f,-0.577402115f),
                            new Vector2(-0.4037714f,-0.419204712f),
                            new Vector2(-0.38613224f,-0.139307022f),
                            new Vector2(-0.29548645f,0.201221466f),
                            new Vector2(0.176755905f,0.556043625f),
                            new Vector2(0.212548405f,0.645978808f),
                            new Vector2(-0.11740835f,0.445566058f),
                            new Vector2(-0.380677879f,0.21764271f),
                            new Vector2(-0.494505882f,-0.180198669f),
                            new Vector2(-0.500583649f,-0.493202209f),
                            new Vector2(-0.266391188f,-0.665067554f),
                            new Vector2(0.517600715f,-0.671428561f),
                            new Vector2(0.424634635f,-0.621515989f)
                        });
                        ScoopEnd.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                        ScoopEnd.AddComponent<AudioSourceTimeScaleBehaviour>();
                        ScoopEnd.AddComponent<Optout>();

                        Transform Bonnet = Instance.transform.GetChild(3);
                        Bonnet.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";
                        Bonnet.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        Bonnet.GetComponent<FixedJoint2D>().breakForce = 500f;
                        Bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.57f, 0.126f);
                        Bonnet.gameObject.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = -65f,
                            max = 0f
                        };
                        Bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.60437727f,-0.121885993f),
                            new Vector2(0.337554693f,0.0142583549f),
                            new Vector2(-0.600000024f,0.157142863f),
                            new Vector2(-0.600000024f,0.096889019f),
                            new Vector2(0.228675604f,-0.0248475224f),
                            new Vector2(0.587595582f,-0.163000911f)
                        });

                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.gameObject.AddComponent<BoxCollider2D>();
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Background";
                        
                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.452088356f,1.01831245f),
                            new Vector2(0.404644996f,1.11053586f),
                            new Vector2(-0.54285717f,1.11428571f),
                            new Vector2(-0.54285717f,-1.11428571f),
                            new Vector2(0.54285717f,-1.11428571f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(0.499f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].anchor = new Vector2(1.4283f, -1.3711f);
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = Wheel;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6428571f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = 4;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Wheels[0].connectedBody.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");

                        Wheels[1].anchor = new Vector2(-1.572f, -1.3711f);
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = Wheel;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6428571f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = 4;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Wheels[1].connectedBody.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");

                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            Bonnet.localPosition = new Vector3(2.441f, 0.243f);
                            GasTank.localScale = new Vector3(0.9f, 0.9f);
                            GasTank.localPosition = new Vector3(-0.26f, -0.97f);
                            FrontDoor.localPosition = new Vector3(0.129f, 0.6714f);
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(1.4283f, -1.3711f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(-1.572f, -1.3711f);

                            Transform Engine = Instance.transform.GetChild(7);
                            Engine.localScale = new Vector3(0.9f, 0.9f);
                            Engine.localPosition = new Vector3(-2.38f, -0.48f);

                            Instance.AddComponent<ObjectSerialisationHelper>().Object = ScoopTwo;
                            Instance.AddComponent<ObjectSerialisationHelper>().Object = ScoopEnd;
                        };

                        NoCollide noCollide = Instance.GetComponent<NoCollide>();
                        noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Dump Truck
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Dump Truck",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "A huge truck for rocks, ore, sand and other things.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Dump Truck.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite Wheel = ModAPI.LoadSprite("Textures/Dump Truck/Wheel.png");
                        Sprite BodyTop = ModAPI.LoadSprite("Textures/Dump Truck/Body2.png");

                        float MotorSpeed = -1000f;

                        Transform Body = Instance.transform.GetChild(0);
                        Body.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Body.GetComponent<SpriteRenderer>().sortingOrder = 3;
                        PhysicalBehaviour BodyPhysical = Body.GetComponent<PhysicalBehaviour>();
                        BodyPhysical.TrueInitialMass = 30f;
                        BodyPhysical.InitialMass = 30f;
                        BodyPhysical.rigidbody.mass = 30f;
                        foreach (BoxCollider2D BodyBoxCollider in Body.GetComponents<BoxCollider2D>())
                        {
                            UnityEngine.Object.Destroy(BodyBoxCollider);
                        }
                        PolygonCollider2D BodyCollider = Body.GetComponent<PolygonCollider2D>();
                        BodyCollider.pathCount = 2;
                        BodyCollider.SetPath(0, new Vector2[]
                        {
                            new Vector2(5.57152843f, 0.0901030973f),
                            new Vector2(5.57152843f, 0.200000003f),
                            new Vector2(5.17030954f, 0.204286426f),
                            new Vector2(5.17448997f, 0.732856274f),
                            new Vector2(4.81959248f, 0.952042818f),
                            new Vector2(4.41988564f, 2.1553421f),
                            new Vector2(3.34562874f, 2.15534592f),
                            new Vector2(3.00912857f, 1.49501228f),
                            new Vector2(2.55532074f, 1.48259163f),
                            new Vector2(2.315588f, 1.55578423f),
                            new Vector2(2.69932175f, 2.31271744f),
                            new Vector2(4.30949783f, 2.31061554f),
                            new Vector2(5.43244457f, 2.31122446f),
                            new Vector2(5.42938852f, 2.49326611f),
                            new Vector2(2.605757f, 2.49704766f),
                            new Vector2(1.40671158f, 0.124526978f),
                            new Vector2(-3.41459751f, 0.944307327f),
                            new Vector2(-5.50994587f, 1.66304207f),
                            new Vector2(-5.3538723f, 2.2164917f),
                            new Vector2(-5.5185318f, 2.25366378f),
                            new Vector2(-5.68185234f, 1.93126988f),
                            new Vector2(-5.68185234f, 1.55019736f),
                            new Vector2(-4.87706852f, 1.30618286f),
                            new Vector2(-3.58030462f, 0.656758904f),
                            new Vector2(-2.38441086f, -0.378349304f),
                            new Vector2(-0.0786851048f, -1.21481812f),
                            new Vector2(4.91599798f, -1.0606091f),
                            new Vector2(5.42175293f, -1.23095512f),
                            new Vector2(5.43134594f, -1.3969481f),
                            new Vector2(5.66725445f, -1.40488458f),
                            new Vector2(5.65272713f, -1.08600235f),
                            new Vector2(5.43330097f, -1.0919317f),
                            new Vector2(5.21218538f, 0.0847333074f)
                        });
                        BodyCollider.SetPath(1, new Vector2[]
                        {
                            new Vector2(4.6931572f, 0.919160843f),
                            new Vector2(4.72842789f, 0.240978241f),
                            new Vector2(3.94766426f, 0.237251282f),
                            new Vector2(4.27340698f, 0.668769836f),
                            new Vector2(3.79418755f, 0.664012909f),
                            new Vector2(3.73222542f, 1.33666039f),
                            new Vector2(3.58490562f, 1.33571434f),
                            new Vector2(3.6974411f, 0.230018616f),
                            new Vector2(2.59620667f, 0.239179611f),
                            new Vector2(3.41618538f, 2.07945824f),
                            new Vector2(4.39726448f, 2.06842804f)
                        });
                        Body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Dump Truck/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Dump Truck/Background.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Dump Truck/Door.png")
                            }
                        };
                        Body.GetChild(0).localPosition = Vector3.zero;

                        GameObject BackBodyTop = new GameObject("BackBodyTop");
                        BackBodyTop.transform.parent = Body;
                        BackBodyTop.transform.localRotation = Quaternion.identity;
                        BackBodyTop.transform.localScale = Vector3.one;
                        BackBodyTop.transform.localPosition = new Vector3(0f, 0f);
                        SpriteRenderer BackBodyTopRenderer = BackBodyTop.AddComponent<SpriteRenderer>();
                        BackBodyTopRenderer.sprite = BodyTop;
                        BackBodyTopRenderer.sortingOrder = 9;
                        BackBodyTopRenderer.sortingLayerName = "Foreground";
                        BackBodyTop.AddComponent<Optout>();

                        Instance.transform.GetChild(3).gameObject.SetActive(false);
                        Instance.transform.GetChild(4).gameObject.SetActive(false);
                        Instance.transform.GetChild(6).gameObject.SetActive(false);

                        Transform GasTank = Instance.transform.GetChild(8);
                        GasTank.gameObject.AddComponent<BoxCollider2D>();
                        GasTank.GetComponent<SpriteRenderer>().sortingLayerName = "Background";

                        Transform FrontDoor = Instance.transform.GetChild(5);
                        FrontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.585714281f,0.899999976f),
                            new Vector2(-0.585714281f,-0.899999976f),
                            new Vector2(0.585714281f,-0.899999976f),
                            new Vector2(0.580579698f,-0.160259575f),
                            new Vector2(0.179360211f,0.898656011f)
                        });
                        FrontDoor.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.544f, 0f);
                        FrontDoor.GetComponent<SpriteRenderer>().sortingOrder = 7;
                        FrontDoor.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        WheelJoint2D[] Wheels = Body.GetComponents<WheelJoint2D>();
                        Wheels[0].anchor = new Vector2(-1.4433f, -1.3301f);
                        Wheels[0].breakForce = 35000f;
                        Wheels[0].suspension = new JointSuspension2D
                        {
                            frequency = 15f,
                            angle = 90f,
                            dampingRatio = 0.5f
                        };
                        Wheels[0].connectedBody.GetComponent<Rigidbody2D>().mass = 35f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = Wheel;
                        Wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 1.3f;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = 4;
                        Wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Wheels[0].connectedBody.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");

                        Wheels[1].anchor = new Vector2(3.614f, -1.3301f);
                        Wheels[1].suspension = new JointSuspension2D
                        {
                            frequency = 15f,
                            angle = 90f,
                            dampingRatio = 0.5f
                        };
                        Wheels[1].connectedBody.GetComponent<Rigidbody2D>().mass = 35f;
                        Wheels[1].breakForce = 35000f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = Wheel;
                        Wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 1.3f;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = 8;
                        Wheels[1].connectedBody.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                        Wheels[1].connectedBody.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");

                        CarSerialisationHelper CarSerialisationHelper = Instance.AddComponent<CarSerialisationHelper>();
                        CarSerialisationHelper.AfterFirstSpawn = () =>
                        {
                            Body.GetComponent<CarBehaviour>().MotorSpeed = MotorSpeed;

                            GasTank.localScale = new Vector3(0.9f, 0.9f);
                            GasTank.localPosition = new Vector3(0.59f, -0.7f);
                            FrontDoor.localPosition = new Vector3(4.128f, 1.159f);
                            Wheels[0].connectedBody.transform.localPosition = new Vector2(-1.4433f, -1.3301f);
                            Wheels[1].connectedBody.transform.localPosition = new Vector2(3.614f, -1.3301f);

                            Transform Engine = Instance.transform.GetChild(7);
                            Engine.localScale = new Vector3(0.9f, 0.9f);
                            Engine.localPosition = new Vector3(1.65f, -0.5500011f);
                        };

                        NoCollide NoCollide = Instance.GetComponent<NoCollide>();
                        NoCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        NoCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        #endregion
        #region Light Tank
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = FindTank(),
                    NameOverride = "Light Tank",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "This tank is slightly weaker, but faster and smaller.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Light Tank.png"),
                    AfterSpawn = (Instance) =>
                    {
                        //Main       
                        UnityEngine.Object.Destroy(Instance.GetComponentInChildren<RandomCarTextureBehaviour>());
                        var TankHead = Instance.transform.Find("Head");
                        var TankBody = Instance.transform.Find("Body");
                        var TankBarrel = Instance.transform.Find("Barrel");
                        var TankGasTank = Instance.transform.Find("Gas tank");
                        var TankEngine = Instance.transform.Find("Engine");

                        float ObjectSize = 0.8f;
                        bool flipped = Instance.transform.localScale.x < 0f;
                        Instance.transform.localScale = new Vector2(ObjectSize * (flipped ? -1f : 1f), ObjectSize);

                        var TankSettings = TankBody.GetComponent<CarBehaviour>();
                        TankSettings.MotorSpeed = -1200f;
                        TankSettings.WaterProof = false;

                        var BarrelSettings = TankBarrel.GetComponent<TankBarrelBehaviour>();
                        BarrelSettings.RecoilIntensity = 10f;
                        BarrelSettings.barrelPosition = new Vector2(3.67f, 0f);
                        BarrelSettings.barrelDirection = new Vector2(1, 0);

                        //DestroyColliders


                        BoxCollider2D[] boxColliders = TankHead.GetComponents<BoxCollider2D>();
                        GameObject.Destroy(boxColliders[0]);
                        GameObject.Destroy(boxColliders[1]);
                        GameObject.Destroy(boxColliders[2]);

                        BoxCollider2D[] boxColliders2 = TankBody.GetComponents<BoxCollider2D>();
                        GameObject.Destroy(boxColliders2[0]);
                        GameObject.Destroy(boxColliders2[1]);
                        GameObject.Destroy(boxColliders2[2]);
                        GameObject.Destroy(boxColliders2[3]);
                        GameObject.Destroy(boxColliders2[4]);

                        BoxCollider2D[] boxColliders3 = TankBarrel.GetComponents<BoxCollider2D>();
                        GameObject.Destroy(boxColliders3[0]);

                        //Tank Head
                        TankHead.gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                        ModAPI.LoadSprite("Textures/Light Tank/Head.png"),
                        };
                        PolygonCollider2D TankHeadColl = TankHead.GetComponent<PolygonCollider2D>();
                        TankHeadColl.pathCount = 1;
                        TankHeadColl.SetPath(0, new Vector2[]
                        {
                        new Vector2(2.95045781f,0.0432439595f),
                        new Vector2(1.34521449f,0.258181304f),
                        new Vector2(1.33344543f,0.3404814f),
                        new Vector2(0.929694176f,0.368379593f),
                        new Vector2(0.933256149f,0.538560867f),
                        new Vector2(-0.0516361669f,0.542064607f),
                        new Vector2(-0.0451312065f,0.37304306f),
                        new Vector2(-0.744933128f,0.375880271f),
                        new Vector2(-0.836340904f,0.257914543f),
                        new Vector2(-0.970416069f,0.257637978f),
                        new Vector2(-0.977412224f,0.362969398f),
                        new Vector2(-1.17317295f,0.365386009f),
                        new Vector2(-1.17457795f,0.773522496f),
                        new Vector2(-1.28282881f,0.770900846f),
                        new Vector2(-1.27099133f,0.365733147f),
                        new Vector2(-1.33997166f,0.367469609f),
                        new Vector2(-1.33446443f,0.247959524f),
                        new Vector2(-1.63300419f,0.255822569f),
                        new Vector2(-1.63074493f,0.338354111f),
                        new Vector2(-1.96627712f,0.347113609f),
                        new Vector2(-1.9704113f,0.25564003f),
                        new Vector2(-2.68309355f,0.265252769f),
                        new Vector2(-2.73840904f,0.381319046f),
                        new Vector2(-2.88469887f,0.360998154f),
                        new Vector2(-3.00130129f,0.256615132f),
                        new Vector2(-3.00340843f,0.00197029114f),
                        new Vector2(-3.30819035f,0.00279712677f),
                        new Vector2(-3.30719543f,-0.0833574533f),
                        new Vector2(-3.44698429f,-0.0872545242f),
                        new Vector2(-3.45137191f,-0.467100799f),
                        new Vector2(-3.24917316f,-0.46459198f),
                        new Vector2(-3.25216341f,-0.401038468f),
                        new Vector2(-2.5347023f,-0.398845673f),
                        new Vector2(-2.29516244f,-0.6482234f),
                        new Vector2(-1.55777657f,-0.828571439f),
                        new Vector2(2.62802529f,-0.825949788f),
                        new Vector2(3.17246652f,-0.710890472f),
                        new Vector2(3.29042935f,-0.447713643f)
                        });
                        //Body	
                        TankBody.gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                        ModAPI.LoadSprite("Textures/Light Tank/Body.png"),
                        };
                        PolygonCollider2D TankBodydColl = TankBody.gameObject.AddComponent<PolygonCollider2D>();
                        TankBodydColl.pathCount = 1;
                        TankBodydColl.SetPath(0, new Vector2[]
                        {
                        new Vector2(4.46418333f,-0.572593629f),
                        new Vector2(5.07785463f,0.16264011f),
                        new Vector2(4.84641695f,0.175995678f),
                        new Vector2(4.3703413f,0.258496732f),
                        new Vector2(3.66264009f,0.559426904f),
                        new Vector2(2.2304616f,0.54557991f),
                        new Vector2(2.24285722f,0.603165865f),
                        new Vector2(-2.87403822f,0.599975467f),
                        new Vector2(-2.87142849f,0.709077358f),
                        new Vector2(-4.64309978f,0.70197773f),
                        new Vector2(-4.6506505f,0.594767094f),
                        new Vector2(-5.10258532f,0.161011845f),
                        new Vector2(-4.98828125f,-0.111662403f),
                        new Vector2(-4.32600451f,-0.109058231f),
                        new Vector2(-3.34865928f,-0.554462373f)
                        });
                        //Barrel							
                        TankBarrel.gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                        ModAPI.LoadSprite("Textures/Light Tank/Barrel.png"),
                        };
                        PolygonCollider2D TankBarrelColl = TankBarrel.gameObject.AddComponent<PolygonCollider2D>();
                        TankBarrelColl.pathCount = 1;
                        TankBarrelColl.SetPath(0, new Vector2[]
                        {
                        new Vector2(3.62604022f,-0.177047893f),
                        new Vector2(3.62604046f,0.1282271f),
                        new Vector2(-1.98521709f,0.191445351f),
                        new Vector2(-3.62545967f,0.237767994f),
                        new Vector2(-3.61923695f,-0.298021734f),
                        new Vector2(-2.02290297f,-0.242012441f)
                        });
                        //Gas Tank
                        Instance.transform.Find("Gas tank").gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                        ModAPI.LoadSprite("Textures/Light Tank/Gas Tank.png"),
                        };
                        //Engine
                        Instance.transform.Find("Engine").gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                        ModAPI.LoadSprite("Textures/Light Tank/Engine.png"),
                        };
                        //Wheels
                        Instance.transform.Find("Wheel").gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                                {
                                    ModAPI.LoadSprite("Textures/Light Tank/Wheel.png"),
                                };

                        Instance.transform.Find("Wheel (1)").gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                                    {
                                    ModAPI.LoadSprite("Textures/Light Tank/Wheel2.png"),
                                };

                        Instance.transform.Find("Wheel (2)").gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                                {
                                    ModAPI.LoadSprite("Textures/Light Tank/Wheel2.png"),
                                };

                        Instance.transform.Find("Wheel (3)").gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                                {
                                    ModAPI.LoadSprite("Textures/Light Tank/Wheel2.png"),
                                };

                        Instance.transform.Find("Wheel (4)").gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                                {
                                    ModAPI.LoadSprite("Textures/Light Tank/Wheel2.png"),
                                };

                        Instance.transform.Find("Wheel (5)").gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                                {
                                    ModAPI.LoadSprite("Textures/Light Tank/Wheel2.png"),
                                };

                        Instance.transform.Find("Wheel (6)").gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                                {
                                    ModAPI.LoadSprite("Textures/Light Tank/Wheel2.png"),
                                };

                        Instance.transform.Find("Wheel (7)").gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                                    ModAPI.LoadSprite("Textures/Light Tank/Wheel2.png"),
                        };

                        Instance.transform.Find("Wheel (8)").gameObject.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                                    ModAPI.LoadSprite("Textures/Light Tank/Wheel.png"),
                        };
                        //Other
                        Instance.AddComponent<NoChildrenCollide>();
                        NoCollide noCollide = Instance.AddComponent<NoCollide>();
                        noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                        //
                    }
                }
            );
        #endregion
#endregion
#region Sea Vehicles
        #region Raft
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Plank"),
                    NameOverride = "Raft",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "A raft for ocean survival. Hopefully you won't be overtaken by a giant killer shark.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Raft.png"),
                    AfterSpawn = (Instance) =>
                    {
                        //Main	
                        Sprite CustomSprite = ModAPI.LoadSprite("Textures/SeaVehicles/Raft.png");
                        float WeaponSize = 1.0f;

                        var physOther = UnityEngine.Object.Instantiate<PhysicalProperties>(ModAPI.FindPhysicalProperties("Metal"));
                        Instance.GetComponent<PhysicalBehaviour>().Properties = physOther;
                        PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        //Settings
                        phys.Properties = ModAPI.FindPhysicalProperties("Wood");
                        phys.InitialMass = 10;
                        phys.TrueInitialMass = 10;
                        //phys.BuoyancyModifier = 0.8f;
                        physOther.Buoyancy = 0.7f;
                        phys.GridRes = 30;
                        phys.LocalColliderGridPoints = new Vector2[]
                        {
                            new Vector2(-2.5f, -0.07f),
                            new Vector2(2.5f, -0.07f),
                            new Vector2(2f, -0.07f),
                            new Vector2(-2f, -0.07f),
                            new Vector2(0.5f, -0.07f),
                            new Vector2(-0.5f, -0.07f),
                            new Vector2(1.5f, -0.07f),
                            new Vector2(-1.5f, -0.07f),
                            new Vector2(1f, -0.07f),
                            new Vector2(-1f, -0.07f),
                            new Vector2(0.8f, -0.07f),
                            new Vector2(-0.8f, -0.07f),
                            new Vector2(1.8f, -0.07f),
                            new Vector2(-1.8f, -0.07f),
                            new Vector2(1.4f, -0.07f),
                            new Vector2(-1.4f, -0.07f),
                            new Vector2(0.4f, -0.07f),
                            new Vector2(-0.4f, -0.07f),
                            new Vector2(0.2f, -0.07f),
                            new Vector2(-0.2f, -0.07f),
                            new Vector2(1.2f, -0.07f),
                            new Vector2(-1.2f, -0.07f),
                        };
                        //Other
                        var SpriteRenderer = Instance.GetComponent<SpriteRenderer>();
                        SpriteRenderer.sprite = CustomSprite;
                        SpriteRenderer.sortingLayerName = "Default";
                        SpriteRenderer.sortingOrder = -10;
                        foreach (Collider2D Collison in Instance.GetComponents<Collider2D>())
                        {
                            Collider2D.Destroy(Collison);
                        }
                        Instance.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(2.5010047f,-0.0708995461f),
                            new Vector2(2.50065565f,0.0726805329f),
                            new Vector2(2.3540802f,0.213154539f),
                            new Vector2(-2.35070658f,0.21009706f),
                            new Vector2(-2.49860334f,0.0691809505f),
                            new Vector2(-2.50279236f,-0.0718711466f),
                            new Vector2(-2.35617208f,-0.213043839f),
                            new Vector2(2.35634232f,-0.213154554f)

                        });
                        bool flipped = Instance.transform.localScale.x < 0f;
                        Instance.transform.localScale = new Vector2(WeaponSize * (flipped ? -1f : 1f), WeaponSize);
                        //	
                    }
                }
            );
        #endregion
        #region Wooden Boat
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Plank"),
                    NameOverride = "Wooden Boat",
                    NameToOrderByOverride = "MoreVehicles",
                    DescriptionOverride = "Travel the sea on this wonderful boat!",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnails/Boat.png"),
                    AfterSpawn = (Instance) =>
                    {
                        //Main	
                        Sprite CustomSprite = ModAPI.LoadSprite("Textures/SeaVehicles/Boat.png");
                        float WeaponSize = 1.2f;

                        var physOther = UnityEngine.Object.Instantiate<PhysicalProperties>(ModAPI.FindPhysicalProperties("Metal"));
                        Instance.GetComponent<PhysicalBehaviour>().Properties = physOther;
                        PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        //Settings
                        phys.Properties = ModAPI.FindPhysicalProperties("Wood");
                        phys.InitialMass = 10;
                        phys.TrueInitialMass = 10;
                        phys.BuoyancyModifier = 1f;
                        physOther.Buoyancy = 0.7f;
                        phys.GridRes = 50;
                        phys.LocalColliderGridPoints = new Vector2[]
                        {
                            new Vector2(-2.5f, -0.07f),
                            new Vector2(2.5f, -0.07f),
                            new Vector2(2f, -0.07f),
                            new Vector2(-2f, -0.07f),
                            new Vector2(0.5f, -0.07f),
                            new Vector2(-0.5f, -0.07f),
                            new Vector2(1.5f, -0.07f),
                            new Vector2(-1.5f, -0.07f),
                            new Vector2(1f, -0.07f),
                            new Vector2(-1f, -0.07f),
                            new Vector2(0.8f, -0.07f),
                            new Vector2(-0.8f, -0.07f),
                            new Vector2(1.8f, -0.07f),
                            new Vector2(-1.8f, -0.07f),
                            new Vector2(1.4f, -0.07f),
                            new Vector2(-1.4f, -0.07f),
                            new Vector2(0.4f, -0.07f),
                            new Vector2(-0.4f, -0.07f),
                            new Vector2(0.2f, -0.07f),
                            new Vector2(-0.2f, -0.07f),
                            new Vector2(1.2f, -0.07f),
                            new Vector2(-1.2f, -0.07f),
                        };
                        //Other
                        var SpriteRenderer = Instance.GetComponent<SpriteRenderer>();
                        SpriteRenderer.sprite = CustomSprite;
                        SpriteRenderer.sortingLayerName = "Foreground";
                        SpriteRenderer.sortingOrder = 9;
                        foreach (Collider2D Collison in Instance.GetComponents<Collider2D>())
                        {
                            Collider2D.Destroy(Collison);
                        }
                        Instance.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(1.75355268f,0.371487379f),
                            new Vector2(1.54061508f,-0.0269851685f),
                            new Vector2(1.05295849f,-0.31082058f),
                            new Vector2(-1.57209206f,-0.322158813f),
                            new Vector2(-1.75483131f,-0.1004076f),
                            new Vector2(-1.76906395f,0.262514114f),
                            new Vector2(-1.83318138f,0.260807991f),
                            new Vector2(-1.91406631f,0.268387794f),
                            new Vector2(-1.92119503f,0.170456886f),
                            new Vector2(-1.83997822f,0.159018517f),
                            new Vector2(-1.82086754f,-0.134806633f),
                            new Vector2(-1.61663413f,-0.379937887f),
                            new Vector2(1.07399154f,-0.372306585f),
                            new Vector2(1.53167915f,-0.13051796f),
                            new Vector2(1.81284428f,0.262667656f),
                            new Vector2(1.90846038f,0.294569254f),
                            new Vector2(1.92857146f,0.385714293f)

                        });
                        bool flipped = Instance.transform.localScale.x < 0f;
                        Instance.transform.localScale = new Vector2(WeaponSize * (flipped ? -1f : 1f), WeaponSize);
                        //	
                    }
                }
            );
        #endregion
#endregion
        }
    }
#region Scripts
    public class CarSmoothBrakingBehaviour : MonoBehaviour
    {
        [SkipSerialisation]
        private CarBehaviour CarBehaviour;
        
        public bool SmoothBrakingIsSet = false; //не трогать
        [SkipSerialisation]
        public float DefaultTorque = 150f; //wheel.motor.maxMotorTorque по умолчанию
        [SkipSerialisation]
        public float SmoothBrakingTorque = 50f; //wheel.motor.maxMotorTorque при торможении, пока скрорось колес больше SpeedForFullBraking
        [SkipSerialisation]
        public float SpeedForFullBraking = 300f; //скрорость для начала полного торможения

        private void Awake()
        {
            CarBehaviour = GetComponent<CarBehaviour>();
        }
        private void FixedUpdate()
        {
            if (!CarBehaviour.Activated && GetWheelsAverageSpeed() * (CarBehaviour.MotorSpeed > 0 ? 1f : -1f) > SpeedForFullBraking)
            {
                if (!SmoothBrakingIsSet)
                {
                    Debug.Log("SmoothBrakingTorque");
                    SetWheelsTorque(SmoothBrakingTorque);
                    SmoothBrakingIsSet = true;
                }   
            }
            else
            {  
                if (SmoothBrakingIsSet)
                {
                    Debug.Log("DefaultTorque");
                    SetWheelsTorque(DefaultTorque);
                    SmoothBrakingIsSet = false;
                }
            }
        }
        private float GetWheelsAverageSpeed()
        {
            int wheelsCount = 0;
            float overallSpeed = 0f;
            foreach (WheelJoint2D wheel in CarBehaviour.WheelJoints)
            {
                if (wheel == null)
                {
                    continue;
                }
                wheelsCount++;
                overallSpeed += wheel.jointSpeed;
            }
            return overallSpeed / wheelsCount;
        }
        private void SetWheelsTorque(float torque)
        {
            foreach (WheelJoint2D wheel in CarBehaviour.WheelJoints)
            {
                if (wheel == null)
                {
                    continue;
                }
                JointMotor2D motor = wheel.motor;
                motor.maxMotorTorque = torque;
                wheel.motor = motor;
            }
        }
    }
    [SkipSerialisation]
    public class SmokeController : MonoBehaviour
    {
        private void Start()
        {
            carBehaviour = GetComponent<CarBehaviour>();
        }
        private void Update()
        {
            if (!Smoke.isPlaying && carBehaviour.Activated)
            {
                Smoke.Play();
            }
            else if (!carBehaviour.Activated)
            {
                Smoke.Stop();
            }
        }
        private void OnDisable()
        {
            if (Smoke.isPlaying)
            {
                Smoke.Stop();
            }
        }

        private CarBehaviour carBehaviour;
        public ParticleSystem Smoke;
    }
    [SkipSerialisation]
    public class CarSerialisationHelper : MonoBehaviour
    {
        private void Start()
        {
            bool firstSpawn = true;
            foreach (MonoBehaviour monoBehaviour in transform.root.GetComponentsInChildren<MonoBehaviour>())
            {
                if (monoBehaviour.GetType().Name == "FirstSpawn")
                {
                    firstSpawn = false;
                    break;
                }
            }
            RandomCarTextureBehaviour randomCarTextures = GetComponentInChildren<RandomCarTextureBehaviour>();
            if (randomCarTextures && randomCarTextures.Textures.Length <= 1)
            {
                PhysicalBehaviour phys = randomCarTextures.GetComponent<PhysicalBehaviour>();
                ContextMenuButton nextTexture = phys.ContextMenuOptions.Buttons.Find((button) => button.Identity == "nextSkin");
                ContextMenuButton previousTexture = phys.ContextMenuOptions.Buttons.Find((button) => button.Identity == "previousSkin");
                phys.ContextMenuOptions.Buttons.Remove(nextTexture);
                phys.ContextMenuOptions.Buttons.Remove(previousTexture);
            }
            if (firstSpawn)
            {
                gameObject.AddComponent<FirstSpawn>();
                if (randomCarTextures)
                {
                    randomCarTextures.chosenIndex = UnityEngine.Random.Range(0, randomCarTextures.Textures.Length);
                    randomCarTextures.SendMessage("SetSpritesToIndex", SendMessageOptions.DontRequireReceiver);
                    carOptics = randomCarTextures.gameObject.AddComponent<CarOpticsBehaviour>();
                }
                AfterFirstSpawn?.Invoke();
            }
            else
            {
                if (randomCarTextures)
                {
                    CarOpticsBehaviour carOptics = randomCarTextures.GetComponent<CarOpticsBehaviour>();
                    if (carOptics)
                    {
                        this.carOptics = randomCarTextures.GetComponent<CarOpticsBehaviour>();
                    }
                    else
                    {
                        this.carOptics = randomCarTextures.gameObject.AddComponent<CarOpticsBehaviour>();
                    }
                }
            }
            AfterSpawn?.Invoke();
            if (transform.localScale.x > 0)
            {
                Right?.Invoke();
            }
            else
            {
                Left?.Invoke();
            }
        }

        public Action AfterFirstSpawn;
        public Action AfterSpawn;
        public Action Right;
        public Action Left;
        public CarOpticsBehaviour carOptics;
    }
    public class FirstSpawn : MonoBehaviour
    {
    }
    [SkipSerialisation]
    public class NoCollideWithObjectsInTrigger : MonoBehaviour
    {
        private void Start()
        {
            collidersWithoutTrigger = GetComponents<Collider2D>().ToList();
            collidersWithoutTrigger.Remove(Trigger);
        }
        private void OnTriggerEnter2D(Collider2D collider)
        {
            if (collider.isTrigger || noCollides.ContainsKey(collider))
            {
                return;
            }
            NoCollide noCollide = gameObject.AddComponent<NoCollide>();
            noCollide.NoCollideSetA = Colliders;
            noCollide.NoCollideSetB = new Collider2D[]
            {
                collider
            };
            noCollides.Add(collider, noCollide);
        }
        private void OnTriggerExit2D(Collider2D collider)
        {
            if (!noCollides.ContainsKey(collider))
            {
                return;
            }
            Destroy(noCollides[collider]);
            noCollides.Remove(collider);
        }
        public void RemoveCollider(int index)
        {
            List<Collider2D> colliders = Colliders.ToList();
            colliders.RemoveAt(index);
            Colliders = colliders.ToArray();
            Dictionary<Collider2D, NoCollide>.KeyCollection otherColliders = noCollides.Keys;
            foreach (Collider2D collider in otherColliders.ToList())
            {
                OnTriggerExit2D(collider);
                OnTriggerEnter2D(collider);
            }
        }
        private void OnCableConnect(WireBehaviour wire)
        {
            if (wire.untypedJoint.enableCollision)
            {
                return;
            }
            wire.untypedJoint.enableCollision = true;
            if (wire.untypedJoint.connectedBody == GetComponent<Rigidbody2D>())
            {
                colliders.Add(wire, wire.GetComponents<Collider2D>());
            }
            else
            {
                colliders.Add(wire, wire.untypedJoint.connectedBody.GetComponents<Collider2D>());
            }
            foreach (Collider2D collider in collidersWithoutTrigger)
            {
                foreach (Collider2D otherCollider in colliders[wire])
                {
                    IgnoreCollisionStackController.IgnoreCollisionSubstituteMethod(collider, otherCollider);
                }
            }
            connectedWires.Add(wire);
        }
        private void OnCableDisconnect(WireBehaviour wire)
        {
            if (!connectedWires.Contains(wire))
            {
                return;
            }
            foreach (Collider2D collider in collidersWithoutTrigger)
            {
                foreach (Collider2D otherCollider in colliders[wire])
                {
                    IgnoreCollisionStackController.IgnoreCollisionSubstituteMethod(collider, otherCollider, false);
                }
            }
            connectedWires.Remove(wire);
            colliders.Remove(wire);
        }

        public Collider2D[] Colliders;
        public Collider2D Trigger;
        private List<Collider2D> collidersWithoutTrigger;
        private List<WireBehaviour> connectedWires = new List<WireBehaviour>();
        private Dictionary<WireBehaviour, Collider2D[]> colliders = new Dictionary<WireBehaviour, Collider2D[]>();
        private Dictionary<Collider2D, NoCollide> noCollides = new Dictionary<Collider2D, NoCollide>();
    }
    public class CarWheelsBehaviour : MonoBehaviour
    {
        private void OnJointBreak2D(Joint2D joint)
        {
            if (Wheels.Contains(joint))
            {
                joint.connectedBody.transform.parent = transform.parent;
            }
        }

        [SkipSerialisation]
        public WheelJoint2D[] Wheels = new WheelJoint2D[0];
    }
    public class ObjectSerialisationHelper : MonoBehaviour
    {
        private void Start()
        {
            if (Object != null)
            {
                joints = Object.GetComponents<Joint2D>();
            }
        }
        private void OnBeforeSerialise()
        {
            disabledCollision = Object.layer == 10;
            isWeightless = Object.GetComponent<PhysicalBehaviour>().IsWeightless;
            frezee = Object.GetComponent<FreezeBehaviour>();
            temperature = Object.GetComponent<PhysicalBehaviour>().Temperature;

            List<bool> jointBrokenList = new List<bool>();
            for (int i = 0; i < joints.Length; i++)
            {
                jointBrokenList.Add(!joints[i]);
            }
            jointBroken = jointBrokenList.ToArray();

            CarOpticsBehaviour carOptics = Object.GetComponent<CarOpticsBehaviour>();
            if (carOptics)
            {
                headlightsActivated = carOptics.HeadlightsActivated;
                turnSignalsActivated = carOptics.TurnSignalsActivated;
                startMotorSpeed = carOptics.startMotorSpeed;
            }

            BackDoorBehaviour backDoorBehaviour = Object.GetComponent<BackDoorBehaviour>();
            if (backDoorBehaviour)
            {
                backDoorActivated = backDoorBehaviour.Activated;
                spareWheelJointBroken = backDoorBehaviour.spareWheelJointBroken;
            }

            if (Object.GetComponent<DamagableMachineryBehaviour>())
            {
                destroyed = Object.GetComponent<DamagableMachineryBehaviour>().Destroyed;
            }

            InsideBehaviour insideBehaviour = Object.GetComponent<InsideBehaviour>();
            if (insideBehaviour)
            {
                showInside = insideBehaviour.ShowInside;
            }

            transformData = new TransformPrototype(Object.transform.localPosition, Object.transform.eulerAngles.z, Object.transform.localScale);
            originalScale = transform.localScale;
            objectName = Utils.GetHierachyPath(Object.transform);

            if (Object.GetComponent<CarBehaviour>())
            {
                carSpeed = Object.GetComponent<CarBehaviour>().MotorSpeed;
            }

            if (Object.GetComponent<RandomCarTextureBehaviour>())
            {
                carTextureIndex = Object.GetComponent<RandomCarTextureBehaviour>().chosenIndex;
            }
        }
        private void OnAfterDeserialise()
        {
            GameObject gameObject = transform.Find(objectName).gameObject;
            gameObject.transform.position = gameObject.transform.parent.TransformPoint(transformData.RelativePosition);
            gameObject.transform.eulerAngles = new Vector3(0f, 0f, (transform.localScale == originalScale) ? transformData.RelativeRotation + this.gameObject.transform.eulerAngles.z : -transformData.RelativeRotation - this.gameObject.transform.eulerAngles.z);
            gameObject.transform.localScale = transformData.LocalScale;

            gameObject.layer = disabledCollision ? 10 : 9;
            gameObject.GetComponent<PhysicalBehaviour>().IsWeightless = isWeightless;
            gameObject.GetComponent<PhysicalBehaviour>().Temperature = temperature;

            if (frezee)
            {
                gameObject.AddComponent<FreezeBehaviour>();
            }

            joints = gameObject.GetComponents<Joint2D>();
            for (int i = 0; i < joints.Length; i++)
            {
                if (jointBroken[i])
                {
                    Destroy(joints[i]);
                }
            }

            CarOpticsBehaviour carOptics = gameObject.GetComponent<CarOpticsBehaviour>();
            if (carOptics)
            {
                carOptics.firstSpawn = false;
                carOptics.HeadlightsActivated = headlightsActivated;
                carOptics.TurnSignalsActivated = turnSignalsActivated;
                carOptics.startMotorSpeed = startMotorSpeed;
            }

            BackDoorBehaviour backDoorBehaviour = gameObject.GetComponent<BackDoorBehaviour>();
            if (backDoorBehaviour)
            {
                backDoorBehaviour.Activated = backDoorActivated;
                backDoorBehaviour.spareWheelJointBroken = spareWheelJointBroken;
            }

            if (gameObject.GetComponent<DamagableMachineryBehaviour>())
            {
                gameObject.GetComponent<DamagableMachineryBehaviour>().Destroyed = destroyed;
            }

            InsideBehaviour inside = gameObject.GetComponent<InsideBehaviour>();
            if (inside)
            {
                inside.ShowInside = showInside;
                inside.UpdateInside();
            }

            if (gameObject.GetComponent<CarBehaviour>())
            {
                gameObject.GetComponent<CarBehaviour>().MotorSpeed = carSpeed;
            }

            if (gameObject.GetComponent<RandomCarTextureBehaviour>())
            {
                gameObject.GetComponent<RandomCarTextureBehaviour>().chosenIndex = carTextureIndex;
            }

            Object = gameObject;
        }

        public bool disabledCollision;
        public bool isWeightless;
        public bool frezee;
        public bool[] jointBroken;
        public bool headlightsActivated;
        public bool turnSignalsActivated;
        public bool backDoorActivated;
        public bool spareWheelJointBroken;
        public bool destroyed;
        public bool showInside;
        [SkipSerialisation]
        public GameObject Object;
        private Joint2D[] joints;
        public TransformPrototype transformData;
        public Vector3 originalScale;
        public string objectName;
        public float temperature;
        public float startMotorSpeed;
        public float carSpeed;
        public int carTextureIndex;
    }
    public class CarOpticsBehaviour : MonoBehaviour
    {
        private void Start()
        {
            phys = gameObject.GetComponent<PhysicalBehaviour>();
            Car = gameObject.GetComponent<CarBehaviour>();
            for (int i = 0; i < Headlights.Length; i++)
            {
                Headlights[i].SetActive(false);
            }
            if (ReversingLamp != null)
            {
                ReversingLamp.SetActive(false);
            }
            if (firstSpawn)
            {
                startMotorSpeed = Car.MotorSpeed;
            }
            phys.ContextMenuOptions.Buttons.Add(new ContextMenuButton("cruiseControl", "Set speed limit", "Sets speed limit", new UnityAction[1]
            {
                () =>
                {
                    DialogBox dialog = null;
                    dialog = DialogBoxManager.TextEntry("Set the speed limit\n<color=green><size=26>Currently: " + Mathf.Abs(Car.MotorSpeed).ToString() + "</size></color>", "Speed limit", new DialogButton[]
                    {
                        new DialogButton("Apply", true, () =>
                        {
                            float speedLimit;
                            if(float.TryParse(dialog.EnteredText, out speedLimit))
                            {
                                if(speedLimit > 0f)
                                {
                                    if(speedLimit >= Mathf.Abs(startMotorSpeed))
                                    {
                                        Car.MotorSpeed = (Car.MotorSpeed > 0f) ? Mathf.Abs(startMotorSpeed) : startMotorSpeed;
                                        return;
                                    }
                                    if(Car.MotorSpeed > 0f)
                                    {
                                        Car.MotorSpeed = speedLimit;
                                    }
                                    else
                                    {
                                        Car.MotorSpeed = -speedLimit;
                                    }
                                }
                                else
                                {
                                    if(speedLimit <= startMotorSpeed)
                                    {
                                        Car.MotorSpeed = (Car.MotorSpeed > 0f) ? Mathf.Abs(startMotorSpeed) : startMotorSpeed;
                                        return;
                                    }
                                    if(Car.MotorSpeed > 0f)
                                    {
                                        Car.MotorSpeed = Mathf.Abs(speedLimit);
                                    }
                                    else
                                    {
                                        Car.MotorSpeed = speedLimit;
                                    }
                                }
                            }
                        }),
                        new DialogButton("Off", true, () =>
                        {
                            Car.MotorSpeed = startMotorSpeed;
                        }),
                        new DialogButton("Cancel", true)
                    });
                }
            }));
            phys.ContextMenuOptions.Buttons.Add(new ContextMenuButton("toggleHeadlights", () => HeadlightsActivated ? "Disable headlights" : "Enable headlights", "Toggles headlights", () =>
            {
                HeadlightsActivated = !HeadlightsActivated;
            })
            {
                Condition = () => Headlights.Length != 0 && Headlights != null
            });
            phys.ContextMenuOptions.Buttons.Add(new ContextMenuButton("toggleTurnSignals", () => TurnSignalsActivated ? "Disable turn signals" : "Enable turn signals", "Toggles turn signals", () =>
            {
                TurnSignalsActivated = !TurnSignalsActivated;
                UpdateActivation();
            })
            {
                Condition = () => TurnSignals.Length != 0 && TurnSignals != null
            });
            if (FailedEngineStartup)
            {
                carAudioSources = Car.audioSourcePosition.GetComponents<AudioSource>();
                failedEngineStartupSource = Car.audioSourcePosition.gameObject.AddComponent<AudioSource>();
                failedEngineStartupSource.spread = 0f;
                failedEngineStartupSource.volume = 1f;
                failedEngineStartupSource.minDistance = 10f;
                failedEngineStartupSource.spatialBlend = 1f;
                failedEngineStartupSource.dopplerLevel = 1f;
                gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            }
            UpdateActivation();
            firstSpawn = false;
        }
        private void Use()
        {
            if (FailedEngineStartup)
            {
                if (Car.Activated)
                {
                    if ((double)UnityEngine.Random.value > FailedEngineStartupChance)
                    {
                        carAudioSources[0].Stop();
                        carAudioSources[1].Stop();
                        Car.Activated = false;
                        failedEngineStartupSource.PlayOneShot(FailedEngineStartupSound, 1f);
                    }
                }
            }
        }
        private void Update()
        {
            for (int i = 0; i < Headlights.Length; i++)
            {
                if (HeadlightsActivated)
                {
                    Headlights[i].SetActive(true);
                }
                if (!HeadlightsActivated)
                {
                    Headlights[i].SetActive(false);
                }
            }
            if (ReversingLamp != null)
            {
                if (Car.MotorSpeed > 0 && Car.Activated && !ReversingLampActivated)
                {
                    ReversingLampActivated = true;
                    ReversingLamp.SetActive(true);
                }
                if (Car.MotorSpeed < 0 && ReversingLampActivated || !Car.Activated)
                {
                    ReversingLampActivated = false;
                    ReversingLamp.SetActive(false);
                }
            }
        }
        public void UpdateActivation()
        {
            if (TurnSignals.Length != 0)
            {
                if (TurnSignalsActivated)
                {
                    StopAllCoroutines();
                    StartCoroutine(Blink());
                }
                else
                {
                    for (int i = 0; i < TurnSignals.Length; i++)
                    {
                        TurnSignals[i].SetActive(false);
                    }
                    StopAllCoroutines();
                }
            }
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            if (ActivateTurnSignalsOnCollision && TurnSignals.Length != 0)
            {
                if (collision.GetContact(0).normalImpulse > BreakingThreshold)
                {
                    TurnSignalsActivated = true;
                    UpdateActivation();
                }
            }
        }
        private IEnumerator Blink()
        {
            while (TurnSignalsActivated)
            {
                yield return new WaitForSeconds(0.3f);
                for (int i = 0; i < TurnSignals.Length; i++)
                {
                    TurnSignals[i].SetActive(true);
                }
                yield return new WaitForSeconds(0.3f);
                for (int i = 0; i < TurnSignals.Length; i++)
                {
                    TurnSignals[i].SetActive(false);
                }
            }
        }

        public bool firstSpawn = true;
        public bool HeadlightsActivated;
        public bool TurnSignalsActivated;
        public bool ReversingLampActivated;
        public bool ActivateTurnSignalsOnCollision;
        public bool FailedEngineStartup;
        [SkipSerialisation]
        public GameObject[] Headlights = new GameObject[0];
        [SkipSerialisation]
        public GameObject[] TurnSignals = new GameObject[0];
        [SkipSerialisation]
        public GameObject ReversingLamp;
        private PhysicalBehaviour phys;
        private AudioSource[] carAudioSources;
        private AudioSource failedEngineStartupSource;
        [SkipSerialisation]
        public CarBehaviour Car;
        [SkipSerialisation]
        public AudioClip FailedEngineStartupSound;
        public float BreakingThreshold = 20f;
        public float startMotorSpeed;
        public double FailedEngineStartupChance = 0.8d;
    }
    public class RandomCarTextureChildBehaviour : MonoBehaviour
    {
        private void Start()
        {
            phys = gameObject.GetComponent<PhysicalBehaviour>();
            renderer = gameObject.GetComponent<SpriteRenderer>();
            randomCarTextures = transform.root.gameObject.GetComponentInChildren<RandomCarTextureBehaviour>();
            if (randomCarTextures)
            {
                randomCarTextures.OnAfterChange.AddListener(new UnityAction(SyncSprite));
                randomCarTextures.OnAfterChange.Invoke();
            }
        }
        private void SyncSprite()
        {
            renderer.sprite = Sprites[randomCarTextures.chosenIndex];
            if (phys)
            {
                phys.RefreshOutline();
            }
        }

        private PhysicalBehaviour phys;
        private SpriteRenderer renderer;
        private RandomCarTextureBehaviour randomCarTextures;
        public Sprite[] Sprites = new Sprite[0];
    }
    public class BackDoorBehaviour : MonoBehaviour
    {
        private void Start()
        {
            renderer = GetComponent<SpriteRenderer>();
            randomCarTextures = transform.root.GetComponentInChildren<RandomCarTextureBehaviour>();
            randomCarTextures.OnAfterChange.AddListener(new UnityAction(UpdateActivation));
            randomCarTextures.OnAfterChange.Invoke();
            startLayerName = renderer.sortingLayerName;
            UpdateActivation();
        }
        private void Use()
        {
            Activated = !Activated;
            UpdateActivation();
        }
        private void Update()
        {
            if (HasSpareWheel && !SpareWheelJoint && !spareWheelJointBroken)
            {
                spareWheelJointBroken = true;
            }
        }
        private void UpdateActivation()
        {
            if (Activated)
            {
                renderer.sprite = Textures[randomCarTextures.chosenIndex].OpenSprite;
                if (HasSpareWheel && !spareWheelJointBroken)
                {
                    SpareWheelJoint.connectedBody.transform.parent = transform;
                    SpareWheelJoint.connectedBody.gameObject.SetActive(false);
                    renderer.sprite = Textures[randomCarTextures.chosenIndex].OpenSpriteWithWheel;
                }
                renderer.sortingLayerName = "Bottom";
                gameObject.layer = 10;
            }
            else
            {
                if (HasSpareWheel && !spareWheelJointBroken)
                {
                    SpareWheelJoint.connectedBody.transform.parent = transform.parent;
                    SpareWheelJoint.connectedBody.gameObject.SetActive(true);
                }
                renderer.sprite = Textures[randomCarTextures.chosenIndex].ClosedSprite;
                renderer.sortingLayerName = startLayerName;
                gameObject.layer = 9;
                OnClosed?.Invoke();
            }
            GetComponent<PhysicalBehaviour>().RefreshOutline();
        }
        private void OnBeforeSerialise()
        {
            if (HasSpareWheel && !spareWheelJointBroken && Activated)
            {
                SpareWheelJoint.connectedBody.transform.parent = transform.parent;
                foreach (ObjectSerialisationHelper objectSerialisationHelper in transform.parent.GetComponents<ObjectSerialisationHelper>())
                {
                    if (objectSerialisationHelper.Object == SpareWheelJoint.connectedBody.gameObject)
                    {
                        objectSerialisationHelper.transformData = new TransformPrototype(SpareWheelJoint.connectedBody.transform.localPosition, SpareWheelJoint.connectedBody.transform.eulerAngles.z, SpareWheelJoint.connectedBody.transform.localScale);
                        objectSerialisationHelper.objectName = Utils.GetHierachyPath(SpareWheelJoint.connectedBody.transform);
                    }
                }
                SpareWheelJoint.connectedBody.transform.parent = transform;
            }
        }

        public bool Activated;
        public bool HasSpareWheel = true;
        public bool spareWheelJointBroken;
        [SkipSerialisation]
        public FixedJoint2D SpareWheelJoint;
        private SpriteRenderer renderer;
        private RandomCarTextureBehaviour randomCarTextures;
        [SkipSerialisation]
        public DoorSprites[] Textures = new DoorSprites[0];
        public UnityEvent OnClosed = new UnityEvent();
        private string startLayerName;
        public struct DoorSprites
        {
            public Sprite OpenSprite;
            public Sprite OpenSpriteWithWheel;
            public Sprite ClosedSprite;
        }
    }
    public class InsideBehaviour : MonoBehaviour
    {
        private void Start()
        {
            GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("toggleInside", () => ShowInside ? "Hide inside" : "Show inside", "Toggles inside", () =>
            {
                ShowInside = !ShowInside;
                UpdateInside();
            }));
            UpdateInside();
        }
        public void UpdateInside()
        {
            if (ShowInside)
            {
                foreach (SpriteRenderer renderer in ForegroundRenderers)
                {
                    renderer.enabled = false;
                }
            }
            else
            {
                foreach (SpriteRenderer renderer in ForegroundRenderers)
                {
                    renderer.enabled = true;
                }
            }
        }

        public bool ShowInside;
        [SkipSerialisation]
        public SpriteRenderer[] ForegroundRenderers = new SpriteRenderer[0];
        [SkipSerialisation]
        public UnityEvent OnToggleInside = new UnityEvent();
    }
    [SkipSerialisation]
    public class ActOnJointBreak : MonoBehaviour
    {
        private void Update()
        {
            if (!broken && !Joint)
            {
                broken = true;
                OnBreak?.Invoke();
            }
        }
        private void OnJointBreak2D(Joint2D joint)
        {
            if (joint == Joint)
            {
                OnBreak?.Invoke();
                broken = true;
            }
        }

        private bool broken;
        public Joint2D Joint;
        public UnityEvent OnBreak = new UnityEvent();
    }
#endregion
}